@@ -31,7 +31,9 @@
 #include "python/tensorstore/data_type.h"
 #include "python/tensorstore/json_type_caster.h"
 #include "python/tensorstore/serialization.h"
+#include "python/tensorstore/tensorstore_module_components.h"
 #include "tensorstore/data_type.h"
+#include "tensorstore/internal/global_initializer.h"
 #include "tensorstore/util/executor.h"
 #include "tensorstore/util/quote_string.h"
 #include "tensorstore/util/str_cat.h"
@@ -51,8 +53,8 @@ pybind11::dtype GetNumpyDtype(int type_num) {
 DataType GetDataTypeOrThrow(std::string_view name) {
   auto d = GetDataType(name);
   if (!d.valid()) {
-    throw py::value_error(
-        StrCat("No TensorStore data type with name: ", QuoteString(name)));
+    throw py::value_error(tensorstore::StrCat(
+        "No TensorStore data type with name: ", QuoteString(name)));
   }
   return d;
 }
@@ -74,9 +76,9 @@ py::dtype GetNumpyDtypeOrThrow(DataType dtype) {
   if (type_num != -1) {
     return GetNumpyDtype(type_num);
   }
-  throw py::value_error(
-      StrCat("No NumPy dtype corresponding to TensorStore data type: ",
-             QuoteString(dtype.name())));
+  throw py::value_error(tensorstore::StrCat(
+      "No NumPy dtype corresponding to TensorStore data type: ",
+      QuoteString(dtype.name())));
 }
 
 DataType GetDataType(pybind11::dtype dt) {
@@ -95,9 +97,9 @@ DataType GetDataType(pybind11::dtype dt) {
 DataType GetDataTypeOrThrow(py::dtype dt) {
   auto dtype = GetDataType(dt);
   if (dtype.valid()) return dtype;
-  throw py::value_error(
-      StrCat("No TensorStore data type corresponding to NumPy dtype: ",
-             py::cast<std::string>(py::repr(dt))));
+  throw py::value_error(tensorstore::StrCat(
+      "No TensorStore data type corresponding to NumPy dtype: ",
+      py::cast<std::string>(py::repr(dt))));
 }
 
 py::object GetTypeObjectOrThrow(DataType dtype) {
@@ -151,7 +153,7 @@ Overload:
       "name", [](DataType self) { return std::string(self.name()); });
 
   cls.def("__repr__", [](DataType self) {
-    return StrCat("dtype(", QuoteString(self.name()), ")");
+    return tensorstore::StrCat("dtype(", QuoteString(self.name()), ")");
   });
 
   EnablePicklingFromSerialization(cls);
@@ -185,7 +187,6 @@ Overload:
       [](DataType self, DataTypeLike other) { return self == other.value; },
       py::arg("other"));
 }
-}  // namespace
 
 void RegisterDataTypeBindings(pybind11::module m, Executor defer) {
   if (!internal_python::RegisterNumpyBfloat16()) {
@@ -203,6 +204,10 @@ void RegisterDataTypeBindings(pybind11::module m, Executor defer) {
   }
 }
 
+TENSORSTORE_GLOBAL_INITIALIZER {
+  RegisterPythonComponent(RegisterDataTypeBindings, /*priority=*/-800);
+}
+}  // namespace
 }  // namespace internal_python
 }  // namespace tensorstore
 
