@@ -24,17 +24,19 @@
 #include <variant>
 #include <vector>
 
+#include "absl/base/optimization.h"
 #include "absl/strings/escaping.h"
 #include "python/tensorstore/dim_expression.h"
 #include "python/tensorstore/numpy_indexing_spec.h"
 #include "python/tensorstore/sequence_parameter.h"
 #include "python/tensorstore/subscript_method.h"
+#include "python/tensorstore/tensorstore_module_components.h"
 #include "tensorstore/index.h"
 #include "tensorstore/index_space/dim_expression.h"
 #include "tensorstore/index_space/dimension_identifier.h"
 #include "tensorstore/index_space/dimension_index_buffer.h"
 #include "tensorstore/index_space/index_transform.h"
-#include "tensorstore/util/assert_macros.h"
+#include "tensorstore/internal/global_initializer.h"
 #include "tensorstore/util/result.h"
 #include "tensorstore/util/span.h"
 #include "tensorstore/util/status.h"
@@ -48,17 +50,18 @@ namespace py = ::pybind11;
 void AppendDimensionSelectionRepr(std::string* out,
                                   span<const DynamicDimSpec> dims) {
   if (dims.empty()) {
-    StrAppend(out, "()");
+    tensorstore::StrAppend(out, "()");
   }
   for (size_t i = 0; i < dims.size(); ++i) {
     const auto& d = dims[i];
     if (auto* index = std::get_if<DimensionIndex>(&d)) {
-      StrAppend(out, (i == 0 ? "" : ","), *index);
+      tensorstore::StrAppend(out, (i == 0 ? "" : ","), *index);
     } else if (auto* label = std::get_if<std::string>(&d)) {
-      StrAppend(out, (i == 0 ? "" : ","), "'", absl::CHexEscape(*label), "'");
+      tensorstore::StrAppend(out, (i == 0 ? "" : ","), "'",
+                             absl::CHexEscape(*label), "'");
     } else {
       const auto& slice = std::get<DimRangeSpec>(d);
-      StrAppend(out, (i == 0 ? "" : ","), slice);
+      tensorstore::StrAppend(out, (i == 0 ? "" : ","), slice);
     }
   }
 }
@@ -66,7 +69,7 @@ void AppendDimensionSelectionRepr(std::string* out,
 std::string DimensionSelection::repr() const {
   std::string out = "d[";
   AppendDimensionSelectionRepr(&out, dims);
-  StrAppend(&out, "]");
+  tensorstore::StrAppend(&out, "]");
   return out;
 }
 
@@ -101,11 +104,11 @@ class PythonTranslateOp : public PythonDimExpression {
       case TranslateOpKind::kTranslateBackwardBy:
         return "backward_by";
     }
-    TENSORSTORE_UNREACHABLE;
+    ABSL_UNREACHABLE();  // COV_NF_LINE
   }
 
   std::string repr() const override {
-    return StrCat(
+    return tensorstore::StrCat(
         parent_->repr(), ".translate_", op_suffix(), "[",
         IndexVectorRepr(indices_, /*implicit=*/true, /*subscript=*/true), "]");
   }
@@ -134,7 +137,7 @@ class PythonStrideOp : public PythonDimExpression {
       : parent_(std::move(parent)), strides_(std::move(strides)) {}
 
   std::string repr() const override {
-    return StrCat(
+    return tensorstore::StrCat(
         parent_->repr(), ".stride[",
         IndexVectorRepr(strides_, /*implicit=*/true, /*subscript=*/true), "]");
   }
@@ -162,11 +165,12 @@ class PythonLabelOp : public PythonDimExpression {
       : parent_(std::move(parent)), labels_(std::move(labels).value) {}
 
   std::string repr() const override {
-    std::string r = StrCat(parent_->repr(), ".label[");
+    std::string r = tensorstore::StrCat(parent_->repr(), ".label[");
     for (size_t i = 0; i < labels_.size(); ++i) {
-      StrAppend(&r, i == 0 ? "" : ",", "'", absl::CHexEscape(labels_[i]), "'");
+      tensorstore::StrAppend(&r, i == 0 ? "" : ",", "'",
+                             absl::CHexEscape(labels_[i]), "'");
     }
-    StrAppend(&r, "]");
+    tensorstore::StrAppend(&r, "]");
     return r;
   }
 
@@ -192,7 +196,7 @@ class PythonDiagonalOp : public PythonDimExpression {
       : parent_(std::move(parent)) {}
 
   std::string repr() const override {
-    return StrCat(parent_->repr(), ".diagonal");
+    return tensorstore::StrCat(parent_->repr(), ".diagonal");
   }
 
   Result<IndexTransform<>> Apply(IndexTransform<> transform,
@@ -218,9 +222,9 @@ class PythonTransposeOp : public PythonDimExpression {
         target_dim_specs_(std::move(target_dim_specs).value) {}
 
   std::string repr() const override {
-    std::string out = StrCat(parent_->repr(), ".transpose[");
+    std::string out = tensorstore::StrCat(parent_->repr(), ".transpose[");
     AppendDimensionSelectionRepr(&out, target_dim_specs_);
-    StrAppend(&out, "]");
+    tensorstore::StrAppend(&out, "]");
     return out;
   }
 
@@ -248,8 +252,9 @@ class PythonIndexOp : public PythonDimExpression {
                          NumpyIndexingSpec spec)
       : parent_(std::move(parent)), spec_(std::move(spec)) {}
   std::string repr() const override {
-    return StrCat(parent_->repr(), GetIndexingModePrefix(spec_.mode), "[",
-                  IndexingSpecRepr(spec_), "]");
+    return tensorstore::StrCat(parent_->repr(),
+                               GetIndexingModePrefix(spec_.mode), "[",
+                               IndexingSpecRepr(spec_), "]");
   }
 
   Result<IndexTransform<>> Apply(IndexTransform<> transform,
@@ -279,8 +284,9 @@ class PythonInitialIndexOp : public PythonDimExpression {
       std::shared_ptr<const DimensionSelection> parent, NumpyIndexingSpec spec)
       : parent_(std::move(parent)), spec_(std::move(spec)) {}
   std::string repr() const override {
-    return StrCat(parent_->repr(), GetIndexingModePrefix(spec_.mode), "[",
-                  IndexingSpecRepr(spec_), "]");
+    return tensorstore::StrCat(parent_->repr(),
+                               GetIndexingModePrefix(spec_.mode), "[",
+                               IndexingSpecRepr(spec_), "]");
   }
 
   Result<IndexTransform<>> Apply(IndexTransform<> transform,
@@ -310,9 +316,9 @@ ClsDimExpression MakeDimExpressionClass(py::module m) {
   return ClsDimExpression(m, "DimExpression", R"(
 Specifies an advanced indexing operation.
 
-`Dimension expressions<python-dim-expressions>` permit indexing using
-`dimension labels<dimension-labels>`, and also support additional operations
-that cannot be performed with plain `python-numpy-style-indexing`.
+:ref:`Dimension expressions<python-dim-expressions>` permit indexing using
+:ref:`dimension labels<dimension-labels>`, and also support additional operations
+that cannot be performed with plain :ref:`python-numpy-style-indexing`.
 
 Group:
   Indexing
@@ -453,7 +459,7 @@ dimensions:
 -----------------------------------------------------------
 
 Specifying a value of :py:obj:`.newaxis` (equal to `None`) adds a new
-dummy/singleton dimension with `implicit bounds<implicit-bounds>`
+dummy/singleton dimension with :ref:`implicit bounds<implicit-bounds>`
 :math:`[0, 1)`:
 
    >>> transform = ts.IndexTransform(input_labels=['x', 'y'])
@@ -559,8 +565,8 @@ An indexing spec consisting solely of an `Ellipsis` term has no effect:
 :ref:`Integer array indexing<python-indexing-integer-array>`
 ------------------------------------------------------------
 
-Specifying an `array_like` *index array* of integer values selects the
-coordinates given by the elements of the array of the selected dimension:
+Specifying an `~numpy.typing.ArrayLike` *index array* of integer values selects
+the coordinates given by the elements of the array of the selected dimension:
 
     >>> x = ts.array([[1, 2, 3], [4, 5, 6]], dtype=ts.int32)
     >>> x = x[ts.d[:].label['x', 'y']]
@@ -577,10 +583,10 @@ coordinates given by the elements of the array of the selected dimension:
       },
     })
 
-As in the example above, if only a single
-index array term is specified, the dimensions of the index array are added to
-the result domain in place of the selected dimension, consistent with
-`direct NumPy-style indexing<python-indexing-integer-array>` in the default
+As in the example above, if only a single index array term is specified, the
+dimensions of the index array are added to the result domain in place of the
+selected dimension, consistent with
+:ref:`direct NumPy-style indexing<python-indexing-integer-array>` in the default
 index array mode.
 
 However, when using NumPy-style indexing with a dimension expression, if more
@@ -613,10 +619,9 @@ dimensions are not well-defined for dimension expressions:
 :ref:`Boolean array indexing<python-indexing-boolean-array>`
 ------------------------------------------------------------
 
-Specifying an `array_like` of `bool` values is equivalent to
-specifying a sequence of integer index arrays containing the
-coordinates of `True` values (in C order), e.g. as obtained from
-`numpy.nonzero`:
+Specifying an `~numpy.typing.ArrayLike` of `bool` values is equivalent to
+specifying a sequence of integer index arrays containing the coordinates of
+`True` values (in C order), e.g. as obtained from `numpy.nonzero`:
 
 Specifying a 1-d `bool` array is equivalent to a single index array of the
 non-zero coordinates:
@@ -1341,13 +1346,13 @@ Group:
 ClsDimensionSelection MakeDimensionSelectionClass(py::module m) {
   return ClsDimensionSelection(m, "d",
                                R"(
-Specifies a dimension selection, for starting a `dimension expression<python-dim-expressions>`.
+Specifies a dimension selection, for starting a :ref:`dimension expression<python-dim-expressions>`.
 
 A dimension selection specifies a sequence of dimensions, either by index or
-`label<dimension-labels>`.
+:ref:`label<dimension-labels>`.
 
 :ref:`python-dim-selections` may be used as part of a
-`dimension expression<python-dim-expression-construction>` to specify the
+:ref:`dimension expression<python-dim-expression-construction>` to specify the
 dimensions to which an indexing operation applies.
 
 Group:
@@ -1391,8 +1396,6 @@ Examples:
       py::arg("other"));
 }
 
-}  // namespace
-
 void RegisterDimExpressionBindings(pybind11::module m, Executor defer) {
   defer([cls = MakeDimExpressionClass(m)]() mutable {
     DefineDimExpressionAttributes(cls);
@@ -1405,6 +1408,12 @@ void RegisterDimExpressionBindings(pybind11::module m, Executor defer) {
   m.attr("newaxis") = py::none();
 }
 
+TENSORSTORE_GLOBAL_INITIALIZER {
+  RegisterPythonComponent(RegisterDimExpressionBindings, /*priority=*/-850);
+}
+
+}  // namespace
+
 bool CastToDimensionSelection(py::handle src, DimensionSelection& out) {
   if (PyUnicode_Check(src.ptr())) {
     out.dims.emplace_back(py::cast<std::string>(src));
