@@ -27,6 +27,8 @@
 #include "python/tensorstore/future.h"
 #include "python/tensorstore/gil_safe.h"
 #include "python/tensorstore/python_imports.h"
+#include "python/tensorstore/tensorstore_module_components.h"
+#include "tensorstore/internal/global_initializer.h"
 #include "tensorstore/util/executor.h"
 #include "tensorstore/util/future.h"
 
@@ -573,7 +575,8 @@ Handle for *consuming* the result of an asynchronous operation.
 
 This type supports several different patterns for consuming results:
 
-- Asynchronously with :py:mod:`asyncio`, using the `await<python:await>` keyword:
+- Asynchronously with :py:mod:`asyncio`, using the :ref:`await<python:await>`
+  keyword:
 
       >>> future = ts.open({
       ...     'driver': 'array',
@@ -620,7 +623,7 @@ This type supports several different patterns for consuming results:
       Callback: { [0, 3) }
 
 If an error occurs, instead of returning a value, :py:obj:`.result()` or
-`await<python:await>` will raise an exception.
+:ref:`await<python:await>` will raise an exception.
 
 This type supports a subset of the interfaces of
 :py:class:`python:concurrent.futures.Future` and
@@ -1015,10 +1018,6 @@ Group:
   Constructors
 )");
 }
-}  // namespace
-
-PyTypeObject* PythonFutureObject::python_type = nullptr;
-PyTypeObject* PythonPromiseObject::python_type = nullptr;
 
 void RegisterFutureBindings(pybind11::module m, Executor defer) {
   defer([cls = MakeFutureClass(m)]() mutable { DefineFutureAttributes(cls); });
@@ -1026,6 +1025,15 @@ void RegisterFutureBindings(pybind11::module m, Executor defer) {
       [cls = MakePromiseClass(m)]() mutable { DefinePromiseAttributes(cls); });
 }
 
+TENSORSTORE_GLOBAL_INITIALIZER {
+  RegisterPythonComponent(RegisterFutureBindings, /*priority=*/-450);
+}
+
+}  // namespace
+
+PyTypeObject* PythonFutureObject::python_type = nullptr;
+PyTypeObject* PythonPromiseObject::python_type = nullptr;
+
 py::object GetCurrentThreadAsyncioEventLoop() {
   if (auto loop =
           py::reinterpret_steal<py::object>(PyObject_CallFunctionObjArgs(
