@@ -29,7 +29,7 @@
 #include <utility>
 #include <vector>
 
-#include "absl/hash/hash.h"
+#include "absl/base/optimization.h"
 #include <nlohmann/json.hpp>
 #include "python/tensorstore/array_type_caster.h"
 #include "python/tensorstore/dim_expression.h"
@@ -42,8 +42,8 @@
 #include "python/tensorstore/sequence_parameter.h"
 #include "python/tensorstore/serialization.h"
 #include "python/tensorstore/status.h"
+#include "python/tensorstore/tensorstore_module_components.h"
 #include "tensorstore/array.h"
-#include "tensorstore/data_type.h"
 #include "tensorstore/index.h"
 #include "tensorstore/index_interval.h"
 #include "tensorstore/index_space/dimension_identifier.h"
@@ -53,11 +53,10 @@
 #include "tensorstore/index_space/json.h"
 #include "tensorstore/index_space/output_index_map.h"
 #include "tensorstore/index_space/output_index_method.h"
+#include "tensorstore/internal/global_initializer.h"
 #include "tensorstore/internal/json_binding/json_binding.h"
 #include "tensorstore/rank.h"
 #include "tensorstore/strided_layout.h"
-#include "tensorstore/util/assert_macros.h"
-#include "tensorstore/util/bit_span.h"
 #include "tensorstore/util/element_pointer.h"
 #include "tensorstore/util/quote_string.h"
 #include "tensorstore/util/span.h"
@@ -79,7 +78,7 @@ bool operator==(const OutputIndexMap& a, const OutputIndexMap& b) {
       return a.stride == b.stride && a.index_array == b.index_array &&
              a.index_range == b.index_range;
   }
-  TENSORSTORE_UNREACHABLE;  // COV_NF_LINE
+  ABSL_UNREACHABLE();  // COV_NF_LINE
 }
 
 HomogeneousTuple<Index> GetExclusiveMax(IndexDomainView<> domain) {
@@ -143,8 +142,9 @@ namespace {
 DimensionIndex NormalizePythonDimensionIndex(PythonDimensionIndex i,
                                              DimensionIndex size) {
   if (i.value < -size || i.value >= size) {
-    throw py::index_error(StrCat("Index ", i.value, " is outside valid range [",
-                                 -size, ", ", size, ")"));
+    throw py::index_error(tensorstore::StrCat("Index ", i.value,
+                                              " is outside valid range [",
+                                              -size, ", ", size, ")"));
   }
   if (i.value < 0) i.value += size;
   return i.value;
@@ -177,8 +177,8 @@ IndexTransformBuilder<> InitializeIndexTransformBuilder(
   const char* input_rank_field = nullptr;
   if (input_rank) {
     if (!IsValidRank(*input_rank)) {
-      throw py::value_error(
-          StrCat("Invalid ", input_rank_field_name, ": ", *input_rank));
+      throw py::value_error(tensorstore::StrCat(
+          "Invalid ", input_rank_field_name, ": ", *input_rank));
     }
     input_rank_field = input_rank_field_name;
   }
@@ -186,16 +186,17 @@ IndexTransformBuilder<> InitializeIndexTransformBuilder(
   const auto check_rank = [&](DimensionIndex rank, const char* field_name) {
     if (!input_rank) {
       if (!IsValidRank(rank)) {
-        throw py::value_error(StrCat("Rank specified by `", field_name, "` (",
-                                     rank, ") exceeds maximum rank of ",
-                                     kMaxRank));
+        throw py::value_error(
+            tensorstore::StrCat("Rank specified by `", field_name, "` (", rank,
+                                ") exceeds maximum rank of ", kMaxRank));
       }
       input_rank = rank;
       input_rank_field = field_name;
     } else if (*input_rank != rank) {
-      throw py::value_error(StrCat("Rank specified by `", field_name, "` (",
-                                   rank, ") does not match rank specified by `",
-                                   input_rank_field, "` (", *input_rank, ")"));
+      throw py::value_error(
+          tensorstore::StrCat("Rank specified by `", field_name, "` (", rank,
+                              ") does not match rank specified by `",
+                              input_rank_field, "` (", *input_rank, ")"));
     }
   };
   if (input_inclusive_min) {
@@ -208,8 +209,9 @@ IndexTransformBuilder<> InitializeIndexTransformBuilder(
   const auto check_upper_bound = [&](DimensionIndex rank,
                                      const char* field_name) {
     if (upper_bound_field) {
-      throw py::value_error(StrCat("Cannot specify both `", upper_bound_field,
-                                   "` and `", field_name, "`"));
+      throw py::value_error(tensorstore::StrCat("Cannot specify both `",
+                                                upper_bound_field, "` and `",
+                                                field_name, "`"));
     } else {
       upper_bound_field = field_name;
     }
@@ -233,7 +235,8 @@ IndexTransformBuilder<> InitializeIndexTransformBuilder(
     check_rank(input_labels->size(), input_labels_field_name);
   }
   if (!input_rank) {
-    throw py::value_error(StrCat("Must specify `", input_rank_field_name, "`"));
+    throw py::value_error(
+        tensorstore::StrCat("Must specify `", input_rank_field_name, "`"));
   }
   if (output_rank && !IsValidRank(*output_rank)) {
     throw py::value_error(
@@ -306,16 +309,18 @@ void SetOutputIndexMaps(
 std::string OutputIndexMapToString(const OutputIndexMap& m) {
   switch (m.method) {
     case OutputIndexMethod::constant:
-      return StrCat("OutputIndexMap(offset=", m.offset, ")");
+      return tensorstore::StrCat("OutputIndexMap(offset=", m.offset, ")");
     case OutputIndexMethod::single_input_dimension:
-      return StrCat("OutputIndexMap(offset=", m.offset, ", stride=", m.stride,
-                    ", input_dimension=", m.input_dimension, ")");
+      return tensorstore::StrCat("OutputIndexMap(offset=", m.offset,
+                                 ", stride=", m.stride,
+                                 ", input_dimension=", m.input_dimension, ")");
     case OutputIndexMethod::array:
-      return StrCat("OutputIndexMap(offset=", m.offset, ", stride=", m.stride,
-                    ", index_array=", m.index_array,
-                    ", index_range=", m.index_range, ")");
+      return tensorstore::StrCat("OutputIndexMap(offset=", m.offset,
+                                 ", stride=", m.stride,
+                                 ", index_array=", m.index_array,
+                                 ", index_range=", m.index_range, ")");
   }
-  TENSORSTORE_UNREACHABLE;  // COV_NF_LINE
+  ABSL_UNREACHABLE();  // COV_NF_LINE
 }
 
 using OutputIndexMapRangeContainer =
@@ -323,7 +328,7 @@ using OutputIndexMapRangeContainer =
 
 auto MakeIndexDomainClass(py::module m) {
   return py::class_<IndexDomain<>>(m, "IndexDomain", R"(
-`Domain<index-domain>` (including bounds and optional dimension labels) of an N-dimensional :ref:`index space<index-space>`.
+:ref:`Domain<index-domain>` (including bounds and optional dimension labels) of an N-dimensional :ref:`index space<index-space>`.
 
 Logically, an :py:class:`.IndexDomain` is the cartesian product of a sequence of `Dim` objects.
 
@@ -1074,7 +1079,7 @@ Group:
 )");
 
   cls.def(
-      "__repr__", [](const IndexDomain<>& d) { return StrCat(d); },
+      "__repr__", [](const IndexDomain<>& d) { return tensorstore::StrCat(d); },
       "Returns the string representation.");
 
   cls.def(
@@ -1654,10 +1659,10 @@ Group:
       "__call__",
       [](const IndexTransform<>& self, SequenceParameter<Index> indices) {
         if (static_cast<DimensionIndex>(indices.size()) != self.input_rank()) {
-          throw std::invalid_argument(
-              StrCat("input indices vector of length ", indices.size(),
-                     " cannot be used with index transform with input rank ",
-                     self.input_rank()));
+          throw std::invalid_argument(tensorstore::StrCat(
+              "input indices vector of length ", indices.size(),
+              " cannot be used with index transform with input rank ",
+              self.input_rank()));
         }
         Index output_indices[kMaxRank];
         ThrowStatusException(self.TransformIndices(
@@ -1688,7 +1693,8 @@ Group:
       py::arg("indices"));
 
   cls.def(
-      "__repr__", [](const IndexTransform<>& t) { return StrCat(t); },
+      "__repr__",
+      [](const IndexTransform<>& t) { return tensorstore::StrCat(t); },
       "Returns the string representation.");
 
   cls.def(
@@ -2021,9 +2027,9 @@ Constructs an unbounded interval ``(-inf, +inf)``.
 Args:
   label: :ref:`Dimension label<dimension-labels>`.
   implicit_lower: Indicates whether the lower bound is
-    `implicit<implicit-bounds>`.
+    :ref:`implicit<implicit-bounds>`.
   implicit_upper: Indicates whether the upper bound is
-    `implicit<implicit-bounds>`.
+    :ref:`implicit<implicit-bounds>`.
 
 Examples:
 
@@ -2070,9 +2076,9 @@ Args:
   label: :ref:`Dimension label<dimension-labels>`.
   inclusive_min: Inclusive lower bound.  Defaults to :python:`0`.
   implicit_lower: Indicates whether the lower bound is
-    `implicit<implicit-bounds>`.
+    :ref:`implicit<implicit-bounds>`.
   implicit_upper: Indicates whether the upper bound is
-    `implicit<implicit-bounds>`.  Defaults to :python:`False` if
+    :ref:`implicit<implicit-bounds>`.  Defaults to :python:`False` if
     :python:`size` is specified, otherwise :python:`True`.
 
 Examples:
@@ -2106,17 +2112,17 @@ Overload:
                 label.value_or(""));
           }),
           R"(
-Constructs a half-open interval ``[inclusive_min, exclusive_max)```.
+Constructs a half-open interval ``[inclusive_min, exclusive_max)``.
 
 Args:
   inclusive_min: Inclusive lower bound.
   exclusive_max: Exclusive upper bound.
   label: :ref:`Dimension label<dimension-labels>`.
   implicit_lower: Indicates whether the lower bound is
-    `implicit<implicit-bounds>`.  Defaults to :python:`False` if
+    :ref:`implicit<implicit-bounds>`.  Defaults to :python:`False` if
     ``inclusive_min`` is specified, otherwise :python:`True`.
   implicit_upper: Indicates whether the upper bound is
-    `implicit<implicit-bounds>`.  Defaults to :python:`False` if
+    :ref:`implicit<implicit-bounds>`.  Defaults to :python:`False` if
     ``exclusive_max`` is specified, otherwise :python:`True`.
 
 Examples:
@@ -2158,10 +2164,10 @@ Args:
   inclusive_max: Inclusive upper bound.
   label: :ref:`Dimension label<dimension-labels>`.
   implicit_lower: Indicates whether the lower bound is
-    `implicit<implicit-bounds>`.  Defaults to :python:`False` if
+    :ref:`implicit<implicit-bounds>`.  Defaults to :python:`False` if
     ``inclusive_min`` is specified, otherwise :python:`True`.
   implicit_upper: Indicates whether the upper bound is
-    `implicit<implicit-bounds>`.  Defaults to :python:`False` if
+    :ref:`implicit<implicit-bounds>`.  Defaults to :python:`False` if
     ``exclusive_max`` is specified, otherwise :python:`True`.
 
 Examples:
@@ -2530,7 +2536,8 @@ Group:
 )");
 
   cls.def(
-      "__str__", [](const IndexDomainDimension<>& x) { return StrCat(x); },
+      "__str__",
+      [](const IndexDomainDimension<>& x) { return tensorstore::StrCat(x); },
       R"(
 Returns the string representation of the interval.
 
@@ -2770,7 +2777,7 @@ Overload:
                 IndexDomainDimension<>(OptionallyImplicitIndexInterval(
                     self.index_range, false, false)));
         }
-        TENSORSTORE_UNREACHABLE;  // COV_NF_LINE
+        ABSL_UNREACHABLE();  // COV_NF_LINE
       },
       [](py::tuple t) {
         OutputIndexMap map;
@@ -2847,7 +2854,7 @@ void DefineOutputIndexMapsAttributes(
 auto MakeOutputIndexMethodClass(py::module m) {
   return py::enum_<OutputIndexMethod>(m, "OutputIndexMethod",
                                       R"(
-Indicates the `output index method<output-index-methods>` of an :py:class:`OutputIndexMap`.
+Indicates the :ref:`output index method<output-index-methods>` of an :py:class:`OutputIndexMap`.
 
 See also:
   - :py:obj:`IndexTransform.output`
@@ -2869,8 +2876,6 @@ void DefineOutputIndexMethodAttributes(py::enum_<OutputIndexMethod>& cls) {
   cls.value("array", OutputIndexMethod::array);
 }
 
-}  // namespace
-
 void RegisterIndexSpaceBindings(pybind11::module m, Executor defer) {
   m.attr("inf") = kInfIndex;
 
@@ -2897,5 +2902,10 @@ void RegisterIndexSpaceBindings(pybind11::module m, Executor defer) {
   });
 }
 
+TENSORSTORE_GLOBAL_INITIALIZER {
+  RegisterPythonComponent(RegisterIndexSpaceBindings, /*priority=*/-900);
+}
+
+}  // namespace
 }  // namespace internal_python
 }  // namespace tensorstore
