@@ -21,12 +21,15 @@
 #include "python/tensorstore/gil_safe.h"
 #include "python/tensorstore/result_type_caster.h"
 #include "python/tensorstore/serialization.h"
+#include "python/tensorstore/tensorstore_module_components.h"
 #include "riegeli/bytes/cord_writer.h"
 #include "riegeli/bytes/string_reader.h"
+#include "tensorstore/internal/global_initializer.h"
 #include "tensorstore/internal/no_destructor.h"
 #include "tensorstore/internal/unowned_to_shared.h"
 #include "tensorstore/serialization/serialization.h"
 #include "tensorstore/util/executor.h"
+#include "tensorstore/util/str_cat.h"
 
 // Include this Python header last since it defines some macros that are not
 // properly prefixed and we want to avoid problems with any other headers.
@@ -395,6 +398,26 @@ pybind11::object MakeGlobalPicklableFunction(pybind11::object module,
   return self;
 }
 
+void RegisterSerializationBindings(pybind11::module_ m, Executor defer) {
+  if (PyType_Ready(&DecodableObjectType) != 0) {
+    throw py::error_already_set();
+  }
+  if (PyType_Ready(&EncodableObjectType) != 0) {
+    throw py::error_already_set();
+  }
+  if (PyType_Ready(&GlobalPicklableFunctionObjectType) != 0) {
+    throw py::error_already_set();
+  }
+  m.attr("_Decodable") = py::reinterpret_borrow<py::object>(
+      reinterpret_cast<PyObject*>(&DecodableObjectType));
+  m.attr("_Encodable") = py::reinterpret_borrow<py::object>(
+      reinterpret_cast<PyObject*>(&EncodableObjectType));
+}
+
+TENSORSTORE_GLOBAL_INITIALIZER {
+  RegisterPythonComponent(RegisterSerializationBindings, /*priority=*/0);
+}
+
 }  // namespace
 
 void DefineUnpickleMethod(pybind11::handle cls, pybind11::object function) {
@@ -475,22 +498,6 @@ pybind11::object MakeReduceSingleArgumentReturnValue(pybind11::object callable,
   return reduce_val;
 }
 
-void RegisterSerializationBindings(pybind11::module_ m, Executor defer) {
-  if (PyType_Ready(&DecodableObjectType) != 0) {
-    throw py::error_already_set();
-  }
-  if (PyType_Ready(&EncodableObjectType) != 0) {
-    throw py::error_already_set();
-  }
-  if (PyType_Ready(&GlobalPicklableFunctionObjectType) != 0) {
-    throw py::error_already_set();
-  }
-  m.attr("_Decodable") = py::reinterpret_borrow<py::object>(
-      reinterpret_cast<PyObject*>(&DecodableObjectType));
-  m.attr("_Encodable") = py::reinterpret_borrow<py::object>(
-      reinterpret_cast<PyObject*>(&EncodableObjectType));
-}
-
 }  // namespace internal_python
 
 namespace serialization {
