@@ -18,6 +18,7 @@
 // Other headers must be included after pybind11 to ensure header-order
 // inclusion constraints are satisfied.
 
+#include <cstdint>
 #include <memory>
 #include <new>
 #include <optional>
@@ -30,6 +31,7 @@
 #include "python/tensorstore/future.h"
 #include "python/tensorstore/gil_safe.h"
 #include "python/tensorstore/homogeneous_tuple.h"
+#include "python/tensorstore/index.h"
 #include "python/tensorstore/index_space.h"
 #include "python/tensorstore/json_type_caster.h"
 #include "python/tensorstore/keyword_arguments.h"
@@ -37,6 +39,7 @@
 #include "python/tensorstore/result_type_caster.h"
 #include "python/tensorstore/serialization.h"
 #include "python/tensorstore/spec.h"
+#include "python/tensorstore/tensorstore_module_components.h"
 #include "python/tensorstore/tensorstore_class.h"
 #include "python/tensorstore/transaction.h"
 #include "python/tensorstore/write_futures.h"
@@ -47,6 +50,7 @@
 #include "tensorstore/data_type.h"
 #include "tensorstore/driver/array/array.h"
 #include "tensorstore/index_space/index_transform.h"
+#include "tensorstore/internal/global_initializer.h"
 #include "tensorstore/internal/json/pprint_python.h"
 #include "tensorstore/open.h"
 #include "tensorstore/open_mode.h"
@@ -602,7 +606,7 @@ Args:
   source: Source array, :ref:`broadcast-compatible<index-domain-alignment>` with
     :python:`self.domain` and with a data type convertible to
     :python:`self.dtype`.  May be an existing :py:obj:`TensorStore` or any
-    `array_like`, including a scalar.
+    :py:obj:`~numpy.typing.ArrayLike`, including a scalar.
 
 Returns:
 
@@ -747,6 +751,104 @@ writes to be read:
 )",
       py::arg("source"));
 
+  cls.def(
+      "resize",
+      [](Self& self,
+         std::optional<SequenceParameter<OptionallyImplicitIndex>>
+             inclusive_min,
+         std::optional<SequenceParameter<OptionallyImplicitIndex>>
+             exclusive_max,
+         bool resize_metadata_only, bool resize_tied_bounds, bool expand_only,
+         bool shrink_only) {
+        if (!inclusive_min) {
+          inclusive_min =
+              std::vector<OptionallyImplicitIndex>(self.value.rank());
+        }
+        if (!exclusive_max) {
+          exclusive_max =
+              std::vector<OptionallyImplicitIndex>(self.value.rank());
+        }
+        tensorstore::ResizeOptions options = {};
+        if (resize_metadata_only) {
+          options.mode = options.mode | tensorstore::resize_metadata_only;
+        }
+        if (resize_tied_bounds) {
+          options.mode = options.mode | tensorstore::resize_tied_bounds;
+        }
+        if (expand_only) {
+          options.mode = options.mode | tensorstore::expand_only;
+        }
+        if (shrink_only) {
+          options.mode = options.mode | tensorstore::shrink_only;
+        }
+        return PythonFutureWrapper<TensorStore<>>(
+            tensorstore::Resize(
+                self.value,
+                std::vector<Index>(inclusive_min.value().begin(),
+                                   inclusive_min.value().end()),
+                std::vector<Index>(exclusive_max.value().begin(),
+                                   exclusive_max.value().end()),
+                options),
+            self.reference_manager());
+      },
+      R"(
+Resizes the current domain, persistently modifying the stored representation.
+
+Depending on the :py:param`resize_metadata_only`, if the bounds are shrunk,
+existing elements outside of the new bounds may be deleted. If the bounds are
+expanded, elements outside the existing bounds will initially contain either the
+fill value, or existing out-of-bounds data remaining after a prior resize
+operation.
+
+Example:
+
+    >>> dataset = await ts.open(
+    ...     {
+    ...         'driver': 'zarr',
+    ...         'kvstore': {
+    ...             'driver': 'memory'
+    ...         }
+    ...     },
+    ...     dtype=ts.uint32,
+    ...     shape=[3, 3],
+    ...     create=True)
+    >>> await dataset.write(np.arange(9, dtype=np.uint32).reshape((3, 3)))
+    >>> dataset = await dataset.resize(exclusive_max=(3, 2))
+    >>> await dataset.read()
+    array([[0, 1],
+           [3, 4],
+           [6, 7]], dtype=uint32)
+
+Args:
+
+  inclusive_min: Sequence of length :python:`self.rank()` specifying the new
+    inclusive min bounds.  A bound of :python:`None` indicates no change.
+  exclusive_max: Sequence of length :python:`self.rank()` specifying the new
+    exclusive max bounds.  A bound of :python:`None` indicates no change.
+  resize_metadata_only: Requests that, if applicable, the resize operation
+    affect only the metadata but not delete data chunks that are outside of the
+    new bounds.
+  resize_tied_bounds: Requests that the resize be permitted even if other
+    bounds tied to the specified bounds must also be resized.  This option
+    should be used with caution.
+  expand_only: Fail if any bounds would be reduced.
+  shrink_only: Fail if any bounds would be increased.
+
+Returns:
+
+  Future that resolves to a copy of :python:`self` with the updated bounds, once
+  the resize operation completes.
+
+Group:
+  I/O
+
+)",
+      py::arg("inclusive_min") = std::nullopt,
+      py::arg("exclusive_max") = std::nullopt,
+      py::arg("resize_metadata_only") = false,
+      py::arg("resize_tied_bounds") = false, py::arg("expand_only") = false,
+      py::arg("shrink_only") = false);
+
   cls.def(
       "__array__",
       [](Self& self, std::optional<py::dtype> dtype,
@@ -1223,7 +1325,7 @@ Args:
   source: Source array, :ref:`broadcast-compatible<index-domain-alignment>` with
     :python:`self[indices].domain` and with a data type convertible to
     :python:`self.dtype`.  May be an existing :py:obj:`TensorStore` or any
-    `array_like`, including a scalar.
+    :py:obj:`~numpy.typing.ArrayLike`, including a scalar.
 
 Transactional writes are also supported:
 
@@ -1354,7 +1456,7 @@ Args:
   source: Source array, :ref:`broadcast-compatible<index-domain-alignment>` with
     :python:`self.oindex[indices].domain` and with a data type convertible to
     :python:`self.dtype`.  May be an existing :py:obj:`TensorStore` or any
-    `array_like`, including a scalar.
+    :py:obj:`~numpy.typing.ArrayLike`, including a scalar.
 
 .. warning::
 
@@ -1478,7 +1580,7 @@ Args:
   source: Source array, :ref:`broadcast-compatible<index-domain-alignment>` with
     :python:`self.vindex[indices].domain` and with a data type convertible to
     :python:`self.dtype`.  May be an existing :py:obj:`TensorStore` or any
-    `array_like`, including a scalar.
+    :py:obj:`~numpy.typing.ArrayLike`, including a scalar.
 
 .. warning::
 
@@ -1641,7 +1743,7 @@ Args:
   source: Source array, :ref:`broadcast-compatible<index-domain-alignment>` with
     :python:`self.domain[transform]` and with a data type convertible to
     :python:`self.dtype`.  May be an existing :py:obj:`TensorStore` or any
-    `array_like`, including a scalar.
+    :py:obj:`~numpy.typing.ArrayLike`, including a scalar.
 
 .. warning::
 
@@ -1774,7 +1876,7 @@ Args:
   source: Source array, :ref:`broadcast-compatible<index-domain-alignment>` with
     :python:`self.domain[transform]` and with a data type convertible to
     :python:`self.dtype`.  May be an existing :py:obj:`TensorStore` or any
-    `array_like`, including a scalar.
+    :py:obj:`~numpy.typing.ArrayLike`, including a scalar.
 
 .. warning::
 
@@ -2421,8 +2523,6 @@ Group:
   });
 }
 
-}  // namespace
-
 void RegisterTensorStoreBindings(pybind11::module m, Executor defer) {
   defer([cls = MakeTensorStoreClass(m), m]() mutable {
     DefineTensorStoreAttributes(cls);
@@ -2430,5 +2530,10 @@ void RegisterTensorStoreBindings(pybind11::module m, Executor defer) {
   });
 }
 
+TENSORSTORE_GLOBAL_INITIALIZER {
+  RegisterPythonComponent(RegisterTensorStoreBindings, /*priority=*/-1000);
+}
+
+}  // namespace
 }  // namespace internal_python
 }  // namespace tensorstore
