@@ -17,6 +17,7 @@
 #include <algorithm>
 #include <limits>
 
+#include "absl/log/absl_check.h"
 #include "absl/status/status.h"
 #include "riegeli/varint/varint_reading.h"
 #include "riegeli/varint/varint_writing.h"
@@ -48,7 +49,7 @@ bool CompareArraysEqual(
 void CopyArrayImplementation(
     const ArrayView<const void, dynamic_rank, offset_origin>& source,
     const ArrayView<void, dynamic_rank, offset_origin>& dest) {
-  TENSORSTORE_CHECK(source.dtype() == dest.dtype());
+  ABSL_CHECK_EQ(source.dtype(), dest.dtype());
   internal::IterateOverArrays({&source.dtype()->copy_assign, nullptr},
                               /*status=*/nullptr,
                               /*constraints=*/skip_repeated_elements, source,
@@ -117,13 +118,14 @@ void PrintArrayDimension(
 }
 
 std::string DescribeForCast(DataType dtype, DimensionIndex rank) {
-  return StrCat("array with ", StaticCastTraits<DataType>::Describe(dtype),
-                " and ", StaticCastTraits<DimensionIndex>::Describe(rank));
+  return tensorstore::StrCat(
+      "array with ", StaticCastTraits<DataType>::Describe(dtype), " and ",
+      StaticCastTraits<DimensionIndex>::Describe(rank));
 }
 
 absl::Status ArrayOriginCastError(span<const Index> shape) {
-  return absl::InvalidArgumentError(StrCat("Cannot translate array with shape ",
-                                           shape, " to have zero origin."));
+  return absl::InvalidArgumentError(tensorstore::StrCat(
+      "Cannot translate array with shape ", shape, " to have zero origin."));
 }
 
 }  // namespace internal_array
@@ -184,7 +186,7 @@ void AppendToString(
   const span<const Index> origin = array.origin();
   if (std::any_of(origin.begin(), origin.end(),
                   [](Index x) { return x != 0; })) {
-    StrAppend(result, " @ ", origin);
+    tensorstore::StrAppend(result, " @ ", origin);
   }
 }
 
