@@ -27,6 +27,7 @@
 #include <limits>
 #include <type_traits>
 
+#include "absl/log/absl_check.h"
 #include "absl/status/status.h"
 #include "tensorstore/box.h"
 #include "tensorstore/data_type.h"
@@ -34,7 +35,6 @@
 #include "tensorstore/rank.h"
 #include "tensorstore/serialization/fwd.h"
 #include "tensorstore/strided_layout.h"
-#include "tensorstore/util/assert_macros.h"
 #include "tensorstore/util/element_pointer.h"
 #include "tensorstore/util/element_traits.h"
 #include "tensorstore/util/garbage_collection/fwd.h"
@@ -1619,7 +1619,7 @@ ArrayIterateResult IterateOverArrays(
     ElementwiseClosure<sizeof...(Array), absl::Status*> closure,
     absl::Status* status, IterationConstraints constraints,
     const Array&... array) {
-  TENSORSTORE_CHECK(ArraysHaveSameShapes(array...));
+  ABSL_CHECK(ArraysHaveSameShapes(array...));
   const std::array<std::ptrdiff_t, sizeof...(Array)> element_sizes{
       {array.dtype().size()...}};
   return IterateOverStridedLayouts(
