@@ -16,49 +16,53 @@
 
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
+#include "tensorstore/box.h"
 #include "tensorstore/rank.h"
+#include "tensorstore/serialization/serialization.h"
+#include "tensorstore/serialization/test_util.h"
 #include "tensorstore/util/status.h"
 #include "tensorstore/util/status_testutil.h"
-#include "tensorstore/util/to_string.h"
+#include "tensorstore/util/str_cat.h"
 
 namespace {
-using tensorstore::Box;
-using tensorstore::BoxView;
-using tensorstore::dynamic_rank;
-using tensorstore::HasBoxDomain;
-using tensorstore::Index;
-using tensorstore::IndexInterval;
-using tensorstore::IsCastConstructible;
-using tensorstore::kInfIndex;
-using tensorstore::kInfSize;
-using tensorstore::MatchesStatus;
-using tensorstore::MutableBoxView;
-using tensorstore::span;
-using tensorstore::StaticRankCast;
-using tensorstore::unchecked;
+
+using ::tensorstore::Box;
+using ::tensorstore::BoxView;
+using ::tensorstore::dynamic_rank;
+using ::tensorstore::HasBoxDomain;
+using ::tensorstore::Index;
+using ::tensorstore::IndexInterval;
+using ::tensorstore::IsStaticCastConstructible;
+using ::tensorstore::kInfIndex;
+using ::tensorstore::kInfSize;
+using ::tensorstore::MatchesStatus;
+using ::tensorstore::MutableBoxView;
+using ::tensorstore::span;
+using ::tensorstore::StaticRankCast;
+using ::tensorstore::unchecked;
+using ::tensorstore::serialization::TestSerializationRoundTrip;
 using ::testing::ElementsAre;
 using ::testing::ElementsAreArray;
 
-static_assert(std::is_convertible<BoxView<3>, BoxView<>>::value, "");
-static_assert(!std::is_constructible<BoxView<3>, BoxView<>>::value, "");
-static_assert(!std::is_assignable<BoxView<3>, BoxView<>>::value, "");
-static_assert(!std::is_assignable<Box<3>, Box<>>::value, "");
-static_assert(!std::is_constructible<Box<3>, Box<>>::value, "");
-static_assert(!std::is_constructible<BoxView<3>, Box<>>::value, "");
-static_assert(
-    !std::is_constructible<MutableBoxView<3>, MutableBoxView<>>::value, "");
-static_assert(!std::is_constructible<MutableBoxView<3>, Box<>>::value, "");
-static_assert(std::is_constructible<MutableBoxView<3>, Box<3>&>::value, "");
+static_assert(std::is_convertible_v<BoxView<3>, BoxView<>>);
+static_assert(!std::is_constructible_v<BoxView<3>, BoxView<>>);
+static_assert(!std::is_assignable_v<BoxView<3>, BoxView<>>);
+static_assert(!std::is_assignable_v<Box<3>, Box<>>);
+static_assert(!std::is_constructible_v<Box<3>, Box<>>);
+static_assert(!std::is_constructible_v<BoxView<3>, Box<>>);
+static_assert(!std::is_constructible_v<MutableBoxView<3>, MutableBoxView<>>);
+static_assert(!std::is_constructible_v<MutableBoxView<3>, Box<>>);
+static_assert(std::is_constructible_v<MutableBoxView<3>, Box<3>&>);
 
-static_assert(IsCastConstructible<BoxView<3>, BoxView<>>::value, "");
-static_assert(IsCastConstructible<Box<3>, BoxView<>>::value, "");
-static_assert(IsCastConstructible<Box<3>, Box<>>::value, "");
-static_assert(IsCastConstructible<BoxView<>, BoxView<3>>::value, "");
-static_assert(IsCastConstructible<MutableBoxView<3>, Box<3>&>::value, "");
-static_assert(!IsCastConstructible<MutableBoxView<>, const Box<3>&>::value, "");
-static_assert(!IsCastConstructible<BoxView<2>, BoxView<3>>::value, "");
-static_assert(!IsCastConstructible<BoxView<2>, Box<3>>::value, "");
-static_assert(!IsCastConstructible<Box<3>, Box<2>>::value, "");
+static_assert(IsStaticCastConstructible<BoxView<3>, BoxView<>>);
+static_assert(IsStaticCastConstructible<Box<3>, BoxView<>>);
+static_assert(IsStaticCastConstructible<Box<3>, Box<>>);
+static_assert(IsStaticCastConstructible<BoxView<>, BoxView<3>>);
+static_assert(IsStaticCastConstructible<MutableBoxView<3>, Box<3>&>);
+static_assert(!IsStaticCastConstructible<MutableBoxView<>, const Box<3>&>);
+static_assert(!IsStaticCastConstructible<BoxView<2>, BoxView<3>>);
+static_assert(!IsStaticCastConstructible<BoxView<2>, Box<3>>);
+static_assert(!IsStaticCastConstructible<Box<3>, Box<2>>);
 
 TEST(BoxTest, DefaultConstructDynamic) {
   Box<> box;
@@ -85,6 +89,13 @@ TEST(BoxTest, SizeConstruct) {
   EXPECT_THAT(box.shape(), ElementsAre(kInfSize, kInfSize, kInfSize));
 }
 
+TEST(BoxTest, ShapeArrayConstruct) {
+  std::array<Index, 3> shape{{1, 2, 3}};
+  Box<> box(shape);
+  EXPECT_THAT(box.origin(), ElementsAre(0, 0, 0));
+  EXPECT_THAT(box.shape(), ElementsAre(1, 2, 3));
+}
+
 TEST(BoxTest, DynamicRankSpanConstruct) {
   const Index origin[] = {1, 2, 3};
   const Index shape[] = {3, 4, 5};
@@ -113,7 +124,7 @@ TEST(BoxTest, ConstructFromBoxView) {
 TEST(BoxTest, DeduceFromShapeArray) {
   const Index shape[] = {3, 4, 5};
   auto box = Box(shape);
-  static_assert(std::is_same<decltype(box), Box<3>>::value, "");
+  static_assert(std::is_same_v<decltype(box), Box<3>>);
   EXPECT_THAT(box.origin(), ElementsAre(0, 0, 0));
   EXPECT_THAT(box.shape(), ElementsAre(3, 4, 5));
 }
@@ -121,7 +132,7 @@ TEST(BoxTest, DeduceFromShapeArray) {
 TEST(BoxTest, DeduceFromShapeSpanStatic) {
   const Index shape[] = {3, 4, 5};
   auto box = Box(span(shape));
-  static_assert(std::is_same<decltype(box), Box<3>>::value, "");
+  static_assert(std::is_same_v<decltype(box), Box<3>>);
   EXPECT_THAT(box.origin(), ElementsAre(0, 0, 0));
   EXPECT_THAT(box.shape(), ElementsAre(3, 4, 5));
 }
@@ -129,7 +140,7 @@ TEST(BoxTest, DeduceFromShapeSpanStatic) {
 TEST(BoxTest, DeduceFromShapeSpanDynamic) {
   const Index shape[] = {3, 4, 5};
   auto box = Box(span<const Index>(shape));
-  static_assert(std::is_same<decltype(box), Box<>>::value, "");
+  static_assert(std::is_same_v<decltype(box), Box<>>);
   EXPECT_THAT(box.origin(), ElementsAre(0, 0, 0));
   EXPECT_THAT(box.shape(), ElementsAre(3, 4, 5));
 }
@@ -138,7 +149,7 @@ TEST(BoxTest, DeduceFromOriginAndShapeArrays) {
   const Index origin[] = {1, 2, 3};
   const Index shape[] = {3, 4, 5};
   auto box = Box(origin, shape);
-  static_assert(std::is_same<decltype(box), Box<3>>::value, "");
+  static_assert(std::is_same_v<decltype(box), Box<3>>);
   EXPECT_THAT(box.origin(), ElementsAre(1, 2, 3));
   EXPECT_THAT(box.shape(), ElementsAre(3, 4, 5));
 }
@@ -147,7 +158,7 @@ TEST(BoxTest, DeduceFromOriginAndShapeSpansStatic) {
   const Index origin[] = {1, 2, 3};
   const Index shape[] = {3, 4, 5};
   auto box = Box(span(origin), span(shape));
-  static_assert(std::is_same<decltype(box), Box<3>>::value, "");
+  static_assert(std::is_same_v<decltype(box), Box<3>>);
   EXPECT_THAT(box.origin(), ElementsAre(1, 2, 3));
   EXPECT_THAT(box.shape(), ElementsAre(3, 4, 5));
 }
@@ -156,7 +167,7 @@ TEST(BoxTest, DeduceFromOriginAndShapeDynamic) {
   const Index origin[] = {1, 2, 3};
   const Index shape[] = {3, 4, 5};
   auto box = Box(span<const Index>(origin), span<const Index>(shape));
-  static_assert(std::is_same<decltype(box), Box<>>::value, "");
+  static_assert(std::is_same_v<decltype(box), Box<>>);
   EXPECT_THAT(box.origin(), ElementsAre(1, 2, 3));
   EXPECT_THAT(box.shape(), ElementsAre(3, 4, 5));
 }
@@ -166,7 +177,7 @@ TEST(BoxTest, DeduceFromBoxView) {
   const Index shape[] = {3, 4, 5};
   BoxView<3> box(origin, shape);
   auto box2 = Box(box);
-  static_assert(std::is_same<decltype(box2), Box<3>>::value, "");
+  static_assert(std::is_same_v<decltype(box2), Box<3>>);
   EXPECT_THAT(box2.shape(), ElementsAreArray(shape));
   EXPECT_THAT(box2.origin(), ElementsAreArray(origin));
 }
@@ -176,7 +187,7 @@ TEST(BoxTest, DeduceFromBox) {
   const Index shape[] = {3, 4, 5};
   Box<3> box(origin, shape);
   auto box2 = Box(box);
-  static_assert(std::is_same<decltype(box2), Box<3>>::value, "");
+  static_assert(std::is_same_v<decltype(box2), Box<3>>);
   EXPECT_THAT(box2.shape(), ElementsAreArray(shape));
   EXPECT_THAT(box2.origin(), ElementsAreArray(origin));
 }
@@ -307,7 +318,7 @@ TEST(BoxViewTest, DynamicRankSpanConstruct) {
 TEST(BoxViewTest, DeduceFromShapeArray) {
   const Index shape[] = {3, 4, 5};
   auto box = BoxView(shape);
-  static_assert(std::is_same<decltype(box), BoxView<3>>::value, "");
+  static_assert(std::is_same_v<decltype(box), BoxView<3>>);
   EXPECT_THAT(box.origin(), ElementsAre(0, 0, 0));
   EXPECT_THAT(box.shape(), ElementsAre(3, 4, 5));
 }
@@ -315,7 +326,7 @@ TEST(BoxViewTest, DeduceFromShapeArray) {
 TEST(BoxViewTest, DeduceFromShapeSpanStatic) {
   const Index shape[] = {3, 4, 5};
   auto box = BoxView(span(shape));
-  static_assert(std::is_same<decltype(box), BoxView<3>>::value, "");
+  static_assert(std::is_same_v<decltype(box), BoxView<3>>);
   EXPECT_THAT(box.origin(), ElementsAre(0, 0, 0));
   EXPECT_THAT(box.shape(), ElementsAre(3, 4, 5));
 }
@@ -323,7 +334,7 @@ TEST(BoxViewTest, DeduceFromShapeSpanStatic) {
 TEST(BoxViewTest, DeduceFromShapeSpanDynamic) {
   const Index shape[] = {3, 4, 5};
   auto box = BoxView(span<const Index>(shape));
-  static_assert(std::is_same<decltype(box), BoxView<>>::value, "");
+  static_assert(std::is_same_v<decltype(box), BoxView<>>);
   EXPECT_THAT(box.origin(), ElementsAre(0, 0, 0));
   EXPECT_THAT(box.shape(), ElementsAre(3, 4, 5));
 }
@@ -332,7 +343,7 @@ TEST(BoxViewTest, DeduceFromOriginAndShapeArrays) {
   const Index origin[] = {1, 2, 3};
   const Index shape[] = {3, 4, 5};
   auto box = BoxView(origin, shape);
-  static_assert(std::is_same<decltype(box), BoxView<3>>::value, "");
+  static_assert(std::is_same_v<decltype(box), BoxView<3>>);
   EXPECT_THAT(box.origin(), ElementsAre(1, 2, 3));
   EXPECT_THAT(box.shape(), ElementsAre(3, 4, 5));
 }
@@ -341,7 +352,7 @@ TEST(BoxViewTest, DeduceFromOriginAndShapeSpansStatic) {
   const Index origin[] = {1, 2, 3};
   const Index shape[] = {3, 4, 5};
   auto box = BoxView(span(origin), span(shape));
-  static_assert(std::is_same<decltype(box), BoxView<3>>::value, "");
+  static_assert(std::is_same_v<decltype(box), BoxView<3>>);
   EXPECT_THAT(box.origin(), ElementsAre(1, 2, 3));
   EXPECT_THAT(box.shape(), ElementsAre(3, 4, 5));
 }
@@ -350,7 +361,7 @@ TEST(BoxViewTest, DeduceFromOriginAndShapeDynamic) {
   const Index origin[] = {1, 2, 3};
   const Index shape[] = {3, 4, 5};
   auto box = BoxView(span<const Index>(origin), span<const Index>(shape));
-  static_assert(std::is_same<decltype(box), BoxView<>>::value, "");
+  static_assert(std::is_same_v<decltype(box), BoxView<>>);
   EXPECT_THAT(box.origin(), ElementsAre(1, 2, 3));
   EXPECT_THAT(box.shape(), ElementsAre(3, 4, 5));
 }
@@ -360,7 +371,7 @@ TEST(BoxViewTest, DeduceFromBoxView) {
   const Index shape[] = {3, 4, 5};
   BoxView<3> box(origin, shape);
   auto box2 = BoxView(box);
-  static_assert(std::is_same<decltype(box2), BoxView<3>>::value, "");
+  static_assert(std::is_same_v<decltype(box2), BoxView<3>>);
   EXPECT_THAT(box2.shape(), ElementsAreArray(shape));
   EXPECT_THAT(box2.origin(), ElementsAreArray(origin));
 }
@@ -370,7 +381,7 @@ TEST(BoxViewTest, DeduceFromBox) {
   const Index shape[] = {3, 4, 5};
   const Box<3> box(origin, shape);
   auto box2 = BoxView(box);
-  static_assert(std::is_same<decltype(box2), BoxView<3>>::value, "");
+  static_assert(std::is_same_v<decltype(box2), BoxView<3>>);
   EXPECT_THAT(box2.shape(), ElementsAreArray(shape));
   EXPECT_THAT(box2.origin(), ElementsAreArray(origin));
 }
@@ -432,7 +443,7 @@ TEST(BoxViewTest, StaticRankCast) {
       StaticRankCast<2>(box),
       MatchesStatus(absl::StatusCode::kInvalidArgument,
                     "Cannot cast box with rank of 3 to box with rank of 2"));
-  static_assert(std::is_same<decltype(box2), BoxView<3>>::value, "");
+  static_assert(std::is_same_v<decltype(box2), BoxView<3>>);
   EXPECT_THAT(box2.shape(), ElementsAreArray(shape));
   EXPECT_THAT(box2.origin(), ElementsAreArray(origin));
 }
@@ -475,7 +486,7 @@ TEST(MutableBoxViewTest, DeduceFromOriginAndShapeArrays) {
   Index origin[] = {1, 2, 3};
   Index shape[] = {3, 4, 5};
   auto box = BoxView(origin, shape);
-  static_assert(std::is_same<decltype(box), MutableBoxView<3>>::value, "");
+  static_assert(std::is_same_v<decltype(box), MutableBoxView<3>>);
   EXPECT_EQ(3, box.rank());
   EXPECT_EQ(box.origin().data(), origin);
   EXPECT_EQ(box.shape().data(), shape);
@@ -485,7 +496,7 @@ TEST(MutableBoxViewTest, DeduceFromOriginAndShapeSpansStatic) {
   Index origin[] = {1, 2, 3};
   Index shape[] = {3, 4, 5};
   auto box = BoxView(span(origin), span(shape));
-  static_assert(std::is_same<decltype(box), MutableBoxView<3>>::value, "");
+  static_assert(std::is_same_v<decltype(box), MutableBoxView<3>>);
   EXPECT_EQ(3, box.rank());
   EXPECT_EQ(box.origin().data(), origin);
   EXPECT_EQ(box.shape().data(), shape);
@@ -495,7 +506,7 @@ TEST(MutableBoxViewTest, DeduceFromOriginAndShapeDynamic) {
   Index origin[] = {1, 2, 3};
   Index shape[] = {3, 4, 5};
   auto box = BoxView(span<Index>(origin), span<Index>(shape));
-  static_assert(std::is_same<decltype(box), MutableBoxView<>>::value, "");
+  static_assert(std::is_same_v<decltype(box), MutableBoxView<>>);
   EXPECT_EQ(3, box.rank());
   EXPECT_EQ(box.origin().data(), origin);
   EXPECT_EQ(box.shape().data(), shape);
@@ -506,7 +517,7 @@ TEST(MutableBoxViewTest, DeduceFromBox) {
   const Index shape[] = {3, 4, 5};
   Box<3> box(origin, shape);
   auto box2 = BoxView(box);
-  static_assert(std::is_same<decltype(box2), MutableBoxView<3>>::value, "");
+  static_assert(std::is_same_v<decltype(box2), MutableBoxView<3>>);
   EXPECT_EQ(box2.shape().data(), box.shape().data());
   EXPECT_EQ(box2.origin().data(), box.origin().data());
 }
@@ -516,7 +527,7 @@ TEST(MutableBoxViewTest, DeduceFromMutableBoxView) {
   Index shape[] = {3, 4, 5};
   MutableBoxView<3> box(origin, shape);
   auto box2 = BoxView(box);
-  static_assert(std::is_same<decltype(box2), MutableBoxView<3>>::value, "");
+  static_assert(std::is_same_v<decltype(box2), MutableBoxView<3>>);
   EXPECT_EQ(box2.shape().data(), box.shape().data());
   EXPECT_EQ(box2.origin().data(), box.origin().data());
 }
@@ -583,7 +594,7 @@ TEST(MutableBoxViewTest, StaticRankCast) {
   Index shape[] = {3, 4, 5};
   MutableBoxView<> box(origin, shape);
   auto box2 = StaticRankCast<3, unchecked>(box);
-  static_assert(std::is_same<decltype(box2), MutableBoxView<3>>::value, "");
+  static_assert(std::is_same_v<decltype(box2), MutableBoxView<3>>);
   EXPECT_THAT(box2.shape(), ElementsAreArray(shape));
   EXPECT_THAT(box2.origin(), ElementsAreArray(origin));
 }
@@ -656,7 +667,7 @@ TEST(BoxTest, Contains) {
   const Index indices1[] = {2, 3};
   const Index indices2[] = {0, 3};
   const Index indices3[] = {0};
-  const Index indices4[] = {2};
+  Index indices4[] = {2};
 
   auto span1 = span(indices1);
   auto span2 = span(indices2);
@@ -716,10 +727,10 @@ TEST(BoxTest, Contains) {
 }
 
 TEST(BoxTest, GetBoxDomainOf) {
-  static_assert(!HasBoxDomain<int>::value, "");
-  static_assert(HasBoxDomain<BoxView<>>::value, "");
-  static_assert(HasBoxDomain<Box<>>::value, "");
-  static_assert(HasBoxDomain<MutableBoxView<>>::value, "");
+  static_assert(!HasBoxDomain<int>);
+  static_assert(HasBoxDomain<BoxView<>>);
+  static_assert(HasBoxDomain<Box<>>);
+  static_assert(HasBoxDomain<MutableBoxView<>>);
   Box<> box({1, 2}, {3, 4});
   BoxView<> view = box;
   EXPECT_EQ(box, GetBoxDomainOf(box));
@@ -738,11 +749,7 @@ TEST(BoxTest, DeductionGuides) {
   auto box = Box({1, 2}, {3, 4});
   static_assert(std::is_same_v<decltype(box), Box<2>>);
 
-  auto box_view1 = BoxView({1, 2}, {3, 4});
-  static_assert(std::is_same_v<decltype(box_view1), BoxView<2>>);
-
-  auto box_view2 = BoxView(box_view1);
-  static_assert(std::is_same_v<decltype(box_view2), BoxView<2>>);
+  static_assert(std::is_same_v<decltype(BoxView({1, 2}, {3, 4})), BoxView<2>>);
 
   static_assert(decltype(box)::static_rank == 2);
 
@@ -750,4 +757,31 @@ TEST(BoxTest, DeductionGuides) {
   static_assert(std::is_same_v<decltype(box_view), MutableBoxView<2>>);
 }
 
+TEST(BoxTest, IsFinite) {
+  EXPECT_TRUE(IsFinite(Box<>()));
+  EXPECT_TRUE(IsFinite(BoxView<>()));
+  EXPECT_FALSE(IsFinite(Box<>(1)));
+  EXPECT_FALSE(IsFinite(Box<1>()));
+  EXPECT_FALSE(IsFinite(BoxView<>(1)));
+  EXPECT_FALSE(IsFinite(BoxView<>(2)));
+  EXPECT_FALSE(IsFinite(BoxView<2>()));
+  EXPECT_TRUE(IsFinite(Box<3>({1, 2, 3}, {4, 5, 6})));
+  EXPECT_TRUE(IsFinite(BoxView<3>({1, 2, 3}, {4, 5, 6})));
+  EXPECT_TRUE(IsFinite(Box<>({1, 2, 3}, {4, 5, 6})));
+  EXPECT_TRUE(IsFinite(BoxView<>({1, 2, 3}, {4, 5, 6})));
+  EXPECT_TRUE(IsFinite(Box<1>({1}, {4})));
+  EXPECT_FALSE(IsFinite(Box<3>({1, -kInfIndex, 3}, {4, 5, 6})));
+  EXPECT_FALSE(IsFinite(Box<3>({1, kInfIndex - 5, 3}, {4, 6, 6})));
+}
+
+TEST(BoxSerializationTest, StaticRank) {
+  TestSerializationRoundTrip(Box<0>());
+  TestSerializationRoundTrip(Box<3>({1, 2, 3}, {4, 5, 6}));
+}
+
+TEST(BoxSerializationTest, DynamicRank) {
+  TestSerializationRoundTrip(Box<>());
+  TestSerializationRoundTrip(Box({1, 2, 3}, {4, 5, 6}));
+}
+
 }  // namespace
