@@ -14,6 +14,8 @@
 
 #include "tensorstore/context.h"
 
+#include "absl/log/absl_check.h"
+#include "absl/log/absl_log.h"
 #include "absl/status/status.h"
 #include "absl/strings/str_join.h"
 #include "absl/synchronization/mutex.h"
@@ -21,13 +23,13 @@
 #include "tensorstore/context_resource_provider.h"
 #include "tensorstore/internal/heterogeneous_container.h"
 #include "tensorstore/internal/json_binding/json_binding.h"
-#include "tensorstore/internal/logging.h"
 #include "tensorstore/internal/mutex.h"
 #include "tensorstore/internal/no_destructor.h"
 #include "tensorstore/serialization/json.h"
 #include "tensorstore/serialization/json_bindable.h"
 #include "tensorstore/serialization/serialization.h"
 #include "tensorstore/util/quote_string.h"
+#include "tensorstore/util/str_cat.h"
 
 namespace tensorstore {
 namespace internal_context {
@@ -250,8 +252,8 @@ Result<ResourceImplStrongPtr> GetOrCreateResourceStrongPtr(
   if (!spec.provider_) {
     // The provider was not linked in, despite linking in code that depends on
     // it.  This indicates a build configuration error.
-    TENSORSTORE_LOG_FATAL("Context resource provider not registered for: ",
-                          QuoteString(spec.key_));
+    ABSL_LOG(FATAL) << "Context resource provider not registered for: "
+                    << QuoteString(spec.key_);
   }
   const std::string_view key = spec.key_;
   if (key.empty()) {
@@ -340,8 +342,8 @@ class ResourceReference : public ResourceSpecImplBase {
       }
       if (!c->parent_) {
         if (referent != provider_->id_) {
-          return absl::InvalidArgumentError(
-              StrCat("Resource not defined: ", QuoteString(referent)));
+          return absl::InvalidArgumentError(tensorstore::StrCat(
+              "Resource not defined: ", QuoteString(referent)));
         }
         // Create default.
         auto default_spec = MakeDefaultResourceSpec(*provider_, provider_->id_);
@@ -374,7 +376,7 @@ void RegisterContextResourceProvider(
   absl::MutexLock lock(&registry.mutex_);
   auto id = provider->id_;
   if (!registry.providers_.insert(std::move(provider)).second) {
-    TENSORSTORE_LOG_FATAL("Provider ", QuoteString(id), " already registered");
+    ABSL_LOG(FATAL) << "Provider " << QuoteString(id) << " already registered";
   }
 }
 
@@ -390,8 +392,8 @@ const ResourceProviderImplBase& GetProviderOrDie(std::string_view id) {
   auto* provider = GetProvider(id);
   if (!provider) {
     // Indicates a build configuration problem.
-    TENSORSTORE_LOG_FATAL("Context resource provider ", QuoteString(id),
-                          " not registered");
+    ABSL_LOG(FATAL) << "Context resource provider " << QuoteString(id)
+                    << " not registered";
   }
   return *provider;
 }
@@ -411,8 +413,8 @@ std::string_view ParseResourceProvider(std::string_view key) {
 }
 
 absl::Status ProviderNotRegisteredError(std::string_view key) {
-  return absl::InvalidArgumentError(
-      StrCat("Invalid context resource identifier: ", QuoteString(key)));
+  return absl::InvalidArgumentError(tensorstore::StrCat(
+      "Invalid context resource identifier: ", QuoteString(key)));
 }
 
 Result<ResourceSpecImplPtr> ResourceSpecFromJson(
@@ -425,9 +427,9 @@ Result<ResourceSpecImplPtr> ResourceSpecFromJson(
   } else if (auto* s = j.get_ptr<const std::string*>()) {
     auto provider_id = ParseResourceProvider(*s);
     if (provider_id != provider.id_) {
-      return absl::InvalidArgumentError(StrCat("Invalid reference to ",
-                                               QuoteString(provider.id_),
-                                               " resource: ", QuoteString(*s)));
+      return absl::InvalidArgumentError(tensorstore::StrCat(
+          "Invalid reference to ", QuoteString(provider.id_),
+          " resource: ", QuoteString(*s)));
     }
     impl.reset(new ResourceReference(*s));
   } else {
@@ -611,7 +613,7 @@ BuilderImpl::~BuilderImpl() {
       // Find the first number `i` such that `<id>#<i>` is unused.
       size_t i = 0;
       while (true) {
-        key = StrCat(resource->spec_->provider_->id_, "#", i);
+        key = tensorstore::StrCat(resource->spec_->provider_->id_, "#", i);
         if (!ids.count(key)) break;
         ++i;
       }
@@ -1008,7 +1010,7 @@ bool EncodeContextSpecBuilder(serialization::EncodeSink& sink,
     deps.emplace_back(resource, entry.spec);
     entry.shared = true;
   }
-  TENSORSTORE_CHECK(internal_context::Access::impl(builder)->use_count() == 1);
+  ABSL_CHECK_EQ(internal_context::Access::impl(builder)->use_count(), 1);
   // Rely on builder's destructor to update all of the spec keys in
   // `deps`.
   builder = internal::ContextSpecBuilder();
