@@ -17,18 +17,17 @@
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
 #include "absl/algorithm/container.h"
+#include "absl/log/absl_log.h"
 #include "absl/random/bit_gen_ref.h"
 #include "absl/random/random.h"
 #include <nlohmann/json.hpp>
 #include "tensorstore/index_space/dim_expression.h"
 #include "tensorstore/index_space/index_domain_builder.h"
 #include "tensorstore/index_space/index_transform.h"
-#include "tensorstore/index_space/index_transform_builder.h"
 #include "tensorstore/index_space/index_transform_testutil.h"
 #include "tensorstore/index_space/transformed_array.h"
 #include "tensorstore/internal/data_type_random_generator.h"
 #include "tensorstore/internal/json_gtest.h"
-#include "tensorstore/internal/logging.h"
 #include "tensorstore/internal/nditerable_transformed_array.h"
 #include "tensorstore/internal/source_location.h"
 #include "tensorstore/internal/test_util.h"
@@ -36,7 +35,9 @@
 #include "tensorstore/tensorstore.h"
 #include "tensorstore/transaction.h"
 #include "tensorstore/util/execution/sync_flow_sender.h"
+#include "tensorstore/util/iterate_over_index_range.h"
 #include "tensorstore/util/status_testutil.h"
+#include "tensorstore/util/str_cat.h"
 
 namespace tensorstore {
 namespace internal {
@@ -211,7 +212,7 @@ DriverRandomOperationTester::DriverRandomOperationTester(
 
 void DriverRandomOperationTester::TestBasicFunctionality(
     TransactionMode transaction_mode, size_t num_iterations) {
-  SCOPED_TRACE(StrCat("create_spec=", options.create_spec));
+  SCOPED_TRACE(tensorstore::StrCat("create_spec=", options.create_spec));
   Transaction transaction(transaction_mode);
   auto context = Context::Default();
   TENSORSTORE_ASSERT_OK_AND_ASSIGN(
@@ -251,10 +252,11 @@ void DriverRandomOperationTester::TestBasicFunctionality(
     auto transform = GetRandomTransform(gen, options.expected_domain);
     auto random_array = MakeRandomArray(gen, transform.domain().box(),
                                         options.initial_value.dtype());
-    if (log) TENSORSTORE_LOG("i = ", i);
-    SCOPED_TRACE(StrCat("i=", i));
-    SCOPED_TRACE(StrCat("transform=", transform));
-    SCOPED_TRACE(StrCat("original_domain=", options.initial_value.domain()));
+    if (log) ABSL_LOG(INFO) << "i = " << i;
+    SCOPED_TRACE(tensorstore::StrCat("i=", i));
+    SCOPED_TRACE(tensorstore::StrCat("transform=", transform));
+    SCOPED_TRACE(tensorstore::StrCat("original_domain=",
+                                     options.initial_value.domain()));
     auto write_future =
         tensorstore::Write(random_array, store | transform).commit_future;
     TENSORSTORE_ASSERT_OK(write_future.result());
@@ -336,7 +338,7 @@ void DriverRandomOperationTester::TestBasicFunctionality(
 void DriverRandomOperationTester::TestMultiTransactionWrite(
     TransactionMode mode, size_t num_transactions, size_t num_iterations,
     bool use_random_values) {
-  SCOPED_TRACE(StrCat("create_spec=", options.create_spec));
+  SCOPED_TRACE(tensorstore::StrCat("create_spec=", options.create_spec));
   TENSORSTORE_ASSERT_OK_AND_ASSIGN(
       auto store,
       tensorstore::Open(options.create_spec, tensorstore::OpenMode::create)
@@ -394,6 +396,209 @@ void DriverRandomOperationTester::TestMultiTransactionWrite(
   }
 }
 
+void DriverRandomOperationTester::TestConcurrentWrites(
+    TransactionMode transaction_mode, size_t num_iterations) {
+  SCOPED_TRACE(tensorstore::StrCat("create_spec=", options.create_spec));
+  Transaction transaction(transaction_mode);
+  auto context = Context::Default();
+  TENSORSTORE_ASSERT_OK_AND_ASSIGN(
+      auto store, tensorstore::Open(options.create_spec, context, transaction,
+                                    tensorstore::OpenMode::create)
+                      .result());
+  ASSERT_EQ(options.expected_domain, store.domain());
+  // Create random rank-0 array (containing just a single value).
+  auto random_array = MakeRandomArray(gen, Box<>(), store.dtype());
+  auto [promise, future] = PromiseFuturePair<void>::Make(absl::Status());
+  // Perform `num_iterations` concurrent writes.
+  for (std::size_t i = 0; i < num_iterations; ++i) {
+    auto transform = GetRandomTransform(gen, options.expected_domain);
+    LinkError(
+        promise,
+        tensorstore::Write(random_array, store | transform).commit_future);
+  }
+  promise = {};
+  // Wait for writes to complete.
+  TENSORSTORE_ASSERT_OK(future);
+}
+
+absl::Status TestDriverWriteReadChunks(
+    absl::BitGenRef gen, const TestDriverWriteReadChunksOptions& options) {
+  Context context(options.context_spec);
+  const auto is_write = options.total_write_bytes != 0;
+  tensorstore::OpenMode open_mode = is_write
+                                        ? tensorstore::OpenMode::open_or_create
+                                        : tensorstore::OpenMode::open;
+
+  tensorstore::ReadWriteMode read_write_mode =
+      is_write ? tensorstore::ReadWriteMode::read_write
+               : tensorstore::ReadWriteMode::read;
+  TENSORSTORE_ASSIGN_OR_RETURN(
+      auto ts, tensorstore::Open(options.tensorstore_spec, context, open_mode,
+                                 read_write_mode)
+                   .result());
+  ABSL_LOG(INFO) << "ts_spec=" << ts.spec().value();
+
+  Index chunk_shape[kMaxRank] = {0};
+  if (options.chunk_shape) {
+    // --chunk_size holds a list of shapes. Any unspecified shape
+    // elements are defaulted to the tensorstore domain shape
+    if (options.chunk_shape->size() > ts.rank()) {
+      return absl::InvalidArgumentError(
+          "chunk_shape exceeds the TensorStore rank.");
+    }
+    std::copy(options.chunk_shape->begin(), options.chunk_shape->end(),
+              chunk_shape);
+    for (DimensionIndex i = 0; i < ts.rank(); i++) {
+      if (chunk_shape[i] == 0) {
+        // fallback to domain shape.
+        chunk_shape[i] = ts.domain().shape()[i];
+      }
+    }
+  } else if (options.chunk_bytes) {
+    // --chunk_size holds a single value. BenchmarkChunkShape selects the
+    // shape with a hypercube side ratio as close to 1 as possible
+    // (--chunk_size is assumed to be bytes; convert to number of
+    // elements).
+    TENSORSTORE_RETURN_IF_ERROR(tensorstore::internal::ChooseChunkShape(
+        /*shape_constraints=*/tensorstore::ChunkLayout::GridView(
+            tensorstore::ChunkLayout::ChunkElementsBase(
+                *options.chunk_bytes / ts.dtype().size(), false)),
+        ts.domain().box(), span<Index>(chunk_shape, ts.rank())));
+  } else {
+    return absl::InvalidArgumentError(
+        "--chunk_shape or --chunk_bytes must be set.");
+  }
+
+  ABSL_LOG(INFO) << "read/write shape " << span(chunk_shape, ts.rank());
+  ABSL_LOG(INFO) << "Starting writes: " << options.repeat_writes
+                 << ", total_write_bytes=" << options.total_write_bytes;
+  for (int64_t i = 0; i < options.repeat_writes; i++) {
+    TENSORSTORE_RETURN_IF_ERROR(
+        TestDriverReadOrWriteChunks(gen, ts, span(chunk_shape, ts.rank()),
+                                    options.total_write_bytes, options.strategy,
+                                    /*read=*/false));
+  }
+
+  ABSL_LOG(INFO) << "Starting reads: " << options.repeat_reads
+                 << ", total_read_bytes=" << options.total_read_bytes;
+  for (int64_t i = 0; i < options.repeat_reads; i++) {
+    TENSORSTORE_RETURN_IF_ERROR(
+        TestDriverReadOrWriteChunks(gen, ts, span(chunk_shape, ts.rank()),
+                                    options.total_read_bytes, options.strategy,
+                                    /*read=*/true));
+  }
+  return absl::OkStatus();
+}
+
+namespace {
+
+void ForEachChunk(BoxView<> domain, DataType dtype, absl::BitGenRef gen,
+                  span<const Index> chunk_shape, int64_t total_bytes,
+                  TestDriverWriteReadChunksOptions::Strategy strategy,
+                  absl::FunctionRef<int64_t(BoxView<> box)> callback) {
+  if (total_bytes == 0) return;
+
+  const DimensionIndex rank = domain.rank();
+  assert(rank == chunk_shape.size());
+  Index range_extent[kMaxRank] = {};
+  for (size_t i = 0; i < rank; i++) {
+    range_extent[i] = domain.shape()[i] / chunk_shape[i];
+  }
+
+  int64_t current_bytes = 0;
+  switch (strategy) {
+    case TestDriverWriteReadChunksOptions::kSequential: {
+      // Sequential reads/writes.
+      Box<> target(rank);
+
+      while (current_bytes < total_bytes) {
+        IterateOverIndexRange(
+            span(range_extent, rank), [&](span<const Index> indices) -> bool {
+              for (DimensionIndex i = 0; i < rank; i++) {
+                target[i] = IndexInterval::UncheckedSized(
+                    indices[i] * chunk_shape[i], chunk_shape[i]);
+              }
+              current_bytes += callback(target);
+              return current_bytes < total_bytes;
+            });
+      }
+      break;
+    }
+    case TestDriverWriteReadChunksOptions::kRandom: {
+      // Random reads/writes.
+      while (current_bytes < total_bytes) {
+        current_bytes +=
+            callback(ChooseRandomBoxPosition(gen, domain, chunk_shape));
+      }
+      break;
+    }
+    default:
+      ABSL_LOG(FATAL) << "Invalid strategy";
+  }
+}
+
+}  // namespace
+
+absl::Status TestDriverReadOrWriteChunks(
+    absl::BitGenRef gen, tensorstore::TensorStore<> ts,
+    span<const Index> chunk_shape, int64_t total_bytes,
+    TestDriverWriteReadChunksOptions::Strategy strategy, bool read) {
+  if (total_bytes == 0) return absl::OkStatus();
+
+  if (total_bytes < 0) {
+    total_bytes = ts.domain().num_elements() * ts.dtype().size() * -total_bytes;
+  }
+  const int64_t chunk_bytes = ProductOfExtents(chunk_shape) * ts.dtype().size();
+
+  // Record the bytes and chunks completed.
+  std::atomic<int64_t> bytes_completed = 0;
+  std::atomic<int64_t> chunks_completed = 0;
+  auto value_lambda = [&](Promise<void> a_promise, AnyFuture a_future) {
+    bytes_completed.fetch_add(chunk_bytes);
+    chunks_completed.fetch_add(1);
+  };
+
+  SharedOffsetArray<const void> array;
+  if (!read) {
+    array = MakeRandomArray(gen, tensorstore::Box<>(chunk_shape), ts.dtype());
+  }
+
+  auto start_time = absl::Now();
+  auto op = PromiseFuturePair<void>::Make(absl::OkStatus());
+  ForEachChunk(
+      ts.domain().box(), ts.dtype(), gen, chunk_shape, total_bytes, strategy,
+      [&](BoxView<> target) -> int64_t {
+        if (read) {
+          LinkValue(value_lambda, op.promise,
+                    Read(ts | AllDims().BoxSlice(target).TranslateTo(0)));
+
+        } else {
+          LinkValue(value_lambda, op.promise,
+                    Write(array, ts | AllDims().BoxSlice(target).TranslateTo(0))
+                        .commit_future);
+        }
+        return chunk_bytes;
+      });
+
+  // Wait until all operations complete.
+  op.promise = {};
+  op.future.Wait();
+  TENSORSTORE_RETURN_IF_ERROR(op.future.result());
+
+  auto elapsed_s =
+      absl::FDivDuration(absl::Now() - start_time, absl::Seconds(1));
+  double bytes_mb = static_cast<double>(bytes_completed.load()) / 1e6;
+
+  ABSL_LOG(INFO)
+      << (read ? "Read" : "Write") << " summary: "
+      << absl::StrFormat(
+             "%d bytes in %.0f ms:  %.3f MB/second (%d chunks of %d bytes)",
+             bytes_completed.load(), elapsed_s * 1e3, bytes_mb / elapsed_s,
+             chunks_completed.load(), chunk_bytes);
+
+  return absl::OkStatus();
+}
+
 void RegisterTensorStoreDriverBasicFunctionalityTest(
     TensorStoreDriverBasicFunctionalityTestOptions options) {
   if (!options.compare_arrays) {
