@@ -19,13 +19,12 @@
 #include "tensorstore/driver/driver_testutil.h"
 #include "tensorstore/index_space/dim_expression.h"
 #include "tensorstore/index_space/index_domain_builder.h"
-#include "tensorstore/internal/cache/cache.h"
 #include "tensorstore/internal/global_initializer.h"
 #include "tensorstore/internal/parse_json_matches.h"
 #include "tensorstore/kvstore/kvstore.h"
+#include "tensorstore/kvstore/operations.h"
 #include "tensorstore/kvstore/test_util.h"
 #include "tensorstore/open.h"
-#include "tensorstore/util/status.h"
 #include "tensorstore/util/status_testutil.h"
 #include "tensorstore/util/str_cat.h"
 
@@ -42,7 +41,6 @@ using ::tensorstore::kImplicit;
 using ::tensorstore::MatchesStatus;
 using ::tensorstore::Schema;
 using ::tensorstore::span;
-using ::tensorstore::StrCat;
 using ::tensorstore::internal::GetMap;
 using ::tensorstore::internal::ParseJsonMatches;
 using ::tensorstore::internal::TestSpecSchema;
@@ -357,7 +355,7 @@ TEST(N5DriverTest, Resize) {
               {{"cache_pool",
                 {{"total_bytes_limit", enable_cache ? 10000000 : 0}}}})
               .value());
-      SCOPED_TRACE(StrCat("resize_mode=", resize_mode));
+      SCOPED_TRACE(tensorstore::StrCat("resize_mode=", resize_mode));
       // Create the store.i
       ::nlohmann::json storage_spec{{"driver", "memory"}};
       ::nlohmann::json metadata_json = GetBasicResizeMetadata();
@@ -393,7 +391,7 @@ TEST(N5DriverTest, Resize) {
       auto resize_future =
           Resize(store, span<const Index>({kImplicit, kImplicit}),
                  span<const Index>({3, 2}), resize_mode);
-      ASSERT_EQ(absl::OkStatus(), GetStatus(resize_future.result()));
+      TENSORSTORE_ASSERT_OK(resize_future);
       EXPECT_EQ(tensorstore::BoxView({3, 2}),
                 resize_future.value().domain().box());
 
@@ -426,13 +424,9 @@ TEST(N5DriverTest, ResizeMetadataOnly) {
       tensorstore::Open(json_spec, context, tensorstore::OpenMode::create,
                         tensorstore::ReadWriteMode::read_write)
           .result());
-  EXPECT_EQ(
-      absl::OkStatus(),
-      GetStatus(tensorstore::Write(
-                    tensorstore::MakeArray<std::int8_t>({{1, 2, 3}, {4, 5, 6}}),
-                    store | tensorstore::AllDims().TranslateSizedInterval(
-                                {2, 1}, {2, 3}))
-                    .commit_future.result()));
+  TENSORSTORE_EXPECT_OK(tensorstore::Write(
+      tensorstore::MakeArray<std::int8_t>({{1, 2, 3}, {4, 5, 6}}),
+      store | tensorstore::AllDims().TranslateSizedInterval({2, 1}, {2, 3})));
   // Check that key value store has expected contents.
   TENSORSTORE_ASSERT_OK_AND_ASSIGN(
       auto kvs, kvstore::Open(storage_spec, context).result());
@@ -449,7 +443,7 @@ TEST(N5DriverTest, ResizeMetadataOnly) {
   auto resize_future =
       Resize(store, span<const Index>({kImplicit, kImplicit}),
              span<const Index>({3, 2}), tensorstore::resize_metadata_only);
-  ASSERT_EQ(absl::OkStatus(), GetStatus(resize_future.result()));
+  TENSORSTORE_ASSERT_OK(resize_future);
   EXPECT_EQ(tensorstore::BoxView({3, 2}), resize_future.value().domain().box());
 
   ::nlohmann::json resized_metadata_json = metadata_json;
@@ -726,14 +720,14 @@ TEST(N5DriverTest, InvalidSpecInvalidMemberType) {
   for (auto member_name : {"kvstore", "path", "metadata"}) {
     auto spec = GetJsonSpec();
     spec[member_name] = 5;
-    EXPECT_THAT(
-        tensorstore::Open(spec, tensorstore::OpenMode::create,
-                          tensorstore::ReadWriteMode::read_write)
-            .result(),
-        MatchesStatus(absl::StatusCode::kInvalidArgument,
-                      StrCat("Error parsing object member \"", member_name,
-                             "\": "
-                             "Expected .*, but received: 5")));
+    EXPECT_THAT(tensorstore::Open(spec, tensorstore::OpenMode::create,
+                                  tensorstore::ReadWriteMode::read_write)
+                    .result(),
+                MatchesStatus(absl::StatusCode::kInvalidArgument,
+                              tensorstore::StrCat(
+                                  "Error parsing object member \"", member_name,
+                                  "\": "
+                                  "Expected .*, but received: 5")));
   }
 }
 
