@@ -15,10 +15,10 @@
 #include "tensorstore/driver/neuroglancer_precomputed/chunk_encoding.h"
 
 #include "absl/algorithm/container.h"
+#include "absl/base/optimization.h"
 #include "riegeli/bytes/cord_reader.h"
 #include "riegeli/bytes/cord_writer.h"
 #include "tensorstore/internal/compression/neuroglancer_compressed_segmentation.h"
-#include "tensorstore/internal/container_to_shared.h"
 #include "tensorstore/internal/data_type_endian_conversion.h"
 #include "tensorstore/internal/flat_cord_builder.h"
 #include "tensorstore/internal/image/image_info.h"
@@ -26,6 +26,7 @@
 #include "tensorstore/internal/image/jpeg_writer.h"
 #include "tensorstore/util/endian.h"
 #include "tensorstore/util/status.h"
+#include "tensorstore/util/str_cat.h"
 
 namespace tensorstore {
 namespace internal_neuroglancer_precomputed {
@@ -40,9 +41,9 @@ Result<SharedArrayView<const void>> DecodeRawChunk(
     StridedLayoutView<4> chunk_layout, absl::Cord buffer) {
   const Index expected_bytes = ProductOfExtents(shape) * dtype.size();
   if (expected_bytes != static_cast<Index>(buffer.size())) {
-    return absl::InvalidArgumentError(StrCat("Expected chunk length to be ",
-                                             expected_bytes, ", but received ",
-                                             buffer.size(), " bytes"));
+    return absl::InvalidArgumentError(
+        tensorstore::StrCat("Expected chunk length to be ", expected_bytes,
+                            ", but received ", buffer.size(), " bytes"));
   }
   auto flat_buffer = buffer.Flatten();
   if (absl::c_equal(shape, chunk_layout.shape())) {
@@ -164,7 +165,7 @@ Result<SharedArrayView<const void>> DecodeCompressedSegmentationChunk(
           static_cast<std::uint64_t*>(full_decoded_array.data()));
       break;
     default:
-      TENSORSTORE_UNREACHABLE;  // COV_NF_LINE
+      ABSL_UNREACHABLE();  // COV_NF_LINE
   }
   if (!success) {
     return absl::InvalidArgumentError(
@@ -214,7 +215,7 @@ Result<SharedArrayView<const void>> DecodeChunk(
           metadata.dtype, chunk_shape, chunk_layout,
           scale_metadata.compressed_segmentation_block_size, std::move(buffer));
   }
-  TENSORSTORE_UNREACHABLE;  // COV_NF_LINE
+  ABSL_UNREACHABLE();  // COV_NF_LINE
 }
 
 absl::Cord EncodeRawChunk(DataType dtype, span<const Index, 4> shape,
@@ -284,7 +285,7 @@ Result<absl::Cord> EncodeCompressedSegmentationChunk(
           &out);
       break;
     default:
-      TENSORSTORE_UNREACHABLE;  // COV_NF_LINE
+      ABSL_UNREACHABLE();  // COV_NF_LINE
   }
   return absl::Cord(std::move(out));
 }
@@ -309,7 +310,7 @@ Result<absl::Cord> EncodeChunk(span<const Index> chunk_indices,
           metadata.dtype, partial_chunk_shape, array,
           scale_metadata.compressed_segmentation_block_size);
   }
-  TENSORSTORE_UNREACHABLE;  // COV_NF_LINE
+  ABSL_UNREACHABLE();  // COV_NF_LINE
 }
 
 }  // namespace internal_neuroglancer_precomputed
