@@ -28,6 +28,7 @@
 #include "absl/container/flat_hash_set.h"
 #include "absl/container/inlined_vector.h"
 #include "absl/hash/hash.h"
+#include "absl/log/absl_log.h"
 #include "absl/status/status.h"
 #include "tensorstore/array.h"
 #include "tensorstore/box.h"
@@ -50,9 +51,8 @@
 #include "tensorstore/internal/irregular_grid.h"
 #include "tensorstore/internal/json_binding/bindable.h"
 #include "tensorstore/internal/json_binding/json_binding.h"
-#include "tensorstore/internal/json_binding/staleness_bound.h"
+#include "tensorstore/internal/json_binding/staleness_bound.h"  // IWYU pragma: keep
 #include "tensorstore/internal/json_binding/std_array.h"  // IWYU pragma: keep
-#include "tensorstore/internal/logging.h"
 #include "tensorstore/internal/tagged_ptr.h"
 #include "tensorstore/internal/type_traits.h"
 #include "tensorstore/open_mode.h"
@@ -100,8 +100,7 @@ using ::tensorstore::internal::WriteChunk;
 //
 // To implement fill_value, we may need to do the following:
 // 1. In the unspecified case, convert the datatype in GetFillValue() from
-// `json`
-//    to the dtype of the driver.
+//    `json` to the dtype of the driver.
 // 2. Clean the options when propagating them to each layer. Some layers error
 //    when fill_value is set.
 // 3. Handle missing cells on read (see UnmappedReadOp) to submit chunks for
@@ -230,7 +229,7 @@ class StackDriverSpec
                                    internal::GetEffectiveDomain(layers[i]));
       if (!effective_domain.valid()) {
         return absl::InvalidArgumentError(
-            StrCat("layer[", i, "] domain is unspecified"));
+            tensorstore::StrCat("layer[", i, "] domain is unspecified"));
       }
       domains.emplace_back(std::move(effective_domain));
       // validate rank.
@@ -238,7 +237,7 @@ class StackDriverSpec
         rank = domains.back().rank();
       } else if (domains.back().rank() != rank) {
         return absl::InvalidArgumentError(
-            StrCat("layer[", i, "] rank mismatch"));
+            tensorstore::StrCat("layer[", i, "] rank mismatch"));
       }
     }
     return domains;
@@ -358,6 +357,11 @@ Future<internal::Driver::Handle> StackDriverSpec::Open(
   if (read_write_mode == ReadWriteMode::dynamic) {
     read_write_mode = ReadWriteMode::read_write;
   }
+  if (!schema.dtype().valid()) {
+    return absl::InvalidArgumentError(
+        "Unable to infer \"dtype\" in \"stack\" driver");
+  }
+
   TENSORSTORE_ASSIGN_OR_RETURN(auto domains, GetEffectiveDomainsForLayers());
 
   auto driver_ptr =
@@ -402,7 +406,7 @@ absl::Status StackDriver::InitializeGridIndices(
   IterateOverIndexRange<>(
       span<const Index>(start), grid_.shape(), [this](span<const Index> key) {
         if (auto it = grid_to_layer_.find(key); it == grid_to_layer_.end()) {
-          TENSORSTORE_LOG("\"stack\" driver missing grid cell: ", key);
+          ABSL_LOG(INFO) << "\"stack\" driver missing grid cell: " << key;
         }
       });
 #endif
@@ -613,8 +617,8 @@ struct UnmappedReadOp {
                           IndexTransformView<> cell_transform) {
     auto origin = self->grid_.cell_origin(grid_cell_indices);
     return absl::InvalidArgumentError(
-        StrCat("Read cell origin=", span(origin),
-               " missing layer mapping in \"stack\" driver"));
+        tensorstore::StrCat("Read cell origin=", span(origin),
+                            " missing layer mapping in \"stack\" driver"));
   }
 };
 
@@ -673,8 +677,8 @@ struct UnmappedWriteOp {
                           IndexTransformView<> cell_transform) {
     auto origin = self->grid_.cell_origin(grid_cell_indices);
     return absl::InvalidArgumentError(
-        StrCat("Write cell origin=", span(origin),
-               " missing layer mapping in \"stack\" driver"));
+        tensorstore::StrCat("Write cell origin=", span(origin),
+                            " missing layer mapping in \"stack\" driver"));
   }
 };
 
