@@ -14,28 +14,33 @@
 
 #include "tensorstore/index_space/index_transform_builder.h"
 
+#include <vector>
+
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
 #include "tensorstore/array.h"
 #include "tensorstore/box.h"
+#include "tensorstore/index_space/index_domain_builder.h"
+#include "tensorstore/util/dimension_set.h"
 #include "tensorstore/util/span.h"
 #include "tensorstore/util/status.h"
 #include "tensorstore/util/status_testutil.h"
 
 namespace {
 
-using tensorstore::Index;
-using tensorstore::IndexInterval;
-using tensorstore::IndexTransform;
-using tensorstore::IndexTransformBuilder;
-using tensorstore::kInfIndex;
-using tensorstore::kInfSize;
-using tensorstore::MakeArray;
-using tensorstore::MatchesStatus;
-using tensorstore::OutputIndexMethod;
-using tensorstore::span;
-using tensorstore::Status;
-using tensorstore::internal_index_space::TransformAccess;
+using ::tensorstore::DimensionSet;
+using ::tensorstore::Index;
+using ::tensorstore::IndexDomainBuilder;
+using ::tensorstore::IndexInterval;
+using ::tensorstore::IndexTransform;
+using ::tensorstore::IndexTransformBuilder;
+using ::tensorstore::kInfIndex;
+using ::tensorstore::kInfSize;
+using ::tensorstore::MakeArray;
+using ::tensorstore::MatchesStatus;
+using ::tensorstore::OutputIndexMethod;
+using ::tensorstore::span;
+using ::tensorstore::internal_index_space::TransformAccess;
 
 TEST(IndexTransformTest, BuilderValid) {
   auto index_array = MakeArray<Index>({{{1, 0, 2, 2}}});
@@ -52,12 +57,12 @@ TEST(IndexTransformTest, BuilderValid) {
           .output_index_array(3, 7, 9, index_array, IndexInterval::Closed(0, 3))
           .Finalize()
           .value();
-  static_assert(std::is_same<decltype(t), IndexTransform<3, 4>>::value, "");
+  static_assert(std::is_same_v<decltype(t), IndexTransform<3, 4>>);
   EXPECT_THAT(t.input_origin(), ::testing::ElementsAre(1, 2, 3));
   EXPECT_THAT(t.input_shape(), ::testing::ElementsAre(2, 2, 4));
   EXPECT_THAT(t.input_labels(), ::testing::ElementsAre("x", "y", "z"));
-  EXPECT_THAT(t.implicit_lower_bounds(), ::testing::ElementsAre(0, 1, 0));
-  EXPECT_THAT(t.implicit_upper_bounds(), ::testing::ElementsAre(1, 0, 0));
+  EXPECT_THAT(t.implicit_lower_bounds(), DimensionSet({0, 1, 0}));
+  EXPECT_THAT(t.implicit_upper_bounds(), DimensionSet({1, 0, 0}));
   EXPECT_EQ(IndexInterval::UncheckedSized(1, 2),
             t.input_domain()[0].interval());
   EXPECT_EQ(IndexInterval::UncheckedSized(2, 2),
@@ -95,8 +100,9 @@ TEST(IndexTransformTest, BuilderValid) {
   }
   {
     std::array<Index, 4> output_indices;
-    ASSERT_EQ(Status(), t.TransformIndices(span<const Index, 3>({1, 2, 3}),
-                                           output_indices));
+    ASSERT_EQ(
+        absl::OkStatus(),
+        t.TransformIndices(span<const Index, 3>({1, 2, 3}), output_indices));
     EXPECT_THAT(output_indices, ::testing::ElementsAre(4, 26, 6, 16));
   }
 }
@@ -169,8 +175,8 @@ TEST(IndexTransformBuilderTest, Default) {
   auto t = IndexTransformBuilder<>(2, 1).Finalize().value();
   EXPECT_THAT(t.input_origin(), ::testing::ElementsAre(-kInfIndex, -kInfIndex));
   EXPECT_THAT(t.input_shape(), ::testing::ElementsAre(kInfSize, kInfSize));
-  EXPECT_THAT(t.implicit_lower_bounds(), ::testing::ElementsAre(1, 1));
-  EXPECT_THAT(t.implicit_upper_bounds(), ::testing::ElementsAre(1, 1));
+  EXPECT_THAT(t.implicit_lower_bounds(), DimensionSet({1, 1}));
+  EXPECT_THAT(t.implicit_upper_bounds(), DimensionSet({1, 1}));
   EXPECT_THAT(t.input_labels(), ::testing::ElementsAre("", ""));
   auto map = t.output_index_map(0);
   EXPECT_EQ(0, map.offset());
@@ -186,8 +192,8 @@ TEST(IndexTransformBuilderTest, InputOriginSpecified) {
             IndexInterval::UncheckedClosed(1, kInfIndex));
   EXPECT_EQ(t.domain()[1].interval(),
             IndexInterval::UncheckedClosed(2, kInfIndex));
-  EXPECT_THAT(t.implicit_lower_bounds(), ::testing::ElementsAre(0, 0));
-  EXPECT_THAT(t.implicit_upper_bounds(), ::testing::ElementsAre(1, 1));
+  EXPECT_THAT(t.implicit_lower_bounds(), DimensionSet({0, 0}));
+  EXPECT_THAT(t.implicit_upper_bounds(), DimensionSet({1, 1}));
   EXPECT_THAT(t.input_labels(), ::testing::ElementsAre("", ""));
 }
 
@@ -202,8 +208,8 @@ TEST(IndexTransformBuilderTest, ImplicitLowerBoundsSpecified) {
             IndexInterval::UncheckedClosed(-kInfIndex, kInfIndex));
   EXPECT_EQ(t.domain()[1].interval(),
             IndexInterval::UncheckedClosed(-kInfIndex, kInfIndex));
-  EXPECT_THAT(t.implicit_lower_bounds(), ::testing::ElementsAre(1, 0));
-  EXPECT_THAT(t.implicit_upper_bounds(), ::testing::ElementsAre(1, 1));
+  EXPECT_THAT(t.implicit_lower_bounds(), DimensionSet({1, 0}));
+  EXPECT_THAT(t.implicit_upper_bounds(), DimensionSet({1, 1}));
   EXPECT_THAT(t.input_labels(), ::testing::ElementsAre("", ""));
 }
 
@@ -213,8 +219,8 @@ TEST(IndexTransformBuilderTest, InputShapeSpecified) {
       IndexTransformBuilder<>(2, 0).input_shape({5, 10}).Finalize().value();
   EXPECT_EQ(t.domain()[0].interval(), IndexInterval::UncheckedSized(0, 5));
   EXPECT_EQ(t.domain()[1].interval(), IndexInterval::UncheckedSized(0, 10));
-  EXPECT_THAT(t.implicit_lower_bounds(), ::testing::ElementsAre(0, 0));
-  EXPECT_THAT(t.implicit_upper_bounds(), ::testing::ElementsAre(0, 0));
+  EXPECT_THAT(t.implicit_lower_bounds(), DimensionSet({0, 0}));
+  EXPECT_THAT(t.implicit_upper_bounds(), DimensionSet({0, 0}));
   EXPECT_THAT(t.input_labels(), ::testing::ElementsAre("", ""));
 }
 
@@ -229,8 +235,8 @@ TEST(IndexTransformBuilderTest, InputInclusiveMaxSpecified) {
             IndexInterval::UncheckedClosed(-kInfIndex, 5));
   EXPECT_EQ(t.domain()[1].interval(),
             IndexInterval::UncheckedClosed(-kInfIndex, 10));
-  EXPECT_THAT(t.implicit_lower_bounds(), ::testing::ElementsAre(1, 1));
-  EXPECT_THAT(t.implicit_upper_bounds(), ::testing::ElementsAre(0, 0));
+  EXPECT_THAT(t.implicit_lower_bounds(), DimensionSet({1, 1}));
+  EXPECT_THAT(t.implicit_upper_bounds(), DimensionSet({0, 0}));
   EXPECT_THAT(t.input_labels(), ::testing::ElementsAre("", ""));
 }
 
@@ -245,8 +251,8 @@ TEST(IndexTransformBuilderTest, InputExclusiveMaxSpecified) {
             IndexInterval::UncheckedHalfOpen(-kInfIndex, 5));
   EXPECT_EQ(t.domain()[1].interval(),
             IndexInterval::UncheckedHalfOpen(-kInfIndex, 10));
-  EXPECT_THAT(t.implicit_lower_bounds(), ::testing::ElementsAre(1, 1));
-  EXPECT_THAT(t.implicit_upper_bounds(), ::testing::ElementsAre(0, 0));
+  EXPECT_THAT(t.implicit_lower_bounds(), DimensionSet({1, 1}));
+  EXPECT_THAT(t.implicit_upper_bounds(), DimensionSet({0, 0}));
   EXPECT_THAT(t.input_labels(), ::testing::ElementsAre("", ""));
 }
 
@@ -262,8 +268,8 @@ TEST(IndexTransformBuilderTest, ImplicitUpperBoundsSpecified) {
             IndexInterval::UncheckedClosed(-kInfIndex, kInfIndex));
   EXPECT_EQ(t.domain()[1].interval(),
             IndexInterval::UncheckedClosed(-kInfIndex, kInfIndex));
-  EXPECT_THAT(t.implicit_lower_bounds(), ::testing::ElementsAre(1, 1));
-  EXPECT_THAT(t.implicit_upper_bounds(), ::testing::ElementsAre(1, 0));
+  EXPECT_THAT(t.implicit_lower_bounds(), DimensionSet({1, 1}));
+  EXPECT_THAT(t.implicit_upper_bounds(), DimensionSet({1, 0}));
   EXPECT_THAT(t.input_labels(), ::testing::ElementsAre("", ""));
 }
 
@@ -273,74 +279,115 @@ TEST(IndexTransformBuilderTest, SingleInputDimensionDefaults) {
                 .Finalize()
                 .value(),
             IndexTransformBuilder<>(3, 1)
-                .output_single_input_dimension(0, 0, 1, 2)
+                .output_single_input_dimension(0, 2)
                 .Finalize()
                 .value());
 }
 
-TEST(IndexTransformBuilderTest, ErrorHandling) {
-  // input_origin out of range
-  EXPECT_THAT(IndexTransformBuilder<>(2, 1)
-                  .input_origin({-kInfIndex - 1, -kInfIndex})
-                  .Finalize(),
-              MatchesStatus(absl::StatusCode::kInvalidArgument));
+TEST(IndexTransformBuilderTest, InputOriginOutOfRange) {
+  EXPECT_THAT(
+      IndexTransformBuilder<>(2, 1)
+          .input_origin({-kInfIndex - 1, -kInfIndex})
+          .Finalize(),
+      MatchesStatus(absl::StatusCode::kInvalidArgument,
+                    ".* do not specify a valid half-open index interval"));
+}
 
-  // input_shape out of range
-  EXPECT_THAT(IndexTransformBuilder<>(2, 1).input_shape({1, -1}).Finalize(),
-              MatchesStatus(absl::StatusCode::kInvalidArgument));
+TEST(IndexTransformBuilderTest, InputShapeOutOfRange) {
+  EXPECT_THAT(
+      IndexTransformBuilder<>(2, 1).input_shape({1, -1}).Finalize(),
+      MatchesStatus(absl::StatusCode::kInvalidArgument,
+                    "\\(0, -1\\) do not specify a valid sized index interval"));
+}
 
-  // invalid input dimension
-  EXPECT_THAT(IndexTransformBuilder<>(2, 1)
-                  .output_single_input_dimension(0, 0, 1, -1)
-                  .Finalize(),
-              MatchesStatus(absl::StatusCode::kInvalidArgument));
+TEST(IndexTransformBuilderTest, InvalidInputDimensionNegative) {
+  EXPECT_THAT(
+      IndexTransformBuilder<>(2, 1)
+          .output_single_input_dimension(0, 0, 1, -1)
+          .Finalize(),
+      MatchesStatus(absl::StatusCode::kInvalidArgument,
+                    "Input dimension -1 specified for output dimension 0 "
+                    "is outside valid range \\[0, 2\\)"));
+}
 
-  // invalid input dimension
-  EXPECT_THAT(IndexTransformBuilder<>(2, 1)
-                  .output_single_input_dimension(0, 0, 1, 2)
-                  .Finalize(),
-              MatchesStatus(absl::StatusCode::kInvalidArgument));
+TEST(IndexTransformBuilderTest, InvalidInputDimensionPositive) {
+  EXPECT_THAT(
+      IndexTransformBuilder<>(2, 1)
+          .output_single_input_dimension(0, 2)
+          .Finalize(),
+      MatchesStatus(absl::StatusCode::kInvalidArgument,
+                    "Input dimension 2 specified for output dimension 0 "
+                    "is outside valid range \\[0, 2\\)"));
+}
 
-  // invalid index array rank
+TEST(IndexTransformBuilderTest, InvalidIndexArrayRank) {
   EXPECT_THAT(IndexTransformBuilder<>(2, 1)
                   .output_index_array(0, 0, 1, MakeArray<Index>({1}))
                   .Finalize(),
-              MatchesStatus(absl::StatusCode::kInvalidArgument));
+              MatchesStatus(absl::StatusCode::kInvalidArgument,
+                            "Index array for output dimension 0 "
+                            "has rank 1 but must have rank 2"));
+}
 
-  // invalid index array shape
-  EXPECT_THAT(IndexTransformBuilder<>(2, 1)
-                  .input_shape({2, 2})
-                  .output_index_array(
-                      0, 0, 1, MakeArray<Index>({{1, 2}, {3, 4}, {5, 6}}))
-                  .Finalize(),
-              MatchesStatus(absl::StatusCode::kInvalidArgument));
+TEST(IndexTransformBuilderTest, InvalidIndexArrayShape) {
+  EXPECT_THAT(
+      IndexTransformBuilder<>(2, 1)
+          .input_shape({2, 2})
+          .output_index_array(0, 0, 1,
+                              MakeArray<Index>({{1, 2}, {3, 4}, {5, 6}}))
+          .Finalize(),
+      MatchesStatus(absl::StatusCode::kInvalidArgument,
+                    "Index array for output dimension 0 has shape \\{3, 2\\} "
+                    "which does not match input_shape \\{2, 2\\}"));
+}
 
-  // index array depends on input dimension with implicit lower bound
-  EXPECT_THAT(IndexTransformBuilder<>(2, 1)
-                  .input_shape({3, 2})
-                  .implicit_lower_bounds({1, 0})
-                  .output_index_array(
-                      0, 0, 1, MakeArray<Index>({{1, 2}, {3, 4}, {5, 6}}))
-                  .Finalize(),
-              MatchesStatus(absl::StatusCode::kInvalidArgument));
+TEST(IndexTransformBuilderTest, InvalidIndexArrayImplicitLowerBound) {
+  EXPECT_THAT(
+      IndexTransformBuilder<>(2, 1)
+          .input_shape({3, 2})
+          .implicit_lower_bounds({1, 0})
+          .output_index_array(0, 0, 1,
+                              MakeArray<Index>({{1, 2}, {3, 4}, {5, 6}}))
+          .Finalize(),
+      MatchesStatus(absl::StatusCode::kInvalidArgument,
+                    "Index array for output dimension 0 "
+                    "depends on input dimension 0 with implicit bounds"));
+}
 
-  // index array depends on input dimension with implicit upper bound
-  EXPECT_THAT(IndexTransformBuilder<>(2, 1)
-                  .input_shape({3, 2})
-                  .implicit_upper_bounds({1, 0})
-                  .output_index_array(
-                      0, 0, 1, MakeArray<Index>({{1, 2}, {3, 4}, {5, 6}}))
-                  .Finalize(),
-              MatchesStatus(absl::StatusCode::kInvalidArgument));
+TEST(IndexTransformBuilderTest, InvalidIndexArrayImplicitUpperBound) {
+  EXPECT_THAT(
+      IndexTransformBuilder<>(2, 1)
+          .input_shape({3, 2})
+          .implicit_upper_bounds({1, 0})
+          .output_index_array(0, 0, 1,
+                              MakeArray<Index>({{1, 2}, {3, 4}, {5, 6}}))
+          .Finalize(),
+      MatchesStatus(absl::StatusCode::kInvalidArgument,
+                    "Index array for output dimension 0 "
+                    "depends on input dimension 0 with implicit bounds"));
+}
 
-  // invalid index array bounds
+TEST(IndexTransformBuilderTest, InvalidIndexArrayIndexRange) {
   EXPECT_THAT(
       IndexTransformBuilder<>(2, 1)
           .input_shape({2, 2})
           .output_index_array(0, 0, 1, MakeArray<Index>({{1, 2}, {3, 4}}),
                               IndexInterval::Sized(3, -1))
           .Finalize(),
-      MatchesStatus(absl::StatusCode::kInvalidArgument));
+      MatchesStatus(absl::StatusCode::kInvalidArgument,
+                    "\\(3, -1\\) do not specify a valid sized index interval"));
+}
+
+TEST(IndexTransformBuilderTest, InvalidIndexArrayWithUnboundedDomain) {
+  EXPECT_THAT(
+      IndexTransformBuilder(1, 1)
+          .input_origin({tensorstore::kMaxFiniteIndex})
+          .input_shape({2})
+          .output_index_array(0, 0, 1, MakeArray<Index>({1, 2}))
+          .Finalize(),
+      MatchesStatus(absl::StatusCode::kInvalidArgument,
+                    "Index array for output dimension 0 "
+                    "depends on input dimension 0 with infinite bounds"));
 }
 
 TEST(IndexTransformBuilderDeathTest, InvalidArguments) {
@@ -363,18 +410,6 @@ TEST(IndexTransformBuilderDeathTest, InvalidArguments) {
                "invalid output dimension");
 }
 
-// Test the special allocation-free code path for rank zero.
-TEST(IndexTransformBuilderTest, RankZero) {
-  auto transform = IndexTransformBuilder<>(0, 0).Finalize().value();
-  EXPECT_TRUE(transform.input_origin().empty());
-  EXPECT_TRUE(transform.input_shape().empty());
-  EXPECT_TRUE(transform.input_labels().empty());
-  EXPECT_TRUE(transform.output_index_maps().empty());
-
-  auto transform2 = IndexTransformBuilder<>(0, 0).Finalize().value();
-  EXPECT_EQ(TransformAccess::rep(transform), TransformAccess::rep(transform2));
-}
-
 TEST(IndexTransformBuilderTest, OutputStrideZero) {
   // Setting an output stride of zero should make the output index map
   // constant.
@@ -437,7 +472,7 @@ TEST(IndexTransformBuilderTest, ExclusiveMaxAfterShape) {
 }
 
 // Tests that the input domain bounds can be set from a box using the
-// `input_domain_box` method.
+// `input_bounds` method.
 TEST(IndexTransformBuilderTest, InputDomainBox) {
   auto t = IndexTransformBuilder<>(2, 2)
                .input_bounds(tensorstore::BoxView({1, 2}, {2, 3}))
@@ -457,13 +492,69 @@ TEST(IndexTransformBuilderTest, InputDomain) {
                                          .implicit_upper_bounds({1, 0})
                                          .input_labels({"x", "y"})
                                          .Finalize()
-                                         .value());
+                                         .value()
+                                         .domain());
 
   auto t =
       IndexTransformBuilder<>(2, 2).input_domain(domain).Finalize().value();
   EXPECT_EQ(domain, t.domain());
 }
 
+TEST(IndexTransformBuilderTest, OutputIdentityTransform) {
+  EXPECT_THAT(
+      IndexTransformBuilder(2, 2).output_identity_transform().Finalize(),
+      ::testing::Optional(tensorstore::IdentityTransform(2)));
+  EXPECT_EQ(IndexTransformBuilder(3, 2)
+                .output_single_input_dimension(0, 0)
+                .output_single_input_dimension(1, 1)
+                .Finalize()
+                .value(),
+            IndexTransformBuilder(3, 2)
+                .output_identity_transform()
+                .Finalize()
+                .value());
+  EXPECT_EQ(IndexTransformBuilder(2, 3)
+                .output_single_input_dimension(0, 0)
+                .output_single_input_dimension(1, 1)
+                .output_constant(2, 0)
+                .Finalize()
+                .value(),
+            IndexTransformBuilder(2, 3)
+                .output_identity_transform()
+                .Finalize()
+                .value());
+}
+
+TEST(IndexTransformBuilderTest, CopyOutputMap) {
+  TENSORSTORE_ASSERT_OK_AND_ASSIGN(
+      auto t,
+      IndexTransformBuilder(3, 4)
+          .input_origin({1, 2, 3})
+          .input_shape({2, 2, 4})
+          .implicit_lower_bounds({0, 1, 0})
+          .implicit_upper_bounds({1, 0, 0})
+          .input_labels({"x", "y", "z"})
+          .output_constant(0, 4)
+          .output_single_input_dimension(1, 5, 7, 2)
+          .output_constant(2, 6)
+          .output_index_array(3, 7, 9, MakeArray<Index>({{{1, 0, 2, 2}}}),
+                              IndexInterval::Closed(0, 3))
+          .Finalize());
+  EXPECT_THAT(IndexTransformBuilder(3, 4)
+                  .input_domain(t.domain())
+                  .output_maps(t.output_index_maps())
+                  .Finalize(),
+              ::testing::Optional(t));
+  EXPECT_THAT(IndexTransformBuilder(3, 4)
+                  .input_domain(t.domain())
+                  .output_constant(0, 4)
+                  .output_map(1, t.output_index_maps()[1])
+                  .output_map(2, t.output_index_maps()[2])
+                  .output_map(3, t.output_index_maps()[3])
+                  .Finalize(),
+              ::testing::Optional(t));
+}
+
 TEST(InitializeTransformRepForBuilder, Basic) {
   auto source = tensorstore::internal_index_space::TransformRep::Allocate(1, 2);
   source->output_rank = 2;
@@ -479,7 +570,108 @@ TEST(IndexTransformBuilder, NonUniqueLabels) {
   EXPECT_THAT(
       IndexTransformBuilder<>(3, 0).input_labels({"a", "", "a"}).Finalize(),
       MatchesStatus(absl::StatusCode::kInvalidArgument,
-                    "Dimension label \"a\" is not unique"));
+                    "Dimension label\\(s\\) \"a\" not unique"));
+}
+
+TEST(IndexTransformBuilderTest, IndexArrayWithEmptyExplicitDomain) {
+  TENSORSTORE_ASSERT_OK_AND_ASSIGN(auto expected,  //
+                                   IndexTransformBuilder(2, 2)
+                                       .input_shape({0, 2})
+                                       .output_constant(0, 0)
+                                       .output_constant(1, 1)
+                                       .Finalize());
+  EXPECT_THAT(IndexTransformBuilder(2, 2)
+                  .input_shape({0, 2})
+                  .output_index_array(0, 0, 1, MakeArray<Index>({{2, 3}}))
+                  .output_constant(1, 1)
+                  .Finalize(),
+              ::testing::Optional(expected));
+}
+
+TEST(IndexDomainBuilderTest, Null) {
+  IndexDomainBuilder builder(nullptr);
+  EXPECT_FALSE(builder.valid());
+}
+
+TEST(IndexDomainBuilderTest, Basic) {
+  IndexDomainBuilder builder(3);
+  EXPECT_EQ(3, builder.rank());
+  builder.origin(span<const Index, 3>({1, 2, 3}));
+  EXPECT_THAT(builder.origin(), ::testing::ElementsAre(1, 2, 3));
+  builder.shape(span<const Index, 3>({4, 5, 6}));
+  EXPECT_THAT(builder.shape(), ::testing::ElementsAre(4, 5, 6));
+  builder.exclusive_max(span<const Index, 3>({4, 5, 6}));
+  EXPECT_THAT(builder.exclusive_max(), ::testing::ElementsAre(4, 5, 6));
+  builder.inclusive_max(span<const Index, 3>({4, 5, 6}));
+  EXPECT_THAT(builder.inclusive_max(), ::testing::ElementsAre(4, 5, 6));
+  builder.implicit_lower_bounds({0, 1, 1});
+  builder.implicit_upper_bounds({1, 0, 1});
+  EXPECT_THAT(builder.implicit_lower_bounds(), DimensionSet({0, 1, 1}));
+  EXPECT_THAT(builder.implicit_upper_bounds(), DimensionSet({1, 0, 1}));
+  builder.labels(std::vector<std::string>{"x", "y", "z"});
+  EXPECT_THAT(builder.labels(), ::testing::ElementsAre("x", "y", "z"));
+}
+
+// Tests that the labels can be set using the `labels` method.
+TEST(IndexDomainBuilderTest, Labels) {
+  auto d = IndexDomainBuilder(2).labels({"x", "y"}).Finalize().value();
+  EXPECT_THAT(d.labels(), ::testing::ElementsAre("x", "y"));
+}
+
+// Tests that the upper bound can be set using the `inclusive_max` method.
+TEST(IndexDomainBuilderTest, InclusiveMax) {
+  auto d = IndexDomainBuilder(2)
+               .origin({1, 2})
+               .inclusive_max({3, 5})
+               .Finalize()
+               .value();
+  EXPECT_THAT(d.origin(), ::testing::ElementsAre(1, 2));
+  EXPECT_THAT(d.shape(), ::testing::ElementsAre(3, 4));
+}
+
+// Tests that the upper bound can be set using the `shape` method.
+TEST(IndexDomainBuilderTest, Shape) {
+  auto d =
+      IndexDomainBuilder(2).origin({1, 2}).shape({3, 5}).Finalize().value();
+  EXPECT_THAT(d.origin(), ::testing::ElementsAre(1, 2));
+  EXPECT_THAT(d.shape(), ::testing::ElementsAre(3, 5));
+}
+
+// Tests that the upper bound can be set using the `exclusive_max` method.
+TEST(IndexDomainBuilderTest, ExclusiveMax) {
+  auto d = IndexDomainBuilder(2)
+               .origin({1, 2})
+               .exclusive_max({3, 5})
+               .Finalize()
+               .value();
+  EXPECT_THAT(d.origin(), ::testing::ElementsAre(1, 2));
+  EXPECT_THAT(d.shape(), ::testing::ElementsAre(2, 3));
+}
+
+// Tests that the bounds can be set from a box using the `bounds` method.
+TEST(IndexDomainBuilderTest, InputDomainBox) {
+  auto d = IndexDomainBuilder(2)
+               .bounds(tensorstore::BoxView({1, 2}, {2, 3}))
+               .Finalize()
+               .value();
+  EXPECT_THAT(d.origin(), ::testing::ElementsAre(1, 2));
+  EXPECT_THAT(d.shape(), ::testing::ElementsAre(2, 3));
+}
+
+// Tests that the domain can be set from an existing IndexDomain using the
+// `domain` method.
+TEST(IndexDomainBuilderTest, InputDomain) {
+  TENSORSTORE_ASSERT_OK_AND_ASSIGN(tensorstore::IndexDomain<2> domain,
+                                   IndexDomainBuilder<2>()
+                                       .origin({1, 2})
+                                       .shape({3, 4})
+                                       .implicit_lower_bounds({0, 1})
+                                       .implicit_upper_bounds({1, 0})
+                                       .labels({"x", "y"})
+                                       .Finalize());
+  TENSORSTORE_ASSERT_OK_AND_ASSIGN(
+      auto d, IndexDomainBuilder<>(2).domain(domain).Finalize());
+  EXPECT_EQ(domain, d);
 }
 
 }  // namespace
