@@ -26,7 +26,7 @@ namespace internal_index_space {
 absl::Status CheckIndexVectorSize(IndexVectorOrScalarView indices,
                                   DimensionIndex size) {
   if (indices.pointer && indices.size_or_scalar != size)
-    return absl::InvalidArgumentError(StrCat(
+    return absl::InvalidArgumentError(tensorstore::StrCat(
         "Number of dimensions (", size, ") does not match number of indices (",
         indices.size_or_scalar, ")"));
   return absl::OkStatus();
