@@ -18,13 +18,14 @@
 #include <new>
 #include <utility>
 
+#include "absl/base/optimization.h"
 #include "absl/container/fixed_array.h"
+#include "absl/log/absl_check.h"
 #include "absl/status/status.h"
 #include "tensorstore/index_space/index_transform.h"
 #include "tensorstore/index_space/internal/transform_rep_impl.h"
 #include "tensorstore/internal/dimension_labels.h"
 #include "tensorstore/internal/integer_overflow.h"
-#include "tensorstore/internal/memory.h"
 #include "tensorstore/util/division.h"
 #include "tensorstore/util/quote_string.h"
 #include "tensorstore/util/str_cat.h"
@@ -137,9 +138,9 @@ void OutputIndexMap::Assign(DimensionIndex rank, const OutputIndexMap& other) {
 
 TransformRep::Ptr<> TransformRep::Allocate(
     DimensionIndex input_rank_capacity, DimensionIndex output_rank_capacity) {
-  TENSORSTORE_CHECK(input_rank_capacity >= 0 && output_rank_capacity >= 0 &&
-                    input_rank_capacity <= kMaxRank &&
-                    output_rank_capacity <= kMaxRank);
+  ABSL_CHECK(input_rank_capacity >= 0 && output_rank_capacity >= 0 &&
+             input_rank_capacity <= kMaxRank &&
+             output_rank_capacity <= kMaxRank);
   const size_t total_size =
       // header size
       sizeof(TransformRep) +
@@ -313,7 +314,7 @@ bool AreIndexMapsEqual(const OutputIndexMap& a, const OutputIndexMap& b,
                      index_array_data_b.byte_strides));
     }
   }
-  TENSORSTORE_UNREACHABLE;  // COV_NF_LINE
+  ABSL_UNREACHABLE();  // COV_NF_LINE
 }
 
 bool AreDomainsEqual(TransformRep* a, TransformRep* b) {
@@ -477,9 +478,9 @@ absl::Status TransformIndices(TransformRep* data,
   for (DimensionIndex i = 0; i < input_rank; ++i) {
     auto oi_interval = data->input_dimension(i).optionally_implicit_domain();
     if (!Contains(oi_interval.effective_interval(), input_indices[i])) {
-      return absl::OutOfRangeError(
-          StrCat("Index ", input_indices[i], " is not contained in the domain ",
-                 oi_interval, " for input dimension ", i));
+      return absl::OutOfRangeError(tensorstore::StrCat(
+          "Index ", input_indices[i], " is not contained in the domain ",
+          oi_interval, " for input dimension ", i));
     }
   }
   for (DimensionIndex output_dim = 0; output_dim < output_rank; ++output_dim) {
@@ -487,7 +488,8 @@ absl::Status TransformIndices(TransformRep* data,
         output_indices[output_dim],
         output_index_maps[output_dim](input_indices),
         MaybeAnnotateStatus(
-            _, StrCat("Computing index for output dimension ", output_dim)));
+            _, tensorstore::StrCat("Computing index for output dimension ",
+                                   output_dim)));
   }
   return absl::OkStatus();
 }
@@ -500,8 +502,8 @@ absl::Status ReplaceZeroRankIndexArrayIndexMap(Index index,
   Index new_offset;
   if (internal::MulOverflow(index, *output_stride, &new_offset) ||
       internal::AddOverflow(new_offset, *output_offset, output_offset)) {
-    return absl::InvalidArgumentError(
-        StrCat("Integer overflow computing offset for output dimension."));
+    return absl::InvalidArgumentError(tensorstore::StrCat(
+        "Integer overflow computing offset for output dimension."));
   }
   *output_stride = 0;
   return absl::OkStatus();
