@@ -26,7 +26,6 @@
 #include "tensorstore/internal/json_binding/dimension_indexed.h"
 #include "tensorstore/internal/json_binding/json_binding.h"
 #include "tensorstore/internal/json_binding/std_optional.h"
-#include "tensorstore/internal/logging.h"
 #include "tensorstore/internal/type_traits.h"
 #include "tensorstore/json_serialization_options.h"
 #include "tensorstore/rank.h"
@@ -329,9 +328,9 @@ constexpr auto IndexTransformOutputParser(
           if (obj->output) {
             if (output_rank_constraint != dynamic_rank &&
                 obj->output->size() != output_rank_constraint) {
-              return absl::InvalidArgumentError(
-                  StrCat("Expected output rank to be ", output_rank_constraint,
-                         ", but is: ", obj->output->size()));
+              return absl::InvalidArgumentError(tensorstore::StrCat(
+                  "Expected output rank to be ", output_rank_constraint,
+                  ", but is: ", obj->output->size()));
             }
             return absl::OkStatus();
           }
