@@ -15,13 +15,29 @@
 #ifndef TENSORSTORE_INDEX_SPACE_OUTPUT_INDEX_METHOD_H_
 #define TENSORSTORE_INDEX_SPACE_OUTPUT_INDEX_METHOD_H_
 
+#include <iosfwd>
+
 namespace tensorstore {
 
 /// Specifies the method by which the index into a given output dimension of an
 /// index transform is computed from the input indices.
 ///
-/// \see OutputIndexMapRef
-enum class OutputIndexMethod { constant, single_input_dimension, array };
+/// \relates OutputIndexMapRef
+enum class OutputIndexMethod {
+  /// Output index is equal to the constant ``offset`` and does not depend on
+  /// any input dimensions.
+  constant,
+  /// Output index is equal to ``offset + stride * input[input_dimension]``.
+  single_input_dimension,
+  /// Output index is equal to ``offset + stride * array(input)``.
+  array
+};
+
+/// Prints to an output stream.
+///
+/// \relates OutputIndexMethod
+/// \id OutputIndexMethod
+std::ostream& operator<<(std::ostream& os, OutputIndexMethod method);
 
 }  // namespace tensorstore
 
