@@ -16,31 +16,33 @@
 
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
+#include "tensorstore/index_space/index_domain_builder.h"
 #include "tensorstore/index_space/index_transform.h"
 #include "tensorstore/index_space/index_transform_builder.h"
 #include "tensorstore/index_space/internal/transform_rep.h"
-#include "tensorstore/util/bit_vec.h"
+#include "tensorstore/util/dimension_set.h"
 #include "tensorstore/util/status.h"
 #include "tensorstore/util/status_testutil.h"
 
 namespace {
 
-using tensorstore::BitVec;
-using tensorstore::Box;
-using tensorstore::BoxView;
-using tensorstore::DimensionIndex;
-using tensorstore::Index;
-using tensorstore::IndexInterval;
-using tensorstore::IndexTransform;
-using tensorstore::IndexTransformBuilder;
-using tensorstore::kInfIndex;
-using tensorstore::kMinFiniteIndex;
-using tensorstore::MakeArray;
-using tensorstore::MatchesStatus;
-using tensorstore::PropagateBounds;
-using tensorstore::PropagateBoundsToTransform;
-using tensorstore::PropagateExplicitBounds;
-using tensorstore::PropagateExplicitBoundsToTransform;
+using ::tensorstore::Box;
+using ::tensorstore::BoxView;
+using ::tensorstore::DimensionIndex;
+using ::tensorstore::DimensionSet;
+using ::tensorstore::Index;
+using ::tensorstore::IndexDomainBuilder;
+using ::tensorstore::IndexInterval;
+using ::tensorstore::IndexTransform;
+using ::tensorstore::IndexTransformBuilder;
+using ::tensorstore::kInfIndex;
+using ::tensorstore::kMinFiniteIndex;
+using ::tensorstore::MakeArray;
+using ::tensorstore::MatchesStatus;
+using ::tensorstore::PropagateBounds;
+using ::tensorstore::PropagateBoundsToTransform;
+using ::tensorstore::PropagateExplicitBounds;
+using ::tensorstore::PropagateExplicitBoundsToTransform;
 
 // Tests that a default-constructed (invalid) IndexTransform is treated as an
 // identity transform.
@@ -48,21 +50,20 @@ TEST(PropagateExplicitBoundsTest, IdentityTransform) {
   DimensionIndex rank = 2;
   const Box<> b({2, 3}, {4, 5});
   Box<> a(rank);
-  EXPECT_EQ(absl::OkStatus(),
-            PropagateExplicitBounds(b, IndexTransform<>(), a));
+  TENSORSTORE_ASSERT_OK(PropagateExplicitBounds(b, IndexTransform<>(), a));
   EXPECT_EQ(a, b);
 }
 
 TEST(PropagateBoundsTest, IdentityTransform) {
   auto b = Box({2, 3}, {4, 5});
   Box<2> a;
-  BitVec<2> a_implicit_lower_bounds, a_implicit_upper_bounds;
-  auto b_implicit_lower_bounds = BitVec({0, 1});
-  auto b_implicit_upper_bounds = BitVec({1, 0});
-  EXPECT_EQ(absl::OkStatus(),
-            PropagateBounds(b, b_implicit_lower_bounds, b_implicit_upper_bounds,
-                            IndexTransform<2, 2>(), a, a_implicit_lower_bounds,
-                            a_implicit_upper_bounds));
+  DimensionSet a_implicit_lower_bounds, a_implicit_upper_bounds;
+  auto b_implicit_lower_bounds = DimensionSet({0, 1});
+  auto b_implicit_upper_bounds = DimensionSet({1, 0});
+  TENSORSTORE_ASSERT_OK(
+      PropagateBounds(b, b_implicit_lower_bounds, b_implicit_upper_bounds,
+                      IndexTransform<2, 2>(), a, a_implicit_lower_bounds,
+                      a_implicit_upper_bounds));
   EXPECT_EQ(a, b);
   EXPECT_EQ(b_implicit_lower_bounds, a_implicit_lower_bounds);
   EXPECT_EQ(b_implicit_upper_bounds, a_implicit_upper_bounds);
@@ -79,18 +80,17 @@ TEST(PropagateBoundsTest, ValidateOnly) {
                        .value();
   const Box<3> b({2, 3, 4}, {50, 66, 100});
   Box<2> a;
-  BitVec<3> b_implicit_lower_bounds({0, 1, 0});
-  BitVec<3> b_implicit_upper_bounds({1, 0, 0});
-  BitVec<2> a_implicit_lower_bounds, a_implicit_upper_bounds;
+  DimensionSet b_implicit_lower_bounds({0, 1, 0});
+  DimensionSet b_implicit_upper_bounds({1, 0, 0});
+  DimensionSet a_implicit_lower_bounds, a_implicit_upper_bounds;
 
-  EXPECT_EQ(absl::OkStatus(),
-            PropagateBounds(b, b_implicit_lower_bounds, b_implicit_upper_bounds,
-                            transform, a, a_implicit_lower_bounds,
-                            a_implicit_upper_bounds));
+  TENSORSTORE_ASSERT_OK(PropagateBounds(
+      b, b_implicit_lower_bounds, b_implicit_upper_bounds, transform, a,
+      a_implicit_lower_bounds, a_implicit_upper_bounds));
   // Check that the propagated bounds match the transform input domain.
   EXPECT_EQ(BoxView({2, 3}, {5, 10}), a);
-  EXPECT_THAT(a_implicit_lower_bounds, ::testing::ElementsAre(0, 0));
-  EXPECT_THAT(a_implicit_upper_bounds, ::testing::ElementsAre(0, 0));
+  EXPECT_THAT(a_implicit_lower_bounds, DimensionSet({0, 0}));
+  EXPECT_THAT(a_implicit_upper_bounds, DimensionSet({0, 0}));
 }
 
 TEST(PropagateBoundsTest, Constant) {
@@ -100,11 +100,10 @@ TEST(PropagateBoundsTest, Constant) {
                        .Finalize()
                        .value();
   Box<0> a;
-  EXPECT_EQ(absl::OkStatus(),
-            PropagateBounds(/*b_domain=*/Box({2, 1}, {2, 3}),
-                            /*b_implicit_lower_bounds=*/BitVec({1, 0}),
-                            /*b_implicit_upper_bounds=*/BitVec({0, 0}),
-                            transform, a));
+  TENSORSTORE_ASSERT_OK(PropagateBounds(
+      /*b_domain=*/Box({2, 1}, {2, 3}),
+      /*b_implicit_lower_bounds=*/DimensionSet({1, 0}),
+      /*b_implicit_upper_bounds=*/DimensionSet({0, 0}), transform, a));
 }
 
 TEST(PropagateBoundsTest, ConstantError) {
@@ -114,13 +113,27 @@ TEST(PropagateBoundsTest, ConstantError) {
                        .Finalize()
                        .value();
   Box<0> a;
-  EXPECT_THAT(
-      PropagateBounds(/*b_domain=*/Box({2, 1}, {2, 3}),
-                      /*b_implicit_lower_bounds=*/BitVec({0, 1}),
-                      /*b_implicit_upper_bounds=*/BitVec({0, 0}), transform, a),
-      MatchesStatus(absl::StatusCode::kOutOfRange,
-                    "Checking bounds of constant output index map for "
-                    "dimension 0: Index 1 is outside valid range .*"));
+  EXPECT_THAT(PropagateBounds(/*b_domain=*/Box({2, 1}, {2, 3}),
+                              /*b_implicit_lower_bounds=*/DimensionSet({0, 1}),
+                              /*b_implicit_upper_bounds=*/DimensionSet({0, 0}),
+                              transform, a),
+              MatchesStatus(absl::StatusCode::kOutOfRange,
+                            "Checking bounds of constant output index map for "
+                            "dimension 0: Index 1 is outside valid range .*"));
+}
+
+TEST(PropagateBoundsTest, ConstantEmptyDomain) {
+  TENSORSTORE_ASSERT_OK_AND_ASSIGN(auto transform,
+                                   (IndexTransformBuilder<2, 1>()
+                                        .input_shape({0, 2})
+                                        .output_constant(0, 42)
+                                        .Finalize()));
+  Box<2> a;
+  TENSORSTORE_EXPECT_OK(PropagateBounds(
+      /*b_domain=*/Box<1>({5}),
+      /*b_implicit_lower_bounds=*/DimensionSet({0}),
+      /*b_implicit_upper_bounds=*/DimensionSet({0}), transform, a));
+  EXPECT_EQ(a, BoxView({0, 2}));
 }
 
 TEST(PropagateBoundsTest, Propagate0Upper1Lower) {
@@ -135,13 +148,12 @@ TEST(PropagateBoundsTest, Propagate0Upper1Lower) {
                        .Finalize()
                        .value();
   Box<2> a;
-  BitVec<2> a_implicit_lower_bounds, a_implicit_upper_bounds;
-  EXPECT_EQ(
-      absl::OkStatus(),
-      PropagateBounds(/*b_domain=*/Box({2, 3, 4}, {50, 66, 100}),
-                      /*b_implicit_lower_bounds=*/BitVec({0, 1, 1}),
-                      /*b_implicit_upper_bounds=*/BitVec({1, 0, 1}), transform,
-                      a, a_implicit_lower_bounds, a_implicit_upper_bounds));
+  DimensionSet a_implicit_lower_bounds, a_implicit_upper_bounds;
+  TENSORSTORE_ASSERT_OK(PropagateBounds(
+      /*b_domain=*/Box({2, 3, 4}, {50, 66, 100}),
+      /*b_implicit_lower_bounds=*/DimensionSet({0, 1, 1}),
+      /*b_implicit_upper_bounds=*/DimensionSet({1, 0, 1}), transform, a,
+      a_implicit_lower_bounds, a_implicit_upper_bounds));
 
   // a dim 0:
   //   Initial bounds:     [2, 7*)
@@ -157,8 +169,8 @@ TEST(PropagateBoundsTest, Propagate0Upper1Lower) {
 
   // Check that the propagated implicit bits are equal to:
   // `initial && any(dependencies)`.
-  EXPECT_THAT(a_implicit_lower_bounds, ::testing::ElementsAre(0, 1));
-  EXPECT_THAT(a_implicit_upper_bounds, ::testing::ElementsAre(1, 0));
+  EXPECT_THAT(a_implicit_lower_bounds, DimensionSet({0, 1}));
+  EXPECT_THAT(a_implicit_upper_bounds, DimensionSet({1, 0}));
 }
 
 // Tests that implicit bounds from a single dimension of `b` don't constrain the
@@ -168,17 +180,16 @@ TEST(PropagateBoundsTest, PropagateImplicitConstraints1) {
                              .input_origin({-1})
                              .input_exclusive_max({2})
                              .implicit_upper_bounds({1})
-                             .output_single_input_dimension(0, 0)
+                             .output_identity_transform()
                              .Finalize()
                              .value();
   Box<1> a;
-  BitVec<1> a_implicit_lower_bounds, a_implicit_upper_bounds;
-  EXPECT_EQ(
-      absl::OkStatus(),
+  DimensionSet a_implicit_lower_bounds, a_implicit_upper_bounds;
+  TENSORSTORE_ASSERT_OK(
       PropagateBounds(/*b_domain=*/Box({0}, {4}),
-                      /*b_implicit_lower_bounds=*/BitVec({1}),
-                      /*b_implicit_upper_bounds=*/BitVec({0}), transform, a,
-                      a_implicit_lower_bounds, a_implicit_upper_bounds));
+                      /*b_implicit_lower_bounds=*/DimensionSet({1}),
+                      /*b_implicit_upper_bounds=*/DimensionSet({0}), transform,
+                      a, a_implicit_lower_bounds, a_implicit_upper_bounds));
 
   // a dim 0:
   //   Initial bounds:     [-1, 2*)
@@ -189,8 +200,8 @@ TEST(PropagateBoundsTest, PropagateImplicitConstraints1) {
 
   // Check that the propagated implicit bits are equal to:
   // `initial && any(dependencies)`.
-  EXPECT_THAT(a_implicit_lower_bounds, ::testing::ElementsAre(0));
-  EXPECT_THAT(a_implicit_upper_bounds, ::testing::ElementsAre(0));
+  EXPECT_THAT(a_implicit_lower_bounds, DimensionSet({0}));
+  EXPECT_THAT(a_implicit_upper_bounds, DimensionSet({0}));
 }
 
 // Tests that implicit bounds from two dimensions of `b` don't constrain the
@@ -205,13 +216,12 @@ TEST(PropagateBoundsTest, PropagateImplicitConstraints2) {
                              .Finalize()
                              .value();
   Box<1> a;
-  BitVec<1> a_implicit_lower_bounds, a_implicit_upper_bounds;
-  EXPECT_EQ(absl::OkStatus(),
-            PropagateBounds(
-                /*b_domain=*/Box({-1, 0}, {3, 4}),
-                /*b_implicit_lower_bounds=*/BitVec({1, 1}),
-                /*b_implicit_upper_bounds=*/BitVec({1, 0}), transform, a,
-                a_implicit_lower_bounds, a_implicit_upper_bounds));
+  DimensionSet a_implicit_lower_bounds, a_implicit_upper_bounds;
+  TENSORSTORE_ASSERT_OK(PropagateBounds(
+      /*b_domain=*/Box({-1, 0}, {3, 4}),
+      /*b_implicit_lower_bounds=*/DimensionSet({1, 1}),
+      /*b_implicit_upper_bounds=*/DimensionSet({1, 0}), transform, a,
+      a_implicit_lower_bounds, a_implicit_upper_bounds));
 
   // a dim 0:
   //   Initial bounds:     [-1, 2*)
@@ -223,8 +233,8 @@ TEST(PropagateBoundsTest, PropagateImplicitConstraints2) {
 
   // Check that the propagated implicit bits are equal to:
   // `initial && any(dependencies)`.
-  EXPECT_THAT(a_implicit_lower_bounds, ::testing::ElementsAre(0));
-  EXPECT_THAT(a_implicit_upper_bounds, ::testing::ElementsAre(0));
+  EXPECT_THAT(a_implicit_lower_bounds, DimensionSet({0}));
+  EXPECT_THAT(a_implicit_upper_bounds, DimensionSet({0}));
 }
 
 TEST(PropagateBoundsTest, PropagateNegativeStride) {
@@ -238,13 +248,12 @@ TEST(PropagateBoundsTest, PropagateNegativeStride) {
                        .value();
   const Box<1> b({2}, {50});
   Box<2> a;
-  BitVec<1> b_implicit_lower_bounds({0}), b_implicit_upper_bounds({1});
-  BitVec<2> a_implicit_lower_bounds, a_implicit_upper_bounds;
+  DimensionSet b_implicit_lower_bounds({0}), b_implicit_upper_bounds({1});
+  DimensionSet a_implicit_lower_bounds, a_implicit_upper_bounds;
 
-  EXPECT_EQ(absl::OkStatus(),
-            PropagateBounds(b, b_implicit_lower_bounds, b_implicit_upper_bounds,
-                            transform, a, a_implicit_lower_bounds,
-                            a_implicit_upper_bounds));
+  TENSORSTORE_ASSERT_OK(PropagateBounds(
+      b, b_implicit_lower_bounds, b_implicit_upper_bounds, transform, a,
+      a_implicit_lower_bounds, a_implicit_upper_bounds));
 
   // a dim 0:
   //   Initial bounds:     [2, 6*)
@@ -258,8 +267,8 @@ TEST(PropagateBoundsTest, PropagateNegativeStride) {
 
   // Check that the propagated implicit bits are equal to:
   // `initial && any(dependencies)`.
-  EXPECT_THAT(a_implicit_lower_bounds, ::testing::ElementsAre(0, 1));
-  EXPECT_THAT(a_implicit_upper_bounds, ::testing::ElementsAre(0, 0));
+  EXPECT_THAT(a_implicit_lower_bounds, DimensionSet({0, 1}));
+  EXPECT_THAT(a_implicit_upper_bounds, DimensionSet({0, 0}));
 }
 
 TEST(PropagateExplicitBoundsTest, Propagate0Upper1Upper) {
@@ -275,7 +284,7 @@ TEST(PropagateExplicitBoundsTest, Propagate0Upper1Upper) {
                        .value();
   const Box<3> b({2, 3, 4}, {50, 66, 100});
   Box<2> a;
-  EXPECT_EQ(absl::OkStatus(), PropagateExplicitBounds(b, transform, a));
+  TENSORSTORE_ASSERT_OK(PropagateExplicitBounds(b, transform, a));
   EXPECT_EQ(Box<>({2, -9}, {5, 22}), a);
 }
 
@@ -292,7 +301,7 @@ TEST(PropagateExplicitBoundsTest, PropagateExtraExplicit) {
                        .value();
   const Box<3> b({2, 3, 4}, {50, 66, 100});
   Box<3> a;
-  EXPECT_EQ(absl::OkStatus(), PropagateExplicitBounds(b, transform, a));
+  TENSORSTORE_ASSERT_OK(PropagateExplicitBounds(b, transform, a));
   EXPECT_EQ(Box<>({2, -9, 7}, {5, 22, 8}), a);
 }
 
@@ -309,7 +318,7 @@ TEST(PropagateExplicitBoundsTest, PropagateExtraImplicitLower) {
                        .value();
   const Box<3> b({2, 3, 4}, {50, 66, 100});
   Box<3> a;
-  EXPECT_EQ(absl::OkStatus(), PropagateExplicitBounds(b, transform, a));
+  TENSORSTORE_ASSERT_OK(PropagateExplicitBounds(b, transform, a));
   EXPECT_EQ(Box<>({2, -9, 7}, {5, 22, 8}), a);
 }
 
@@ -326,7 +335,7 @@ TEST(PropagateExplicitBoundsTest, PropagateExtraImplicitUpper) {
                        .value();
   const Box<3> b({2, 3, 4}, {50, 66, 100});
   Box<3> a;
-  EXPECT_EQ(absl::OkStatus(), PropagateExplicitBounds(b, transform, a));
+  TENSORSTORE_ASSERT_OK(PropagateExplicitBounds(b, transform, a));
   EXPECT_EQ(Box<>({2, -9, 7}, {5, 22, 8}), a);
 }
 
@@ -360,9 +369,25 @@ TEST(PropagateExplicitBoundsTest, OutOfBounds) {
   Box<2> a;
   EXPECT_THAT(
       PropagateExplicitBounds(b, transform, a),
-      MatchesStatus(absl::StatusCode::kOutOfRange,
-                    "Propagated bounds \\[-9, 11\\) for dimension 1 are "
-                    "incompatible with existing bounds \\[3, 13\\)\\."));
+      MatchesStatus(
+          absl::StatusCode::kOutOfRange,
+          "Propagated bounds \\[-9, 11\\), with size=20, for dimension 1 are "
+          "incompatible with existing bounds \\[3, 13\\), with size=10.*"));
+}
+
+// Tests that bounds checking is not performed in the case of an empty domain.
+TEST(PropagateExplicitBoundsTest, OutOfBoundsEmptyDomain) {
+  auto transform = IndexTransformBuilder<2, 3>()
+                       .input_origin({2, 3})
+                       .input_shape({0, 10})
+                       .output_single_input_dimension(0, 15, 2, 0)
+                       .output_single_input_dimension(1, 30, 3, 1)
+                       .output_single_input_dimension(2, 45, 4, 1)
+                       .Finalize()
+                       .value();
+  const Box<3> b({2, 3, 4}, {50, 60, 100});
+  Box<2> a;
+  TENSORSTORE_EXPECT_OK(PropagateExplicitBounds(b, transform, a));
 }
 
 TEST(PropagateExplicitBoundsTest, OutOfBoundsInfLower) {
@@ -376,11 +401,11 @@ TEST(PropagateExplicitBoundsTest, OutOfBoundsInfLower) {
                        .value();
   const Box<3> b({2, 3, 4}, {50, 60, 100});
   Box<2> a;
-  EXPECT_THAT(
-      PropagateExplicitBounds(b, transform, a),
-      MatchesStatus(absl::StatusCode::kOutOfRange,
-                    "Propagated bounds \\[-9, 11\\) for dimension 1 are "
-                    "incompatible with existing bounds \\(-inf, 4\\)\\."));
+  EXPECT_THAT(PropagateExplicitBounds(b, transform, a),
+              MatchesStatus(
+                  absl::StatusCode::kOutOfRange,
+                  "Propagated bounds \\[-9, 11\\), with size=20, for dimension "
+                  "1 are incompatible with existing bounds \\(-inf, 4\\).*"));
 }
 
 TEST(PropagateExplicitBoundsTest, OutOfBoundsInfUpper) {
@@ -396,9 +421,10 @@ TEST(PropagateExplicitBoundsTest, OutOfBoundsInfUpper) {
   Box<2> a;
   EXPECT_THAT(
       PropagateExplicitBounds(b, transform, a),
-      MatchesStatus(absl::StatusCode::kOutOfRange,
-                    "Propagated bounds \\[-9, 11\\) for dimension 1 are "
-                    "incompatible with existing bounds \\[2, \\+inf\\)\\."));
+      MatchesStatus(
+          absl::StatusCode::kOutOfRange,
+          "Propagated bounds \\[-9, 11\\), with size=20, for dimension 1 are "
+          "incompatible with existing bounds \\[2, \\+inf\\).*"));
 }
 
 TEST(PropagateExplicitBoundsTest, Overflow) {
@@ -430,7 +456,7 @@ TEST(PropagateExplicitBoundsTest, ZeroSize) {
                        .value();
   const Box<3> b({2, 3, 4}, {50, 66, 100});
   Box<2> a;
-  EXPECT_EQ(absl::OkStatus(), PropagateExplicitBounds(b, transform, a));
+  TENSORSTORE_ASSERT_OK(PropagateExplicitBounds(b, transform, a));
   // Check that the propagated bounds match the transform input domain.
   EXPECT_EQ(BoxView({2, 3}, {5, 0}), a);
 }
@@ -441,17 +467,16 @@ TEST(PropagateExplicitBoundsToTransformTest,
      InvalidTransformTreatedAsIdentityTransformDefaultImplicit) {
   IndexTransform<2, 2> t;
   Box<2> output_domain({1, 2}, {3, 4});
-  auto t_result = PropagateExplicitBoundsToTransform(output_domain, t);
-  ASSERT_TRUE(t_result);
-  EXPECT_EQ(IndexTransformBuilder<>(2, 2)
-                .input_bounds(output_domain)
-                .implicit_lower_bounds({0, 0})
-                .implicit_upper_bounds({0, 0})
-                .output_single_input_dimension(0, 0)
-                .output_single_input_dimension(1, 1)
-                .Finalize()
-                .value(),
-            *t_result);
+  TENSORSTORE_ASSERT_OK_AND_ASSIGN(  //
+      auto t_expected,               //
+      IndexTransformBuilder(2, 2)
+          .input_bounds(output_domain)
+          .implicit_lower_bounds({0, 0})
+          .implicit_upper_bounds({0, 0})
+          .output_identity_transform()
+          .Finalize());
+  EXPECT_THAT(PropagateExplicitBoundsToTransform(output_domain, t),
+              ::testing::Optional(t_expected));
 }
 
 // Tests that an invalid transform is correctly treated as an identity transform
@@ -460,18 +485,17 @@ TEST(PropagateBoundsToTransformTest,
      InvalidTransformTreatedAsIdentityTransformImplicit) {
   IndexTransform<2, 2> t;
   Box<2> output_domain({1, 2}, {3, 4});
-  auto t_result = PropagateBoundsToTransform(output_domain, BitVec({1, 0}),
-                                             BitVec({0, 1}), t);
-  ASSERT_TRUE(t_result);
-  EXPECT_EQ(IndexTransformBuilder<>(2, 2)
-                .input_bounds(output_domain)
-                .implicit_lower_bounds({1, 0})
-                .implicit_upper_bounds({0, 1})
-                .output_single_input_dimension(0, 0)
-                .output_single_input_dimension(1, 1)
-                .Finalize()
-                .value(),
-            *t_result);
+  TENSORSTORE_ASSERT_OK_AND_ASSIGN(  //
+      auto t_expected,               //
+      IndexTransformBuilder(2, 2)
+          .input_bounds(output_domain)
+          .implicit_lower_bounds({1, 0})
+          .implicit_upper_bounds({0, 1})
+          .output_identity_transform()
+          .Finalize());
+  EXPECT_THAT(PropagateBoundsToTransform(output_domain, DimensionSet({1, 0}),
+                                         DimensionSet({0, 1}), t),
+              ::testing::Optional(t_expected));
 }
 
 // Tests that calling PropagateExplicitBoundsToTransform with an index transform
@@ -486,9 +510,31 @@ TEST(PropagateExplicitBoundsToTransformTest, IndexArrayNoPropagationNeeded) {
                                    IndexInterval::Closed(1, 2))
                .Finalize()
                .value();
-  auto t_result = PropagateExplicitBoundsToTransform(output_domain, t);
-  ASSERT_TRUE(t_result);
-  EXPECT_EQ(t, *t_result);
+  EXPECT_THAT(PropagateExplicitBoundsToTransform(output_domain, t),
+              ::testing::Optional(t));
+}
+
+// Tests that the index array is converted to a constant map since the domain
+// becomes explicitly empty.
+TEST(PropagateExplicitBoundsToTransformTest, IndexArrayZeroElements) {
+  Box<2> output_domain({0, 2});
+  TENSORSTORE_ASSERT_OK_AND_ASSIGN(  //
+      auto t,                        //
+      IndexTransformBuilder(2, 2)
+          .input_shape({3, 2})
+          .implicit_upper_bounds({1, 0})
+          .output_single_input_dimension(0, 0)
+          .output_index_array(1, 0, 1, MakeArray<Index>({{1, 2}}))
+          .Finalize());
+  TENSORSTORE_ASSERT_OK_AND_ASSIGN(  //
+      auto t_expected,               //
+      IndexTransformBuilder(2, 2)
+          .input_shape({0, 2})
+          .output_single_input_dimension(0, 0)
+          .output_constant(1, 0)
+          .Finalize());
+  EXPECT_THAT(PropagateExplicitBoundsToTransform(output_domain, t),
+              ::testing::Optional(t_expected));
 }
 
 // Tests that calling PropagateExplicitBoundsToTransform with an index transform
@@ -497,39 +543,38 @@ TEST(PropagateExplicitBoundsToTransformTest, IndexArrayNoPropagationNeeded) {
 TEST(PropagateExplicitBoundsToTransformTest,
      SingleInputDimensionNoPropagationNeeded) {
   Box<1> output_domain({1}, {10});
-  auto t = IndexTransformBuilder<1, 1>()
-               .input_origin({11})
-               .input_shape({3})
-               .output_single_input_dimension(0, -32, 3, 0)
-               .Finalize()
-               .value();
-  auto t_result = PropagateExplicitBoundsToTransform(output_domain, t);
-  ASSERT_TRUE(t_result);
-  EXPECT_EQ(t, *t_result);
+  TENSORSTORE_ASSERT_OK_AND_ASSIGN(  //
+      auto t,                        //
+      IndexTransformBuilder(1, 1)
+          .input_origin({11})
+          .input_shape({3})
+          .output_single_input_dimension(0, -32, 3, 0)
+          .Finalize());
+  EXPECT_THAT(PropagateExplicitBoundsToTransform(output_domain, t),
+              ::testing::Optional(t));
 }
 
 /// Tests that the index_range for an index array output index map is correctly
 /// computed using GetAffineTransformDomain.
 TEST(PropagateExplicitBoundsToTransformTest, PropagateToIndexRange) {
   Box<1> output_domain({1}, {10});
-  auto t = IndexTransformBuilder<1, 1>()
-               .input_origin({11})
-               .input_shape({3})
-               .output_index_array(0, 2, 3, MakeArray<Index>({1, 2, 1}))
-               .Finalize()
-               .value();
-  auto t_result = PropagateExplicitBoundsToTransform(output_domain, t);
-  ASSERT_TRUE(t_result);
-  auto t_expected =
-      IndexTransformBuilder<>(1, 1)
+  TENSORSTORE_ASSERT_OK_AND_ASSIGN(  //
+      auto t,                        //
+      IndexTransformBuilder(1, 1)
+          .input_origin({11})
+          .input_shape({3})
+          .output_index_array(0, 2, 3, MakeArray<Index>({1, 2, 1}))
+          .Finalize());
+  TENSORSTORE_ASSERT_OK_AND_ASSIGN(  //
+      auto t_expected,               //
+      IndexTransformBuilder(1, 1)
           .input_origin({11})
           .input_shape({3})
           .output_index_array(0, 2, 3, MakeArray<Index>({1, 2, 1}),
                               IndexInterval::Closed(0, 2))
-          .Finalize()
-          .value();
-
-  EXPECT_EQ(t_expected, *t_result);
+          .Finalize());
+  EXPECT_THAT(PropagateExplicitBoundsToTransform(output_domain, t),
+              ::testing::Optional(t_expected));
 }
 
 /// Tests that implicit bounds do not propagate to the index range of index
@@ -548,49 +593,65 @@ TEST(PropagateBoundsToTransformTest, PropagateToIndexRange) {
             .value();
       };
   EXPECT_THAT(  //
-      PropagateBoundsToTransform(output_domain, BitVec({0}), BitVec({0}),
+      PropagateBoundsToTransform(output_domain, DimensionSet({0}),
+                                 DimensionSet({0}),
                                  get_transform(IndexInterval())),
       ::testing::Optional(get_transform(IndexInterval::Closed(0, 2))));
 
   EXPECT_THAT(  //
-      PropagateBoundsToTransform(output_domain, BitVec({1}), BitVec({0}),
+      PropagateBoundsToTransform(output_domain, DimensionSet({1}),
+                                 DimensionSet({0}),
                                  get_transform(IndexInterval())),
       ::testing::Optional(get_transform(IndexInterval::Closed(-kInfIndex, 2))));
 
   EXPECT_THAT(  //
-      PropagateBoundsToTransform(output_domain, BitVec({0}), BitVec({1}),
+      PropagateBoundsToTransform(output_domain, DimensionSet({0}),
+                                 DimensionSet({1}),
                                  get_transform(IndexInterval())),
       ::testing::Optional(get_transform(IndexInterval::Closed(0, kInfIndex))));
 
   EXPECT_THAT(  //
-      PropagateBoundsToTransform(output_domain, BitVec({1}), BitVec({1}),
+      PropagateBoundsToTransform(output_domain, DimensionSet({1}),
+                                 DimensionSet({1}),
                                  get_transform(IndexInterval())),
       ::testing::Optional(get_transform(IndexInterval())));
 }
 
 TEST(PropagateBoundsToTransformTest, PropagateToInputDomain) {
-  Box<1> output_domain({1}, {10});
+  Box<1> output_bounds({1}, {10});
   auto t = IndexTransformBuilder<1, 1>()
                .implicit_lower_bounds({1})
                .implicit_upper_bounds({1})
                .output_single_input_dimension(0, -32, 3, 0)
                .Finalize()
                .value();
-  auto t_result =
-      PropagateBoundsToTransform(output_domain, BitVec({1}), BitVec({0}), t);
-  ASSERT_TRUE(t_result);
+  TENSORSTORE_ASSERT_OK_AND_ASSIGN(
+      auto propagated_transform,
+      PropagateBoundsToTransform(output_bounds, DimensionSet({1}),
+                                 DimensionSet({0}), t));
   // Since `t` has an implicit input domain, the new input domain is computed
-  // from the `output_domain` using GetAffineTransformDomain:
-  //   {x : 1 <= (x * 3 - 32) <= 10} = {11, 12, 13, 14}
-  auto t_expected = IndexTransformBuilder<1, 1>()
-                        .input_origin({11})
-                        .input_shape({4})
-                        .implicit_lower_bounds({1})
-                        .implicit_upper_bounds({0})
-                        .output_single_input_dimension(0, -32, 3, 0)
-                        .Finalize()
-                        .value();
-  EXPECT_EQ(t_expected, *t_result);
+  // from the `output_bounds` using GetAffineTransformDomain:
+  // `{x : 1 <= (x * 3 - 32) <= 10} = {11, 12, 13, 14}`
+  auto expected_transform = IndexTransformBuilder<1, 1>()
+                                .input_origin({11})
+                                .input_shape({4})
+                                .implicit_lower_bounds({1})
+                                .implicit_upper_bounds({0})
+                                .output_single_input_dimension(0, -32, 3, 0)
+                                .Finalize()
+                                .value();
+  EXPECT_EQ(expected_transform, propagated_transform);
+
+  // Same as above, but using `IndexDomain` overload.
+  TENSORSTORE_ASSERT_OK_AND_ASSIGN(auto output_domain,
+                                   IndexDomainBuilder<1>()
+                                       .bounds(output_bounds)
+                                       .implicit_lower_bounds({1})
+                                       .implicit_upper_bounds({0})
+                                       .Finalize());
+  TENSORSTORE_ASSERT_OK_AND_ASSIGN(
+      auto propagated_transform2, PropagateBoundsToTransform(output_domain, t));
+  EXPECT_EQ(expected_transform, propagated_transform2);
 }
 
 TEST(PropagateExplicitBoundsToTransformTest, OutOfBounds) {
@@ -609,9 +670,10 @@ TEST(PropagateExplicitBoundsToTransformTest, OutOfBounds) {
   // For output dimension 2, {x : 4 <= (x * 4 + 45) <= 103} = [-10, 14].
   EXPECT_THAT(
       PropagateExplicitBoundsToTransform(output_domain, t),
-      MatchesStatus(absl::StatusCode::kOutOfRange,
-                    "Propagated bounds \\[-9, 11\\) for dimension 1 are "
-                    "incompatible with existing bounds \\[3, 13\\)\\."));
+      MatchesStatus(
+          absl::StatusCode::kOutOfRange,
+          "Propagated bounds \\[-9, 11\\), with size=20, for dimension 1 are "
+          "incompatible with existing bounds \\[3, 13\\), with size=10.*"));
 }
 
 TEST(PropagateExplicitBoundsToTransformTest, Overflow) {
