@@ -17,44 +17,43 @@
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
 #include "tensorstore/index_space/dim_expression.h"
+#include "tensorstore/index_space/index_domain_builder.h"
 #include "tensorstore/index_space/index_transform_builder.h"
 #include "tensorstore/index_space/internal/dim_expression_testutil.h"
 #include "tensorstore/util/status.h"
 
 namespace {
 
-using tensorstore::AllDims;
-using tensorstore::DimensionIndex;
-using tensorstore::Dims;
-using tensorstore::Index;
-using tensorstore::IndexInterval;
-using tensorstore::IndexTransformBuilder;
-using tensorstore::kInfIndex;
-using tensorstore::MakeArray;
-using tensorstore::span;
-using tensorstore::internal_index_space::EquivalentIndices;
-using tensorstore::internal_index_space::TestDimExpression;
+using ::tensorstore::AllDims;
+using ::tensorstore::DimensionIndex;
+using ::tensorstore::Dims;
+using ::tensorstore::Index;
+using ::tensorstore::IndexDomainBuilder;
+using ::tensorstore::IndexInterval;
+using ::tensorstore::IndexTransformBuilder;
+using ::tensorstore::kInfIndex;
+using ::tensorstore::MakeArray;
+using ::tensorstore::span;
+using ::tensorstore::internal_index_space::EquivalentIndices;
+using ::tensorstore::internal_index_space::TestDimExpression;
 
 TEST(SingleIndexSliceTest, Example) {
   const auto original_transform = IndexTransformBuilder<3, 3>()
                                       .input_origin({1, 2, 3})
                                       .input_shape({3, 4, 2})
                                       .input_labels({"x", "y", "z"})
-                                      .output_single_input_dimension(0, 0, 1, 0)
-                                      .output_single_input_dimension(1, 0, 1, 1)
-                                      .output_single_input_dimension(2, 0, 1, 2)
+                                      .output_identity_transform()
                                       .Finalize()
                                       .value();
-  const auto expected_new_transform =
-      IndexTransformBuilder<1, 3>()
-          .input_origin({2})
-          .input_shape({4})
-          .input_labels({"y"})
-          .output_constant(0, 2)
-          .output_single_input_dimension(1, 0, 1, 0)
-          .output_constant(2, 4)
-          .Finalize()
-          .value();
+  const auto expected_new_transform = IndexTransformBuilder<1, 3>()
+                                          .input_origin({2})
+                                          .input_shape({4})
+                                          .input_labels({"y"})
+                                          .output_constant(0, 2)
+                                          .output_single_input_dimension(1, 0)
+                                          .output_constant(2, 4)
+                                          .Finalize()
+                                          .value();
   const EquivalentIndices equivalent_indices = {
       {{2, 3, 4}, {3}},
   };
@@ -81,22 +80,19 @@ TEST(SingleIndexSliceTest, ImplicitLowerBound) {
                                       .implicit_lower_bounds({1, 1, 0})
                                       .input_shape({3, 4, 2})
                                       .input_labels({"x", "y", "z"})
-                                      .output_single_input_dimension(0, 0)
-                                      .output_single_input_dimension(1, 1)
-                                      .output_single_input_dimension(2, 2)
+                                      .output_identity_transform()
                                       .Finalize()
                                       .value();
-  const auto expected_new_transform =
-      IndexTransformBuilder<1, 3>()
-          .input_origin({2})
-          .implicit_lower_bounds({1})
-          .input_shape({4})
-          .input_labels({"y"})
-          .output_constant(0, -7)
-          .output_single_input_dimension(1, 0, 1, 0)
-          .output_constant(2, 4)
-          .Finalize()
-          .value();
+  const auto expected_new_transform = IndexTransformBuilder<1, 3>()
+                                          .input_origin({2})
+                                          .implicit_lower_bounds({1})
+                                          .input_shape({4})
+                                          .input_labels({"y"})
+                                          .output_constant(0, -7)
+                                          .output_single_input_dimension(1, 0)
+                                          .output_constant(2, 4)
+                                          .Finalize()
+                                          .value();
   const EquivalentIndices equivalent_indices = {
       {{-7, 3, 4}, {3}},
   };
@@ -130,7 +126,7 @@ TEST(SingleIndexSliceTest, DimSubsetUniformIndexArrayRetained) {
                     IndexTransformBuilder<1, 3>()
                         .input_origin({1})
                         .input_shape({4})
-                        .output_single_input_dimension(0, 0, 1, 0)
+                        .output_single_input_dimension(0, 0)
                         .output_constant(1, 3)
                         .output_constant(2, 3)
                         .Finalize()
@@ -168,7 +164,7 @@ TEST(SingleIndexSliceTest, DimSubsetUniformIndexArrayEliminated) {
                     IndexTransformBuilder<1, 3>()
                         .input_origin({1})
                         .input_shape({4})
-                        .output_single_input_dimension(0, 0, 1, 0)
+                        .output_single_input_dimension(0, 0)
                         .output_constant(1, 3)
                         .output_constant(2, 3)
                         .Finalize()
@@ -208,7 +204,7 @@ TEST(SingleIndexSliceTest, DimSubsetNonUniform) {
                     IndexTransformBuilder<1, 3>()
                         .input_origin({1})
                         .input_shape({4})
-                        .output_single_input_dimension(0, 0, 1, 0)
+                        .output_single_input_dimension(0, 0)
                         .output_constant(1, 3)
                         .output_constant(2, 4)
                         .Finalize()
@@ -251,7 +247,7 @@ TEST(SingleIndexSliceTest, DimSubsetNonUniformLabeled) {
                         .input_origin({1})
                         .input_shape({4})
                         .input_labels({"x"})
-                        .output_single_input_dimension(0, 0, 1, 0)
+                        .output_single_input_dimension(0, 0)
                         .output_constant(1, 3)
                         .output_constant(2, 4)
                         .Finalize()
@@ -291,7 +287,7 @@ TEST(SingleIndexSliceTest, EmptyDomain) {
                         .input_origin({1})
                         .input_shape({0})
                         .input_labels({"x"})
-                        .output_single_input_dimension(0, 0, 1, 0)
+                        .output_single_input_dimension(0, 0)
                         .output_constant(1, 3)
                         .Finalize()
                         .value(),
@@ -307,13 +303,15 @@ TEST(SingleIndexSliceTest, EmptyDomain) {
                     /*equivalent_indices=*/{});
 }
 
-TEST(SingleIndexSliceTest, ErrorHandling) {
+TEST(ErrorHandlingTest, DimensionSelectionRankMismatch) {
   TestDimExpressionError(IndexTransformBuilder<1, 1>().Finalize().value(),
                          AllDims().IndexSlice(span<const Index>({1, 2})),
                          absl::StatusCode::kInvalidArgument,
                          "Number of dimensions .* does not match number of "
                          "indices .*");
+}
 
+TEST(ErrorHandlingTest, OutOfBounds) {
   TestDimExpressionError(IndexTransformBuilder<1, 1>()
                              .input_origin({-10})
                              .input_shape({15})
@@ -321,8 +319,10 @@ TEST(SingleIndexSliceTest, ErrorHandling) {
                              .value(),
                          AllDims().IndexSlice({5}),
                          absl::StatusCode::kOutOfRange,
-                         "Slice index .* is outside valid domain .*");
+                         "Slice mismatch: .* is outside valid domain .*");
+}
 
+TEST(ErrorHandlingTest, OutOfBoundsInfinity) {
   TestDimExpressionError(IndexTransformBuilder<1, 1>()
                              .input_origin({-kInfIndex})
                              .input_shape({15})
@@ -330,9 +330,11 @@ TEST(SingleIndexSliceTest, ErrorHandling) {
                              .value(),
                          AllDims().IndexSlice({-kInfIndex}),
                          absl::StatusCode::kOutOfRange,
-                         "Slice index .* is outside valid domain .*");
+                         "Slice mismatch: .* is outside valid domain .*");
+}
 
-  TestDimExpressionError(
+TEST(ErrorHandlingTest, SingleInputDimensionMapIntegerOverflow) {
+  TestDimExpressionErrorTransformOnly(
       IndexTransformBuilder<1, 1>()
           .input_origin({0})
           .input_shape({10})
@@ -341,9 +343,12 @@ TEST(SingleIndexSliceTest, ErrorHandling) {
           .Finalize()
           .value(),
       AllDims().IndexSlice({1}), absl::StatusCode::kInvalidArgument,
-      "Integer overflow computing offset for output dimension.*");
+      "Integer overflow computing offset for output dimension.*",
+      IndexDomainBuilder<0>().Finalize().value());
+}
 
-  TestDimExpressionError(
+TEST(ErrorHandlingTest, IndexArrayMapIntegerOverflow) {
+  TestDimExpressionErrorTransformOnly(
       IndexTransformBuilder<1, 1>()
           .input_origin({0})
           .input_shape({3})
@@ -352,9 +357,12 @@ TEST(SingleIndexSliceTest, ErrorHandling) {
           .Finalize()
           .value(),
       AllDims().IndexSlice({1}), absl::StatusCode::kInvalidArgument,
-      "Integer overflow computing offset for output dimension.*");
+      "Integer overflow computing offset for output dimension.*",
+      IndexDomainBuilder<0>().Finalize().value());
+}
 
-  TestDimExpressionError(
+TEST(ErrorHandlingTest, IndexArrayMapOutOfBounds) {
+  TestDimExpressionErrorTransformOnly(
       IndexTransformBuilder<1, 1>()
           .input_origin({0})
           .input_shape({3})
@@ -363,7 +371,8 @@ TEST(SingleIndexSliceTest, ErrorHandling) {
           .Finalize()
           .value(),
       AllDims().IndexSlice({1}), absl::StatusCode::kOutOfRange,
-      "Index .* is outside valid range .*");
+      "Index .* is outside valid range .*",
+      IndexDomainBuilder<0>().Finalize().value());
 }
 
 }  // namespace
