@@ -33,29 +33,27 @@ namespace {
 #define TENSORSTORE_EXPECT_OOM(expr) EXPECT_DEATH(expr, "Out of memory");
 #endif
 
-using tensorstore::Box;
-using tensorstore::DimensionIndex;
-using tensorstore::Index;
-using tensorstore::IndexInterval;
-using tensorstore::IndexTransformBuilder;
-using tensorstore::kInfIndex;
-using tensorstore::kInfSize;
-using tensorstore::kMaxFiniteIndex;
-using tensorstore::kMinFiniteIndex;
-using tensorstore::MatchesStatus;
-using tensorstore::OutputIndexMethod;
-using tensorstore::Status;
-using tensorstore::internal::TestConcurrent;
-using tensorstore::internal_index_space::CopyTransformRep;
-using tensorstore::internal_index_space::MoveTransformRep;
-using tensorstore::internal_index_space::MutableRep;
-using tensorstore::internal_index_space::NewOrMutableRep;
-using tensorstore::internal_index_space::OutputIndexMap;
-using tensorstore::internal_index_space::ReplaceZeroRankIndexArrayIndexMap;
-using tensorstore::internal_index_space::TransformAccess;
-using tensorstore::internal_index_space::TransformRep;
-using tensorstore::internal_index_space::ValidateAndIntersectBounds;
-using tensorstore::internal_index_space::ValidateLabelsAreUnique;
+using ::tensorstore::Box;
+using ::tensorstore::DimensionIndex;
+using ::tensorstore::Index;
+using ::tensorstore::IndexInterval;
+using ::tensorstore::IndexTransformBuilder;
+using ::tensorstore::kInfIndex;
+using ::tensorstore::kInfSize;
+using ::tensorstore::kMaxFiniteIndex;
+using ::tensorstore::kMinFiniteIndex;
+using ::tensorstore::MatchesStatus;
+using ::tensorstore::OutputIndexMethod;
+using ::tensorstore::internal::TestConcurrent;
+using ::tensorstore::internal_index_space::CopyTransformRep;
+using ::tensorstore::internal_index_space::MoveTransformRep;
+using ::tensorstore::internal_index_space::MutableRep;
+using ::tensorstore::internal_index_space::NewOrMutableRep;
+using ::tensorstore::internal_index_space::OutputIndexMap;
+using ::tensorstore::internal_index_space::ReplaceZeroRankIndexArrayIndexMap;
+using ::tensorstore::internal_index_space::TransformAccess;
+using ::tensorstore::internal_index_space::TransformRep;
+using ::tensorstore::internal_index_space::ValidateAndIntersectBounds;
 
 TEST(OutputIndexMapTest, Basic) {
   OutputIndexMap map;
@@ -129,9 +127,9 @@ TEST(OutputIndexMapDeathTest, Basic) {
 
 TEST(ReplaceZeroRankIndexArrayIndexMapTest, Basic) {
   Index output_offset = 5, output_stride = 3;
-  EXPECT_EQ(Status(), ReplaceZeroRankIndexArrayIndexMap(
-                          10, IndexInterval::UncheckedClosed(3, 15),
-                          &output_offset, &output_stride));
+  EXPECT_EQ(absl::OkStatus(), ReplaceZeroRankIndexArrayIndexMap(
+                                  10, IndexInterval::UncheckedClosed(3, 15),
+                                  &output_offset, &output_stride));
   EXPECT_EQ(5 + 10 * 3, output_offset);
   EXPECT_EQ(0, output_stride);
 }
@@ -142,17 +140,18 @@ TEST(ReplaceZeroRankIndexArrayIndexMapTest, OutOfBounds) {
                   10, IndexInterval::UncheckedClosed(11, 15), &output_offset,
                   &output_stride),
               MatchesStatus(absl::StatusCode::kOutOfRange,
-                            "Index 10 is outside valid range \\[11, 16\\)\\."));
+                            "Index 10 is outside valid range \\[11, 16\\)"));
 }
 
 TEST(ReplaceZeroRankIndexArrayIndexMapTest, OverflowOffset) {
   Index output_offset = std::numeric_limits<Index>::max(), output_stride = 3;
-  EXPECT_THAT(ReplaceZeroRankIndexArrayIndexMap(
-                  10, IndexInterval::UncheckedClosed(5, 15), &output_offset,
-                  &output_stride),
-              MatchesStatus(
-                  absl::StatusCode::kInvalidArgument,
-                  "Integer overflow computing offset for output dimension\\."));
+  EXPECT_THAT(
+      ReplaceZeroRankIndexArrayIndexMap(10,
+                                        IndexInterval::UncheckedClosed(5, 15),
+                                        &output_offset, &output_stride),
+      MatchesStatus(
+          absl::StatusCode::kInvalidArgument,
+          ".*Integer overflow computing offset for output dimension.*"));
 }
 
 TEST(ReplaceZeroRankIndexArrayIndexMapTest, OverflowStride) {
@@ -162,7 +161,7 @@ TEST(ReplaceZeroRankIndexArrayIndexMapTest, OverflowStride) {
                                         &output_offset, &output_stride),
       MatchesStatus(
           absl::StatusCode::kInvalidArgument,
-          "Integer overflow computing offset for output dimension\\."));
+          ".*Integer overflow computing offset for output dimension.*"));
 }
 
 TEST(Allocate, Basic) {
@@ -180,7 +179,6 @@ TEST(Allocate, Basic) {
 
 TEST(CopyTransformRep, Basic) {
   auto source = TransformRep::Allocate(1, 2);
-  auto dest = TransformRep::Allocate(1, 2);
   source->input_rank = 1;
   source->output_rank = 2;
   source->input_origin()[0] = 5;
@@ -193,7 +191,10 @@ TEST(CopyTransformRep, Basic) {
   source_index_array_data.element_pointer = index_array_ptr;
   source_index_array_data.byte_strides[0] = 0;
   source->input_labels()[0] = "source";
+  // TODO:
+  // tensorstore::internal_index_space::DebugCheckInvariants(source.get());
 
+  auto dest = TransformRep::Allocate(1, 2);
   dest->input_rank = 0;
   dest->output_rank = 0;
   dest->input_origin()[0] = 6;
@@ -231,13 +232,17 @@ TEST(CopyTransformRep, Basic) {
 }
 
 TEST(MoveTransformRep, Basic) {
+  using ::tensorstore::DimensionSet;
+
   auto source = TransformRep::Allocate(1, 2);
-  auto dest = TransformRep::Allocate(1, 2);
   source->input_rank = 1;
   source->output_rank = 2;
+  source->implicit_lower_bounds = DimensionSet::UpTo(source->input_rank);
+  source->implicit_upper_bounds = DimensionSet::UpTo(source->input_rank);
   source->input_origin()[0] = 5;
   source->input_shape()[0] = 2;
   auto& source_map = source->output_index_maps()[0];
+  source_map.SetSingleInputDimension(0);
   source_map.offset() = 3;
   source_map.stride() = 4;
   auto index_array_ptr = std::make_shared<Index>();
@@ -245,7 +250,10 @@ TEST(MoveTransformRep, Basic) {
   source_index_array_data.element_pointer = index_array_ptr;
   source_index_array_data.byte_strides[0] = 0;
   source->input_labels()[0] = "source";
+  // TODO:
+  // tensorstore::internal_index_space::DebugCheckInvariants(source.get());
 
+  auto dest = TransformRep::Allocate(1, 2);
   dest->input_rank = 0;
   dest->output_rank = 0;
   dest->input_origin()[0] = 6;
@@ -254,6 +262,7 @@ TEST(MoveTransformRep, Basic) {
   auto& dest_map = dest->output_index_maps()[0];
   dest_map.offset() = 10;
   dest_map.stride() = 11;
+
   MoveTransformRep(source.get(), dest.get());
 
   EXPECT_EQ(5, source->input_origin()[0]);
@@ -315,10 +324,22 @@ TEST(MutableRepTest, Basic) {
 }
 
 TEST(MutableRepTest, Concurrent) {
+  // MutablePtr checks invariants, so create legal pointers.
+  auto orig = IndexTransformBuilder<>(1, 1)
+                  .input_origin({1})
+                  .input_shape({2})
+                  .input_labels({"a"})
+                  .implicit_lower_bounds({0})
+                  .implicit_upper_bounds({0})
+                  .output_constant(0, 5)
+                  .Finalize()
+                  .value();
+
   TransformRep* orig_ptr;
-  TransformRep::Ptr<> write_ptr = TransformRep::Allocate(1, 0);
+  TransformRep::Ptr<> write_ptr = TransformAccess::rep_ptr(orig);
   write_ptr->output_rank = 0;
   TransformRep::Ptr<> read_ptr;
+
   [[maybe_unused]] std::size_t num_reads_before_write = 0;
   const std::size_t num_iterations = 1000;
   TestConcurrent(
@@ -436,27 +457,10 @@ TEST(ValidateAndIntersectBoundsTest, Failure) {
       });
   EXPECT_THAT(
       status,
-      MatchesStatus(absl::StatusCode::kOutOfRange,
-                    "Propagated bounds \\[5, \\+inf\\) for dimension 1 are "
-                    "incompatible with existing bounds \\[4, 7\\)\\."));
-}
-
-TEST(ValidateLabelsAreUniqueTest, Basic) {
-  EXPECT_EQ(Status(),
-            ValidateLabelsAreUnique(std::vector<std::string>{"a", "b", "c"}));
-  EXPECT_EQ(Status(),
-            ValidateLabelsAreUnique(std::vector<std::string>{"", "", ""}));
-  EXPECT_EQ(Status(), ValidateLabelsAreUnique(
-                          std::vector<std::string>{"a", "b", "", "d", ""}));
-  EXPECT_EQ(Status(), ValidateLabelsAreUnique(std::vector<std::string>{}));
-  EXPECT_THAT(
-      ValidateLabelsAreUnique(std::vector<std::string>{"a", "b", "c", "a"}),
-      MatchesStatus(absl::StatusCode::kInvalidArgument,
-                    "Dimension label \"a\" is not unique"));
-  EXPECT_THAT(
-      ValidateLabelsAreUnique(std::vector<std::string>{"a", "b", "c", "b"}),
-      MatchesStatus(absl::StatusCode::kInvalidArgument,
-                    "Dimension label \"b\" is not unique"));
+      MatchesStatus(
+          absl::StatusCode::kOutOfRange,
+          ".*Propagated bounds are incompatible with existing bounds in "
+          "dimension 1 bounds .* vs. propagated bounds.*"));
 }
 
 }  // namespace
