@@ -17,35 +17,37 @@
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
 #include "tensorstore/index_space/dim_expression.h"
+#include "tensorstore/index_space/index_domain_builder.h"
 #include "tensorstore/index_space/index_transform_builder.h"
 #include "tensorstore/index_space/internal/dim_expression_testutil.h"
 #include "tensorstore/util/status.h"
+#include "tensorstore/util/status_testutil.h"
 
 namespace {
 
-using tensorstore::AllDims;
-using tensorstore::DimensionIndex;
-using tensorstore::Dims;
-using tensorstore::Index;
-using tensorstore::IndexTransformBuilder;
-using tensorstore::kImplicit;
-using tensorstore::kInfIndex;
-using tensorstore::kInfSize;
-using tensorstore::kMaxFiniteIndex;
-using tensorstore::kMinFiniteIndex;
-using tensorstore::MakeArray;
-using tensorstore::span;
-using tensorstore::internal_index_space::EquivalentIndices;
-using tensorstore::internal_index_space::TestDimExpression;
+using ::tensorstore::AllDims;
+using ::tensorstore::DimensionIndex;
+using ::tensorstore::Dims;
+using ::tensorstore::Index;
+using ::tensorstore::IndexDomainBuilder;
+using ::tensorstore::IndexTransformBuilder;
+using ::tensorstore::kImplicit;
+using ::tensorstore::kInfIndex;
+using ::tensorstore::kInfSize;
+using ::tensorstore::kMaxFiniteIndex;
+using ::tensorstore::kMinFiniteIndex;
+using ::tensorstore::MakeArray;
+using ::tensorstore::MatchesStatus;
+using ::tensorstore::span;
+using ::tensorstore::internal_index_space::EquivalentIndices;
+using ::tensorstore::internal_index_space::TestDimExpression;
 
 TEST(TranslateByTest, Example) {
   const auto original_transform = IndexTransformBuilder<3, 3>()
                                       .input_origin({1, 2, 3})
                                       .input_shape({3, 4, 2})
                                       .input_labels({"x", "y", "z"})
-                                      .output_single_input_dimension(0, 0, 1, 0)
-                                      .output_single_input_dimension(1, 0, 1, 1)
-                                      .output_single_input_dimension(2, 0, 1, 2)
+                                      .output_identity_transform()
                                       .Finalize()
                                       .value();
   const auto expected_new_transform =
@@ -54,7 +56,7 @@ TEST(TranslateByTest, Example) {
           .input_shape({3, 4, 2})
           .input_labels({"x", "y", "z"})
           .output_single_input_dimension(0, -10, 1, 0)
-          .output_single_input_dimension(1, 0, 1, 1)
+          .output_single_input_dimension(1, 1)
           .output_single_input_dimension(2, -20, 1, 2)
           .Finalize()
           .value();
@@ -77,14 +79,49 @@ TEST(TranslateByTest, Example) {
                     /*equivalent_indices=*/equivalent_indices);
 }
 
+TEST(TranslateBackwardByTest, Example) {
+  const auto original_transform = IndexTransformBuilder<3, 3>()
+                                      .input_origin({1, 2, 3})
+                                      .input_shape({3, 4, 2})
+                                      .input_labels({"x", "y", "z"})
+                                      .output_identity_transform()
+                                      .Finalize()
+                                      .value();
+  const auto expected_new_transform =
+      IndexTransformBuilder<3, 3>()
+          .input_origin({-9, 2, -17})
+          .input_shape({3, 4, 2})
+          .input_labels({"x", "y", "z"})
+          .output_single_input_dimension(0, 10, 1, 0)
+          .output_single_input_dimension(1, 1)
+          .output_single_input_dimension(2, 20, 1, 2)
+          .Finalize()
+          .value();
+  const EquivalentIndices equivalent_indices = {
+      {{2, 3, 3}, {-8, 3, -17}},
+  };
+  TestDimExpression(/*original_transform=*/original_transform,
+                    /*expression=*/Dims(0, 2).TranslateBackwardBy({10, 20}),
+                    /*expected_new_dimension_selection=*/{0, 2},
+                    /*expected_identity_new_transform=*/expected_new_transform,
+                    /*expected_new_transform=*/expected_new_transform,
+                    /*equivalent_indices=*/equivalent_indices);
+
+  // Test using labels to select dimensions.
+  TestDimExpression(/*original_transform=*/original_transform,
+                    /*expression=*/Dims("x", "z").TranslateBackwardBy({10, 20}),
+                    /*expected_new_dimension_selection=*/{0, 2},
+                    /*expected_identity_new_transform=*/expected_new_transform,
+                    /*expected_new_transform=*/expected_new_transform,
+                    /*equivalent_indices=*/equivalent_indices);
+}
+
 TEST(TranslateToTest, Example) {
   const auto original_transform = IndexTransformBuilder<3, 3>()
                                       .input_origin({1, 2, 3})
                                       .input_shape({3, 4, 2})
                                       .input_labels({"x", "y", "z"})
-                                      .output_single_input_dimension(0, 0, 1, 0)
-                                      .output_single_input_dimension(1, 0, 1, 1)
-                                      .output_single_input_dimension(2, 0, 1, 2)
+                                      .output_identity_transform()
                                       .Finalize()
                                       .value();
   const auto expected_new_transform =
@@ -93,7 +130,7 @@ TEST(TranslateToTest, Example) {
           .input_shape({3, 4, 2})
           .input_labels({"x", "y", "z"})
           .output_single_input_dimension(0, -9, 1, 0)
-          .output_single_input_dimension(1, 0, 1, 1)
+          .output_single_input_dimension(1, 1)
           .output_single_input_dimension(2, -17, 1, 2)
           .Finalize()
           .value();
@@ -168,7 +205,7 @@ TEST(TranslateByTest, OneDimensionalSingleInputDimensionImplicit) {
                     /*expected_new_dimension_selection=*/{0},
                     /*expected_identity_new_transform=*/
                     IndexTransformBuilder<1, 1>()
-                        .output_single_input_dimension(0, 0)
+                        .output_identity_transform()
                         .Finalize()
                         .value(),
                     /*expected_new_transform=*/
@@ -249,7 +286,7 @@ TEST(TranslateByTest, ErrorHandling) {
       AllDims().TranslateBy(span<const Index>({1, 2})),
       absl::StatusCode::kInvalidArgument,
       "Number of dimensions \\(1\\) does not match number of "
-      "indices \\(2\\)\\.");
+      "indices \\(2\\)");
 
   TestDimExpressionError(IndexTransformBuilder<1, 1>()
                              .input_origin({kMinFiniteIndex})
@@ -257,35 +294,26 @@ TEST(TranslateByTest, ErrorHandling) {
                              .Finalize()
                              .value(),
                          AllDims().TranslateBy(-kInfIndex),
-                         absl::StatusCode::kOutOfRange,
-                         "Index offset .* is outside valid range .*");
+                         absl::StatusCode::kInvalidArgument,
+                         ".* is outside valid range .*");
 
   TestDimExpressionError(IndexTransformBuilder<1, 1>()
                              .input_origin({kMinFiniteIndex})
                              .input_shape({10})
                              .Finalize()
                              .value(),
-                         AllDims().TranslateBy(kInfIndex),
-                         absl::StatusCode::kOutOfRange,
-                         "Index offset .* is outside valid range .*");
+                         AllDims().TranslateBy(-1),
+                         absl::StatusCode::kInvalidArgument,
+                         ".* is outside valid range .*");
 
-  TestDimExpressionError(
-      IndexTransformBuilder<1, 1>()
-          .input_origin({kMinFiniteIndex})
-          .input_shape({10})
-          .Finalize()
-          .value(),
-      AllDims().TranslateBy(-1), absl::StatusCode::kInvalidArgument,
-      "Shifted inclusive_min value .* is outside valid range .*");
-
-  TestDimExpressionError(
-      IndexTransformBuilder<1, 1>()
-          .input_origin({kMaxFiniteIndex - 1})
-          .input_shape({2})
-          .Finalize()
-          .value(),
-      AllDims().TranslateBy(1), absl::StatusCode::kInvalidArgument,
-      "Shifted inclusive_max value .* is outside valid range .*");
+  TestDimExpressionError(IndexTransformBuilder<1, 1>()
+                             .input_origin({kMaxFiniteIndex - 1})
+                             .input_shape({2})
+                             .Finalize()
+                             .value(),
+                         AllDims().TranslateBy(1),
+                         absl::StatusCode::kInvalidArgument,
+                         ".* is outside valid range .*");
 
   TestDimExpressionError(IndexTransformBuilder<1, 1>()
                              .output_single_input_dimension(
@@ -312,7 +340,7 @@ TEST(TranslateByTest, DimSubsetUniform) {
                         .input_origin({6, 2, -kInfIndex})
                         .input_shape({4, 5, kInfIndex + 7 + 5})
                         .output_single_input_dimension(0, -5, 1, 0)
-                        .output_single_input_dimension(1, 0, 1, 1)
+                        .output_single_input_dimension(1, 1)
                         .output_single_input_dimension(2, -5, 1, 2)
                         .Finalize()
                         .value(),
@@ -342,7 +370,7 @@ TEST(TranslateByTest, DimSubsetNonUniform) {
                         .input_origin({6, 2, -kInfIndex})
                         .input_shape({4, 5, kInfIndex + 7 + 6})
                         .output_single_input_dimension(0, -5, 1, 0)
-                        .output_single_input_dimension(1, 0, 1, 1)
+                        .output_single_input_dimension(1, 1)
                         .output_single_input_dimension(2, -6, 1, 2)
                         .Finalize()
                         .value(),
@@ -468,7 +496,7 @@ TEST(TranslateToTest, ErrorHandling) {
   TestDimExpressionError(IndexTransformBuilder<1, 1>().Finalize().value(),
                          AllDims().TranslateTo(1),
                          absl::StatusCode::kInvalidArgument,
-                         "Interval \\(-inf, \\+inf\\) is not bounded below\\.");
+                         "Interval \\(-inf, \\+inf\\) is not bounded below");
 
   TestDimExpressionError(
       IndexTransformBuilder<1, 1>()
@@ -477,8 +505,37 @@ TEST(TranslateToTest, ErrorHandling) {
           .Finalize()
           .value(),
       AllDims().TranslateTo(std::numeric_limits<Index>::max()),
-      absl::StatusCode::kOutOfRange,
-      "Origin [0-9]+ is outside valid range .*\\.");
+      absl::StatusCode::kOutOfRange, "Origin [0-9]+ is outside valid range .*");
+}
+
+TEST(TranslateToTest, IndexDomain) {
+  TENSORSTORE_ASSERT_OK_AND_ASSIGN(
+      auto domain,
+      IndexDomainBuilder<3>().origin({1, 2, 3}).shape({6, 7, 8}).Finalize());
+  TENSORSTORE_ASSERT_OK_AND_ASSIGN(
+      auto translated_domain,
+      IndexDomainBuilder<3>().origin({4, 5, 6}).shape({6, 7, 8}).Finalize());
+  EXPECT_THAT(domain | AllDims().TranslateTo({4, 5, 6}),
+              ::testing::Optional(translated_domain));
+}
+
+TEST(TranslateToTest, IndexDomainOverflow) {
+  TENSORSTORE_ASSERT_OK_AND_ASSIGN(
+      auto transform,
+      IndexTransformBuilder(1, 1)
+          .input_shape({10})
+          .output_single_input_dimension(0, kMaxFiniteIndex, kMaxFiniteIndex, 0)
+          .Finalize());
+  auto domain = transform.domain();
+  TENSORSTORE_ASSERT_OK_AND_ASSIGN(
+      auto translated_domain,
+      IndexDomainBuilder(1).origin({-5}).shape({10}).Finalize());
+  EXPECT_THAT(transform | AllDims().TranslateTo({-5}),
+              MatchesStatus(absl::StatusCode::kInvalidArgument));
+  // Tests that integer overflow does not occur, because output dimensions are
+  // ignored.
+  EXPECT_THAT(domain | AllDims().TranslateTo({-5}),
+              ::testing::Optional(translated_domain));
 }
 
 }  // namespace
