@@ -19,6 +19,8 @@
 #include <mutex>  // NOLINT
 #include <utility>
 
+#include "absl/base/optimization.h"
+#include "absl/log/absl_log.h"
 #include "absl/synchronization/mutex.h"
 #include "absl/time/clock.h"
 #include "absl/time/time.h"
@@ -26,15 +28,12 @@
 #include "tensorstore/internal/cache/cache.h"
 #include "tensorstore/internal/intrusive_linked_list.h"
 #include "tensorstore/internal/intrusive_ptr.h"
-#include "tensorstore/internal/logging.h"
 #include "tensorstore/internal/mutex.h"
 #include "tensorstore/internal/no_destructor.h"
-#include "tensorstore/internal/source_location.h"
 #include "tensorstore/internal/type_traits.h"
 #include "tensorstore/kvstore/generation.h"
 #include "tensorstore/transaction.h"
 #include "tensorstore/util/future.h"
-#include "tensorstore/util/quote_string.h"
 #include "tensorstore/util/result.h"
 #include "tensorstore/util/status.h"
 
@@ -103,14 +102,14 @@ void EntryOrNodeStartRead(EntryOrNode& entry_or_node,
                 std::is_same_v<EntryOrNode, TransactionNode>);
   auto& request_state = entry_or_node.read_request_state_;
   if (request_state.queued.null()) {
-    TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(
-        entry_or_node, "EntryOrNodeStartRead: no pending read request");
+    ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG)
+        << entry_or_node << "EntryOrNodeStartRead: no pending read request";
     return;
   }
   if (!request_state.queued.result_needed()) {
-    TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(
-        entry_or_node,
-        "EntryOrNodeStartRead: pending read request was cancelled");
+    ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG)
+        << entry_or_node
+        << "EntryOrNodeStartRead: pending read request was cancelled";
     request_state.queued = Promise<void>();
     request_state.queued_time = absl::InfinitePast();
     return;
@@ -120,8 +119,8 @@ void EntryOrNodeStartRead(EntryOrNode& entry_or_node,
   request_state.issued = std::move(request_state.queued);
   lock.unlock();
   AcquireReadRequestReference(entry_or_node);
-  TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(entry_or_node,
-                                    "EntryOrNodeStartRead: calling DoRead");
+  ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG)
+      << entry_or_node << "EntryOrNodeStartRead: calling DoRead";
   entry_or_node.DoRead(staleness_bound);
 }
 
@@ -136,8 +135,11 @@ void MaybeStartReadOrWriteback(Entry& entry, UniqueWriterLock<Entry> lock) {
           entry.committing_transaction_node_) {
     TransactionNode* next;
     while (true) {
+      // It is safe to use `std::memory_order_relaxed` since writes are already
+      // protected by the entry's mutex.
       const auto existing_prepare_for_commit_state =
-          committing_transaction_node->prepare_for_commit_state_;
+          committing_transaction_node->prepare_for_commit_state_.load(
+              std::memory_order_relaxed);
       const bool read_request_issued = !read_request_state.issued.null();
       // Determine the transitions we will make on `prepare_for_commit_state_`.
       // We need to update `prepare_for_commit_state_` while holding the `lock`,
@@ -167,21 +169,21 @@ void MaybeStartReadOrWriteback(Entry& entry, UniqueWriterLock<Entry> lock) {
           PendingWritebackQueueAccessor::GetNext(committing_transaction_node);
       if (next == committing_transaction_node ||
           next->transaction() != committing_transaction_node->transaction() ||
-          next->prepare_for_commit_state_ ==
+          next->prepare_for_commit_state_.load(std::memory_order_relaxed) ==
               PrepareForCommitState::kReadyForCommitCalled) {
         next = nullptr;
       }
       lock.unlock();
       switch (existing_prepare_for_commit_state) {
         case PrepareForCommitState::kNone:
-          TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(*committing_transaction_node,
-                                            "PrepareDone");
+          ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG)
+              << *committing_transaction_node << "PrepareDone";
           committing_transaction_node->PrepareDone();
           [[fallthrough]];
         case PrepareForCommitState::kPrepareDoneCalled:
           if (read_request_issued) return;
-          TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(*committing_transaction_node,
-                                            "ReadyForCommit");
+          ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG)
+              << *committing_transaction_node << "ReadyForCommit";
           committing_transaction_node->ReadyForCommit();
           break;
         case PrepareForCommitState::kReadyForCommitCalled:
@@ -214,7 +216,8 @@ void SetReadState(EntryOrNode& entry_or_node, ReadState&& read_state,
                 std::is_same_v<EntryOrNode, TransactionNode>);
   if constexpr (std::is_same_v<EntryOrNode, TransactionNode>) {
     if (entry_or_node.reads_committed_) {
-      assert(entry_or_node.prepare_for_commit_state_ ==
+      assert(entry_or_node.prepare_for_commit_state_.load(
+                 std::memory_order_relaxed) ==
              PrepareForCommitState::kReadyForCommitCalled);
       SetReadState(GetOwningEntry(entry_or_node), std::move(read_state),
                    read_state_size);
@@ -375,10 +378,12 @@ class TransactionNodeDestroyer {
       entry.flags_ |= Entry::kSizeChanged;
       entry.write_state_size_ -=
           (node.write_state_size_ + node.read_request_state_.read_state_size);
-      if (entry.implicit_transaction_node_.tag()) {
+      if (entry.implicit_transaction_node_
+              .tag<AsyncCache::Entry::kImplicitTransactionCommitBlock>()) {
         commit_block_to_release_.reset(
             entry.implicit_transaction_node_->transaction());
-        entry.implicit_transaction_node_.set_tag(0);
+        entry.implicit_transaction_node_
+            .set_tag<AsyncCache::Entry::kImplicitTransactionCommitBlock>(0);
         assert(entry.num_implicit_transactions_ <= 2);
       }
       if (entry.implicit_transaction_node_.get() == &node) {
@@ -407,7 +412,7 @@ void ResolveIssuedWriteback(AsyncCache::TransactionNode& node,
   auto& entry = GetOwningEntry(node);
 
   // Writeback of this transaction must have been in progress.
-  assert(node.prepare_for_commit_state_ ==
+  assert(node.prepare_for_commit_state_.load(std::memory_order_relaxed) ==
          PrepareForCommitState::kReadyForCommitCalled);
   assert(entry.committing_transaction_node_ &&
          entry.committing_transaction_node_->transaction() ==
@@ -453,8 +458,8 @@ void AsyncCache::Entry::WriterUnlock() {
   update.lock = std::move(lock);
   if (flags & kSizeChanged) {
     update.new_size = GetTotalSize(*this);
-    TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(
-        *this, "Entry::WriterUnlock: new_size=", *update.new_size);
+    ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG)
+        << *this << "Entry::WriterUnlock: new_size=" << *update.new_size;
   }
   if (flags & (kStateChanged | kMarkWritebackRequested)) {
     if (num_implicit_transactions_ == 0) {
@@ -482,28 +487,32 @@ size_t AsyncCache::DoGetSizeInBytes(Cache::Entry* base_entry) {
 }
 
 Future<const void> AsyncCache::Entry::Read(absl::Time staleness_bound) {
-  TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(*this,
-                                    "Read: staleness_bound=", staleness_bound);
+  ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG)
+      << *this << "Read: staleness_bound=" << staleness_bound;
   return RequestRead(*this, staleness_bound);
 }
 
 void AsyncCache::Entry::ReadSuccess(ReadState&& read_state) {
-  TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(*this, "ReadSuccess: ", read_state.stamp);
+  ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG)
+      << *this << "ReadSuccess: " << read_state.stamp;
   internal::EntryOrNodeReadSuccess(*this, std::move(read_state));
 }
 
 void AsyncCache::Entry::ReadError(absl::Status error) {
-  TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(*this, "ReadError: error=", error);
+  ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG)
+      << *this << "ReadError: error=" << error;
   internal::EntryOrNodeReadError(*this, std::move(error));
 }
 
 void AsyncCache::DoRequestWriteback(PinnedEntry base_entry) {
   auto& entry = static_cast<Entry&>(*base_entry);
-  TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(entry, "DoRequestWriteack");
+  ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG)
+      << entry << "DoRequestWriteack";
   WeakTransactionNodePtr<TransactionNode> implicit_transaction_node;
   {
     UniqueWriterLock lock(entry);
-    if (entry.implicit_transaction_node_) {
+    if (entry.implicit_transaction_node_
+            .tag<Entry::kImplicitTransactionInitialized>()) {
       implicit_transaction_node.reset(entry.implicit_transaction_node_.get());
     } else {
       return;
@@ -519,27 +528,31 @@ AsyncCache::TransactionNode::TransactionNode(Entry& entry)
 
 Future<const void> AsyncCache::TransactionNode::Read(
     absl::Time staleness_bound) {
-  TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(*this,
-                                    "Read: staleness_bound=", staleness_bound);
-  if (reads_committed_ && (prepare_for_commit_state_ !=
-                           PrepareForCommitState::kReadyForCommitCalled)) {
+  ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG)
+      << *this << "Read: staleness_bound=" << staleness_bound;
+  if (reads_committed_ &&
+      (prepare_for_commit_state_.load(std::memory_order_acquire) !=
+       PrepareForCommitState::kReadyForCommitCalled)) {
     return RequestRead(GetOwningEntry(*this), staleness_bound);
   }
   return RequestRead(*this, staleness_bound);
 }
 
 void AsyncCache::TransactionNode::ReadSuccess(ReadState&& read_state) {
-  TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(*this, "ReadSuccess: ", read_state.stamp);
+  ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG)
+      << *this << "ReadSuccess: " << read_state.stamp;
   internal::EntryOrNodeReadSuccess(*this, std::move(read_state));
 }
 
 void AsyncCache::TransactionNode::ReadError(absl::Status error) {
-  TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(*this, "ReadError: error=", error);
+  ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG)
+      << *this << "ReadError: error=" << error;
   internal::EntryOrNodeReadError(*this, std::move(error));
 }
 
 void AsyncCache::TransactionNode::PrepareForCommit() {
-  TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(*this, "PrepareForCommit");
+  ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG)
+      << *this << "PrepareForCommit";
   // Acquire reference to be released by `Commit`.
   intrusive_ptr_increment(this);
   auto& entry = GetOwningEntry(*this);
@@ -549,7 +562,8 @@ void AsyncCache::TransactionNode::PrepareForCommit() {
   }
   if (this == entry.implicit_transaction_node_.get()) {
     // Commit block must have been released.
-    assert(entry.implicit_transaction_node_.tag() == 0);
+    assert(entry.implicit_transaction_node_
+               .tag<Entry::kImplicitTransactionCommitBlock>() == 0);
     entry.implicit_transaction_node_ = nullptr;
     entry.flags_ |= Entry::kMarkWritebackRequested;
   }
@@ -560,14 +574,15 @@ void AsyncCache::TransactionNode::PrepareForCommit() {
                                         entry.committing_transaction_node_,
                                         this);
     if (entry.committing_transaction_node_->transaction() != transaction()) {
-      TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(*this,
-                                        "Commit: enqueuing for writeback");
+      ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG)
+          << *this << "Commit: enqueuing for writeback";
       return;
     }
     // PrepareDone on the prior node in this same transaction must have been
     // called already, as otherwise `PrepareForCommit` would not have been
     // invoked on this node yet.
-    assert(entry.committing_transaction_node_->prepare_for_commit_state_ >=
+    assert(entry.committing_transaction_node_->prepare_for_commit_state_.load(
+               std::memory_order_relaxed) >=
            PrepareForCommitState::kPrepareDoneCalled);
   } else {
     // No node is already being committed.
@@ -580,7 +595,7 @@ void AsyncCache::TransactionNode::PrepareForCommit() {
 }
 
 void AsyncCache::TransactionNode::Abort() {
-  TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(*this, "Abort");
+  ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG) << *this << "Abort";
   auto& entry = GetOwningEntry(*this);
   UniqueWriterLock lock(entry);
   TransactionNodeDestroyer destroyer(*this);
@@ -589,8 +604,8 @@ void AsyncCache::TransactionNode::Abort() {
 }
 
 void AsyncCache::TransactionNode::WritebackSuccess(ReadState&& read_state) {
-  TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(*this,
-                                    "WritebackSuccess: ", read_state.stamp);
+  ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG)
+      << *this << "WritebackSuccess: " << read_state.stamp;
   auto& entry = GetOwningEntry(*this);
   const size_t read_state_size = GetReadStateSize(entry, read_state.data.get());
   UniqueWriterLock lock{entry};
@@ -611,7 +626,7 @@ void AsyncCache::TransactionNode::WritebackSuccess(ReadState&& read_state) {
 }
 
 void AsyncCache::TransactionNode::WritebackError() {
-  TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(*this, "WritebackError");
+  ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG) << *this << "WritebackError";
   ResolveIssuedWriteback(*this, UniqueWriterLock{GetOwningEntry(*this)});
 }
 
@@ -633,9 +648,9 @@ AsyncCache::Entry::GetTransactionNodeImpl(OpenTransactionPtr& transaction) {
           node.SetTransaction(GetOrCreateOpenTransaction(transaction));
         }
         assert(node.transaction() == transaction.get());
-        TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(
-            node, "New node, implicit=", implicit_transaction,
-            ", transaction=", transaction.get());
+        ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG)
+            << node << "New node, implicit=" << implicit_transaction
+            << ", transaction=" << transaction.get();
         node.initialized_status_ = node.Register();
       }
       if (!node.initialized_status_.ok()) {
@@ -651,14 +666,17 @@ AsyncCache::Entry::GetTransactionNodeImpl(OpenTransactionPtr& transaction) {
       } else if (implicit_transaction) {
         UniqueWriterLock lock(entry);
         if (entry.ShareImplicitTransactionNodes()) {
+          assert(entry.implicit_transaction_node_.get() == &node);
+          entry.implicit_transaction_node_
+              .set_tag<kImplicitTransactionInitialized>(1);
           if (++entry.num_implicit_transactions_ != 1) {
             // Prevent this new implicit transaction from being committed until
             // the existing implicit transaction has been committed, in order to
             // avoid a build-up of implicit transactions, which would waste
             // memory.
             node.transaction()->AcquireCommitBlock();
-            assert(entry.implicit_transaction_node_.get() == &node);
-            entry.implicit_transaction_node_.set_tag(1);
+            entry.implicit_transaction_node_
+                .set_tag<kImplicitTransactionCommitBlock>(1);
             assert(entry.num_implicit_transactions_ == 2);
           }
           entry.flags_ |= Entry::kStateChanged;
@@ -683,7 +701,8 @@ AsyncCache::Entry::GetTransactionNodeImpl(OpenTransactionPtr& transaction) {
           // Allocate new implicit transaction node if there is not already one.
           if (!implicit_transaction_node_ ||
               implicit_transaction_node_.get() == stale_node.get()) {
-            if (implicit_transaction_node_.tag()) {
+            if (implicit_transaction_node_
+                    .tag<kImplicitTransactionCommitBlock>()) {
               release_commit_block = true;
             }
             implicit_transaction_node_ =
@@ -772,7 +791,7 @@ void AsyncCache::TransactionNode::Commit() { intrusive_ptr_decrement(this); }
 void AsyncCache::TransactionNode::WriterLock() { mutex_.WriterLock(); }
 
 void AsyncCache::TransactionNode::WriterUnlock() {
-  TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(*this, "unlock");
+  ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG) << *this << "unlock";
   UniqueWriterLock lock(mutex_, std::adopt_lock);
   if (!size_updated_) return;
   size_updated_ = false;
@@ -804,29 +823,31 @@ absl::Status AsyncCache::TransactionNode::DoInitialize(
 
 void AsyncCache::TransactionNode::DoApply(ApplyOptions options,
                                           ApplyReceiver receiver) {
-  TENSORSTORE_UNREACHABLE;
+  ABSL_UNREACHABLE();  // COV_NF_LINE
 }
 
 void AsyncCache::TransactionNode::Revoke() {
-  TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(*this, "Revoke");
+  ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG) << *this << "Revoke";
   revoked_.store(true, std::memory_order_release);
 }
 
 void AsyncCache::TransactionNode::InvalidateReadState() {
   assert(this->transaction()->commit_started());
-  TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(*this, "InvalidateReadState");
+  ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG)
+      << *this << "InvalidateReadState";
   this->read_request_state_.read_state = ReadState{};
 }
 
 AsyncCache::TransactionNode::~TransactionNode() {
-  TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(*this, "~TransactionNode");
+  ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG)
+      << *this << "~TransactionNode";
   Cache::PinnedEntry(static_cast<Cache::Entry*>(associated_data()),
                      adopt_object_ref);
 }
 
 #ifdef TENSORSTORE_ASYNC_CACHE_DEBUG
 AsyncCache::Entry::~Entry() {
-  TENSORSTORE_ASYNC_CACHE_DEBUG_LOG(*this, "~Entry");
+  ABSL_LOG_IF(INFO, TENSORSTORE_ASYNC_CACHE_DEBUG) << *this << "~Entry";
 }
 #endif
 
