@@ -57,24 +57,18 @@
 #include "tensorstore/driver/chunk.h"
 #include "tensorstore/driver/driver.h"
 #include "tensorstore/index.h"
-#include "tensorstore/index_interval.h"
 #include "tensorstore/index_space/index_transform.h"
-#include "tensorstore/index_space/transformed_array.h"
 #include "tensorstore/internal/async_write_array.h"
 #include "tensorstore/internal/cache/async_cache.h"
 #include "tensorstore/internal/cache/cache.h"
 #include "tensorstore/internal/intrusive_ptr.h"
-#include "tensorstore/internal/mutex.h"
 #include "tensorstore/staleness_bound.h"
 #include "tensorstore/transaction.h"
-#include "tensorstore/util/assert_macros.h"
 #include "tensorstore/util/execution/any_receiver.h"
 #include "tensorstore/util/execution/sender.h"
 #include "tensorstore/util/future.h"
-#include "tensorstore/util/iterate.h"
 #include "tensorstore/util/result.h"
 #include "tensorstore/util/span.h"
-#include "tensorstore/util/status.h"
 
 namespace tensorstore {
 namespace internal {
