@@ -27,6 +27,7 @@
 #include "absl/base/attributes.h"
 #include "absl/container/flat_hash_map.h"
 #include "absl/container/inlined_vector.h"
+#include "absl/log/absl_check.h"
 #include "absl/strings/numbers.h"
 #include "absl/strings/str_join.h"
 #include "absl/strings/str_split.h"
@@ -146,9 +147,8 @@ absl::Cord EncodeRaw(const ChunkGridSpecification& grid,
        ++component_i) {
     const auto& spec = component_specs[component_i];
     auto array = MakeCopy(component_arrays[component_i]);
-    TENSORSTORE_CHECK(
-        tensorstore::internal::RangesEqual(array.shape(), spec.shape()));
-    TENSORSTORE_CHECK(array.dtype() == spec.dtype());
+    ABSL_CHECK(tensorstore::internal::RangesEqual(array.shape(), spec.shape()));
+    ABSL_CHECK(array.dtype() == spec.dtype());
     const size_t num_bytes = spec.num_elements() * spec.dtype().size();
     value.append(reinterpret_cast<const char*>(array.data()), num_bytes);
   }
@@ -278,7 +278,7 @@ std::vector<Index> ParseKey(std::string_view key) {
   std::vector<Index> result;
   for (auto s : absl::StrSplit(key, ',')) {
     Index i = 0;
-    TENSORSTORE_CHECK(absl::SimpleAtoi(s, &i));
+    ABSL_CHECK(absl::SimpleAtoi(s, &i));
     result.push_back(i);
   }
   return result;
@@ -726,27 +726,21 @@ TEST_F(ChunkCacheTest, WriteSingleComponentOneDimensional) {
     EXPECT_EQ(StorageGeneration::Unknown(), r.options.if_equal);
     r(memory_store);
   }
-  {
+  std::vector<std::pair<std::vector<Index>, StorageGeneration>> write_requests;
+  for (size_t i = 0; i < 3; ++i) {
     auto r = mock_store->write_requests.pop();
-    EXPECT_THAT(ParseKey(r.key), ElementsAre(2));
-    EXPECT_EQ(StorageGeneration::Unknown(), r.options.if_equal);
-    r(memory_store);
-  }
-  EXPECT_THAT(GetChunk({2}), ElementsAre(MakeArray<int>({4, 5})));
-  {
-    auto r = mock_store->write_requests.pop();
-    EXPECT_THAT(ParseKey(r.key), ElementsAre(3));
-    EXPECT_EQ(StorageGeneration::NoValue(), r.options.if_equal);
-    r(memory_store);
-  }
-  EXPECT_THAT(GetChunk({3}), ElementsAre(MakeArray<int>({6, 2})));
-  {
-    auto r = mock_store->write_requests.pop();
-    EXPECT_THAT(ParseKey(r.key), ElementsAre(1));
-    EXPECT_EQ(StorageGeneration::NoValue(), r.options.if_equal);
+    write_requests.emplace_back(ParseKey(r.key), r.options.if_equal);
     r(memory_store);
   }
+  EXPECT_THAT(
+      write_requests,
+      ::testing::UnorderedElementsAre(
+          ::testing::Pair(ElementsAre(2), StorageGeneration::Unknown()),
+          ::testing::Pair(ElementsAre(3), StorageGeneration::NoValue()),
+          ::testing::Pair(ElementsAre(1), StorageGeneration::NoValue())));
   EXPECT_THAT(GetChunk({1}), ElementsAre(MakeArray<int>({1, 3})));
+  EXPECT_THAT(GetChunk({2}), ElementsAre(MakeArray<int>({4, 5})));
+  EXPECT_THAT(GetChunk({3}), ElementsAre(MakeArray<int>({6, 2})));
   TENSORSTORE_EXPECT_OK(write_future);
 }
 
