@@ -28,6 +28,8 @@
 
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
+#include "absl/base/optimization.h"
+#include "absl/log/absl_log.h"
 #include "absl/random/random.h"
 #include "absl/status/status.h"
 #include "absl/strings/cord.h"
@@ -38,7 +40,6 @@
 #include "tensorstore/internal/cache/cache.h"
 #include "tensorstore/internal/cache/kvs_backed_cache.h"
 #include "tensorstore/internal/intrusive_ptr.h"
-#include "tensorstore/internal/logging.h"
 #include "tensorstore/internal/mutex.h"
 #include "tensorstore/internal/source_location.h"
 #include "tensorstore/internal/test_util.h"
@@ -49,9 +50,7 @@
 #include "tensorstore/kvstore/test_util.h"
 #include "tensorstore/transaction.h"
 #include "tensorstore/transaction_impl.h"
-#include "tensorstore/util/assert_macros.h"
 #include "tensorstore/util/execution/execution.h"
-#include "tensorstore/util/execution/sender.h"
 #include "tensorstore/util/future.h"
 #include "tensorstore/util/quote_string.h"
 #include "tensorstore/util/result.h"
@@ -115,7 +114,7 @@ Future<absl::Cord> KvsBackedTestCache::Entry::ReadValue(
   struct Receiver {
     Promise<absl::Cord> promise_;
 
-    void set_cancel() { TENSORSTORE_UNREACHABLE; }
+    void set_cancel() { ABSL_UNREACHABLE(); }  // COV_NF_LINE
     void set_error(absl::Status status) { promise_.SetResult(status); }
     void set_value(AsyncCache::ReadState update) {
       promise_.SetResult(*static_cast<const ReadData*>(update.data.get()));
@@ -236,7 +235,7 @@ void KvsRandomOperationTester::PerformRandomAction(
   if (barrier_probability > 0 && absl::Bernoulli(gen, barrier_probability)) {
     transaction->Barrier();
     if (log) {
-      TENSORSTORE_LOG("Barrier");
+      ABSL_LOG(INFO) << "Barrier";
     }
   }
   if (absl::Bernoulli(gen, write_probability)) {
@@ -246,16 +245,16 @@ void KvsRandomOperationTester::PerformRandomAction(
     std::string append = tensorstore::StrCat(", ", ++write_number);
     SimulateWrite(key, clear, append);
     if (log) {
-      TENSORSTORE_LOG("Write: key=", QuoteString(key),
-                      ", cache_key=", cache->cache_identifier(),
-                      ", clear=", clear, ", append=\"", append, "\"");
+      ABSL_LOG(INFO) << "Write: key=" << QuoteString(key)
+                     << ", cache_key=" << cache->cache_identifier()
+                     << ", clear=" << clear << ", append=\"" << append << "\"";
     }
     TENSORSTORE_EXPECT_OK(
         GetCacheEntry(cache, key)->Modify(transaction, clear, append));
   } else {
     KeyRange range{SampleKeyOrEmpty(), SampleKeyOrEmpty()};
     if (log) {
-      TENSORSTORE_LOG("DeleteRange: ", range);
+      ABSL_LOG(INFO) << "DeleteRange: " << range;
     }
     SimulateDeleteRange(range);
     TENSORSTORE_EXPECT_OK(
@@ -266,7 +265,7 @@ void KvsRandomOperationTester::PerformRandomAction(
 void KvsRandomOperationTester::PerformRandomActions() {
   const size_t num_actions = absl::Uniform(gen, 1u, 100u);
   if (log) {
-    TENSORSTORE_LOG("--PerformRandomActions-- ", num_actions);
+    ABSL_LOG(INFO) << "--PerformRandomActions-- " << num_actions;
   }
 
   auto transaction = Transaction(tensorstore::isolated);
@@ -420,11 +419,12 @@ void RegisterKvsBackedCacheBasicTransactionalTest(
           auto old_read_generation =
               AsyncCache::ReadLock<absl::Cord>(*entry).stamp();
           TENSORSTORE_EXPECT_OK(kvstore->Write(a_key, absl::Cord("Z")));
-          EXPECT_THAT(entry->Read(absl::Now()).result(),
-                      MatchesStatus(
-                          absl::StatusCode::kFailedPrecondition,
-                          StrCat("Error reading ", kvstore->DescribeKey(a_key),
-                                 ": existing value contains Z")));
+          EXPECT_THAT(
+              entry->Read(absl::Now()).result(),
+              MatchesStatus(absl::StatusCode::kFailedPrecondition,
+                            tensorstore::StrCat(
+                                "Error reading ", kvstore->DescribeKey(a_key),
+                                ": existing value contains Z")));
           // Read state is not modified.
           EXPECT_THAT(AsyncCache::ReadLock<absl::Cord>(*entry).data(),
                       Pointee(absl::Cord("ghi")));
@@ -460,8 +460,9 @@ void RegisterKvsBackedCacheBasicTransactionalTest(
         EXPECT_THAT(
             transaction.CommitAsync().result(),
             MatchesStatus(absl::StatusCode::kFailedPrecondition,
-                          StrCat("Error reading ", kvstore->DescribeKey(a_key),
-                                 ": existing value contains Z")));
+                          tensorstore::StrCat("Error reading ",
+                                              kvstore->DescribeKey(a_key),
+                                              ": existing value contains Z")));
         EXPECT_THAT(kvstore->Read(a_key).result(),
                     MatchesKvsReadResult(absl::Cord("Z")));
       },
@@ -554,11 +555,12 @@ void RegisterKvsBackedCacheBasicTransactionalTest(
                     transaction));
             TENSORSTORE_ASSERT_OK(entry->Modify(open_transaction, true, "Z"));
           }
-          EXPECT_THAT(transaction.CommitAsync().result(),
-                      MatchesStatus(
-                          absl::StatusCode::kInvalidArgument,
-                          StrCat("Error writing ", kvstore->DescribeKey(a_key),
-                                 ": new value contains Z")));
+          EXPECT_THAT(
+              transaction.CommitAsync().result(),
+              MatchesStatus(absl::StatusCode::kInvalidArgument,
+                            tensorstore::StrCat("Error writing ",
+                                                kvstore->DescribeKey(a_key),
+                                                ": new value contains Z")));
         }
         EXPECT_THAT(AsyncCache::ReadLock<absl::Cord>(*entry).data(),
                     Pointee(absl::Cord("abc")));
@@ -594,17 +596,19 @@ void RegisterKvsBackedCacheBasicTransactionalTest(
                   GetTransactionNode(*entry_b, open_transaction),
                   MatchesStatus(
                       absl::StatusCode::kInvalidArgument,
-                      StrCat("Cannot read/write ", kvstore->DescribeKey(a_key),
-                             " and read/write ", kvstore->DescribeKey(b_key),
-                             " as single atomic transaction")));
+                      tensorstore::StrCat(
+                          "Cannot read/write ", kvstore->DescribeKey(a_key),
+                          " and read/write ", kvstore->DescribeKey(b_key),
+                          " as single atomic transaction")));
             }
             EXPECT_THAT(
                 transaction.future().result(),
                 MatchesStatus(
                     absl::StatusCode::kInvalidArgument,
-                    StrCat("Cannot read/write ", kvstore->DescribeKey(a_key),
-                           " and read/write ", kvstore->DescribeKey(b_key),
-                           " as single atomic transaction")));
+                    tensorstore::StrCat(
+                        "Cannot read/write ", kvstore->DescribeKey(a_key),
+                        " and read/write ", kvstore->DescribeKey(b_key),
+                        " as single atomic transaction")));
           }
         },
         TENSORSTORE_LOC);
