@@ -14,10 +14,10 @@
 
 #include "tensorstore/internal/compression/lzma.h"
 
+#include "absl/base/optimization.h"
+#include "absl/log/absl_check.h"
 #include "absl/status/status.h"
 #include "tensorstore/internal/compression/cord_stream_manager.h"
-#include "tensorstore/util/assert_macros.h"
-#include "tensorstore/util/status.h"
 
 namespace tensorstore {
 namespace lzma {
@@ -58,8 +58,9 @@ absl::Status GetInitErrorStatus(::lzma_ret r) {
     case LZMA_UNSUPPORTED_CHECK:
     case LZMA_PROG_ERROR:
     default:
-      TENSORSTORE_CHECK(false);
+      ABSL_CHECK(false);
   }
+  ABSL_UNREACHABLE();
 }
 
 absl::Status GetEncodeErrorStatus(::lzma_ret r) {
@@ -71,8 +72,9 @@ absl::Status GetEncodeErrorStatus(::lzma_ret r) {
     case LZMA_MEMLIMIT_ERROR:
     case LZMA_MEM_ERROR:
     default:
-      TENSORSTORE_CHECK(false);
+      ABSL_CHECK(false);
   }
+  ABSL_UNREACHABLE();
 }
 
 absl::Status GetDecodeErrorStatus(::lzma_ret r) {
@@ -89,8 +91,9 @@ absl::Status GetDecodeErrorStatus(::lzma_ret r) {
     case LZMA_MEM_ERROR:
     case LZMA_MEMLIMIT_ERROR:
     default:
-      TENSORSTORE_CHECK(false);
+      ABSL_CHECK(false);
   }
+  ABSL_UNREACHABLE();
 }
 
 namespace xz {
