@@ -118,9 +118,9 @@
 ///
 ///     struct CallWithStatusContext {
 ///       using ElementwiseFunctionType = ElementwiseFunction<1>;
-///       ElementwiseFunction<1, Status*> orig_function;
+///       ElementwiseFunction<1, absl::Status*> orig_function;
 ///       void *orig_context;
-///       Status status;
+///       absl::Status status;
 ///       struct LoopTemplate {
 ///         template <typename ArrayAccessor>
 ///         static Index Loop(void* void_context, Index count,
@@ -137,7 +137,7 @@
 ///
 /// Then `wrapper_function` can be used with a context pointer of
 /// `&wrapper_context` to invoke `orig_function` with an extra pointer to a
-/// `Status`.
+/// `absl::Status`.
 
 #include <array>
 #include <cstddef>
@@ -145,12 +145,12 @@
 #include <type_traits>
 #include <utility>
 
+#include "absl/status/status.h"
 #include "tensorstore/index.h"
 #include "tensorstore/internal/integer_overflow.h"
 #include "tensorstore/internal/type_traits.h"
 #include "tensorstore/internal/void_wrapper.h"
 #include "tensorstore/util/byte_strided_pointer.h"
-#include "tensorstore/util/default_iteration_result.h"
 #include "tensorstore/util/result.h"
 #include "tensorstore/util/status.h"
 
@@ -362,7 +362,10 @@ class ElementwiseFunction {
 
   constexpr ElementwiseFunction() = default;
 
-  template <typename LoopTemplate>
+  template <
+      typename LoopTemplate,
+      typename = decltype(&LoopTemplate::template Loop<IterationBufferAccessor<
+                              IterationBufferKind::kContiguous>>)>
   constexpr explicit ElementwiseFunction(LoopTemplate)
       : functions_{
             &LoopTemplate::template Loop<
@@ -464,20 +467,20 @@ struct SimpleElementwiseFunction;
 ///     MyFunc obj;
 ///     // Implicitly converts to `ElementwiseFunction`.
 ///     ElementwiseFunction<2> func =
-///         SimpleElementwiseFunction<MyFunc, int, float>();
+///         SimpleElementwiseFunction<MyFunc(int, float)>();
 ///
 ///     // Implicitly converts to `const ElementwiseFunction*` (static constant)
 ///     const ElementwiseFunction<2>* func_ptr =
-///         SimpleElementwiseFunction<MyFunc, int, float>();
+///         SimpleElementwiseFunction<MyFunc(int, float)>();
 ///
 ///     // Converts to the closure `{ func_ptr, &obj }`.
 ///     ElementwiseClosure<2> closure =
-///         SimpleElementwiseFunction<MyFunc, int, float>::Closure(&obj);
+///         SimpleElementwiseFunction<MyFunc(int, float)>::Closure(&obj);
 ///
 ///     // Converts to the closure `{ func_ptr, nullptr }` since `MyFunc` is
 ///     // empty.
 ///     ElementwiseClosure<2> stateless_closure =
-///         SimpleElementwiseFunction<MyFunc, int, float>();
+///         SimpleElementwiseFunction<MyFunc(int, float)>();
 ///
 /// \tparam Func The function object type.  Must be callable with
 ///     `(Element*..., ExtraArg...)`, and return either a type explicitly
