@@ -18,12 +18,12 @@
 
 #include <gtest/gtest.h>
 
-using tensorstore::internal::GetEnv;
-using tensorstore::internal::SetEnv;
-using tensorstore::internal::UnsetEnv;
-
 namespace {
 
+using ::tensorstore::internal::GetEnv;
+using ::tensorstore::internal::SetEnv;
+using ::tensorstore::internal::UnsetEnv;
+
 TEST(GetEnvTest, Basic) {
   // Env is set
   SetEnv("TENSORSTORE_TEST_ENV_VAR", "test env var");
