@@ -15,24 +15,24 @@
 #include "tensorstore/internal/grid_partition.h"
 
 #include <algorithm>
+#include <cassert>
 #include <memory>
 #include <optional>
 #include <utility>
 #include <vector>
 
-#include "absl/base/macros.h"
 #include "absl/container/fixed_array.h"
+#include "absl/functional/function_ref.h"
+#include "absl/status/status.h"
 #include "tensorstore/array.h"
 #include "tensorstore/index.h"
 #include "tensorstore/index_interval.h"
 #include "tensorstore/internal/grid_partition_impl.h"
 #include "tensorstore/internal/intrusive_ptr.h"
 #include "tensorstore/internal/memory.h"
-#include "tensorstore/util/bit_span.h"
 #include "tensorstore/util/byte_strided_pointer.h"
 #include "tensorstore/util/division.h"
 #include "tensorstore/util/element_pointer.h"
-#include "tensorstore/util/function_view.h"
 #include "tensorstore/util/iterate.h"
 #include "tensorstore/util/result.h"
 #include "tensorstore/util/span.h"
@@ -43,19 +43,20 @@ namespace internal_grid_partition {
 
 namespace {
 
-using internal_index_space::OutputIndexMap;
-using internal_index_space::TransformAccess;
-using internal_index_space::TransformRep;
+using ::tensorstore::internal_index_space::OutputIndexMap;
+using ::tensorstore::internal_index_space::TransformAccess;
+using ::tensorstore::internal_index_space::TransformRep;
+
 using IndexArraySet = IndexTransformGridPartition::IndexArraySet;
 using StridedSet = IndexTransformGridPartition::StridedSet;
 
 struct ConnectedSetIterateParameters {
   const IndexTransformGridPartition& info;
   span<const DimensionIndex> grid_output_dimensions;
-  span<const Index> grid_cell_shape;
+  OutputToGridCellFn output_to_grid_cell;
   IndexTransformView<> transform;
-  FunctionView<Status(span<const Index> grid_cell_indices,
-                      IndexTransformView<> cell_transform)>
+  absl::FunctionRef<absl::Status(span<const Index> grid_cell_indices,
+                                 IndexTransformView<> cell_transform)>
       func;
 };
 
@@ -81,8 +82,8 @@ internal_index_space::TransformRep::Ptr<> InitializeCellTransform(
       TransformRep::Allocate(cell_input_rank, full_input_rank);
   cell_transform->input_rank = cell_input_rank;
   cell_transform->output_rank = full_input_rank;
-  cell_transform->implicit_lower_bounds(cell_input_rank).fill(false);
-  cell_transform->implicit_upper_bounds(cell_input_rank).fill(false);
+  cell_transform->implicit_lower_bounds = false;
+  cell_transform->implicit_upper_bounds = false;
 
   const span<Index> input_origin =
       cell_transform->input_origin().first(cell_input_rank);
@@ -155,6 +156,7 @@ internal_index_space::TransformRep::Ptr<> InitializeCellTransform(
     ++cell_input_dim;
   }
 
+  // Invariants checked in InvokeCallback
   return cell_transform;
 }
 
@@ -165,7 +167,7 @@ class ConnectedSetIterateHelper {
  public:
   explicit ConnectedSetIterateHelper(ConnectedSetIterateParameters params)
       : params_(std::move(params)),
-        grid_cell_indices_(params_.grid_cell_shape.size()),
+        grid_cell_indices_(params_.grid_output_dimensions.size()),
         cell_transform_(InitializeCellTransform(
             params_.info,
             internal_index_space::TransformAccess::rep(params_.transform))) {
@@ -176,7 +178,7 @@ class ConnectedSetIterateHelper {
   ///
   /// This is implemented by recursively iterating over the partitions of each
   /// connected set.
-  Status Iterate() { return IterateOverIndexArraySets(0); }
+  absl::Status Iterate() { return IterateOverIndexArraySets(0); }
 
  private:
   /// Sets the fixed grid cell indices for all grid dimensions that do not
@@ -185,15 +187,15 @@ class ConnectedSetIterateHelper {
   ///
   /// This function is used only by the constructor.
   void InitializeGridCellIndices() {
-    for (DimensionIndex grid_dim = 0; grid_dim < params_.grid_cell_shape.size();
-         ++grid_dim) {
+    for (DimensionIndex grid_dim = 0;
+         grid_dim < params_.grid_output_dimensions.size(); ++grid_dim) {
       const DimensionIndex output_dim =
           params_.grid_output_dimensions[grid_dim];
       const OutputIndexMapRef<> map =
           params_.transform.output_index_map(output_dim);
       if (map.method() != OutputIndexMethod::constant) continue;
       grid_cell_indices_[grid_dim] =
-          FloorOfRatio(map.offset(), params_.grid_cell_shape[grid_dim]);
+          params_.output_to_grid_cell(grid_dim, map.offset(), nullptr);
     }
   }
 
@@ -213,7 +215,7 @@ class ConnectedSetIterateHelper {
   /// \param set_i The next index array connected set over which to iterate, in
   ///     the range `[0, info.index_array_sets().size()]`.
   /// \returns The return value of the last recursively call.
-  Status IterateOverIndexArraySets(DimensionIndex set_i) {
+  absl::Status IterateOverIndexArraySets(DimensionIndex set_i) {
     if (set_i == params_.info.index_array_sets().size()) {
       return IterateOverStridedSets(0);
     }
@@ -279,7 +281,7 @@ class ConnectedSetIterateHelper {
   ///     range `[0, info.strided_sets().size()]`.
   /// \returns The return value of the last recursive call, or the last call to
   ///     `InvokeCallback`.
-  Status IterateOverStridedSets(DimensionIndex set_i) {
+  absl::Status IterateOverStridedSets(DimensionIndex set_i) {
     if (set_i == params_.info.strided_sets().size()) return InvokeCallback();
     const StridedSet strided_set = params_.info.strided_sets()[set_i];
     const IndexInterval domain =
@@ -298,15 +300,13 @@ class ConnectedSetIterateHelper {
       // index corresponding to `input_index`, and constrain `restricted_domain`
       // to the range of this grid cell.
       for (const DimensionIndex grid_dim : strided_set.grid_dimensions) {
-        const Index cell_size = params_.grid_cell_shape[grid_dim];
         const DimensionIndex output_dim =
             params_.grid_output_dimensions[grid_dim];
         const OutputIndexMapRef<> map =
             params_.transform.output_index_map(output_dim);
-        const Index cell_index = grid_cell_indices_[grid_dim] =
-            FloorOfRatio(input_index * map.stride() + map.offset(), cell_size);
-        const IndexInterval cell_range =
-            IndexInterval::UncheckedSized(cell_index * cell_size, cell_size);
+        IndexInterval cell_range;
+        grid_cell_indices_[grid_dim] = params_.output_to_grid_cell(
+            grid_dim, input_index * map.stride() + map.offset(), &cell_range);
         // The check in PrePartitionIndexTransformOverRegularGrid guarantees
         // that GetAffineTransformDomain is successful.
         const IndexInterval cell_domain =
@@ -314,7 +314,7 @@ class ConnectedSetIterateHelper {
                 .value();
         restricted_domain = Intersect(restricted_domain, cell_domain);
       }
-      ABSL_ASSERT(restricted_domain.size() > 0);
+      assert(restricted_domain.size() > 0);
       // Set the input domain for `cell_input_dim` for the duration of the
       // subsequent recursive call to IterateOverStridedSets.
       cell_transform_->input_origin()[cell_input_dim] =
@@ -329,10 +329,11 @@ class ConnectedSetIterateHelper {
 
   /// Calls the iteration callback function.
   ///
-  /// If an error `Status` is returned, iteration should stop.
+  /// If an error `absl::Status` is returned, iteration should stop.
   ///
   /// \error Any error returned by the iteration callback function.
-  Status InvokeCallback() {
+  absl::Status InvokeCallback() {
+    internal_index_space::DebugCheckInvariants(cell_transform_.get());
     auto status = params_.func(
         grid_cell_indices_,
         TransformAccess::Make<IndexTransformView<>>(cell_transform_.get()));
@@ -349,8 +350,8 @@ class ConnectedSetIterateHelper {
   absl::FixedArray<Index, internal::kNumInlinedDims> grid_cell_indices_;
 
   // This stores the current value of `cell_transform[h]`, as defined in
-  // grid_partition.h, for `h = grid_cell_indices_`.  This is modified in place
-  // while iterating over all values for grid_cell_indices_.
+  // grid_partition.h, for `h = grid_cell_indices_`.  This is modified in
+  // place while iterating over all values for grid_cell_indices_.
   internal_index_space::TransformRep::Ptr<> cell_transform_;
 };
 
@@ -359,26 +360,40 @@ class ConnectedSetIterateHelper {
 
 namespace internal {
 
-Status PartitionIndexTransformOverRegularGrid(
+absl::Status PartitionIndexTransformOverGrid(
     span<const DimensionIndex> grid_output_dimensions,
-    span<const Index> grid_cell_shape, IndexTransformView<> transform,
-    FunctionView<Status(span<const Index> grid_cell_indices,
-                        IndexTransformView<> cell_transform)>
+    absl::FunctionRef<Index(DimensionIndex, Index, IndexInterval*)>
+        output_to_grid_cell,
+    IndexTransformView<> transform,
+    absl::FunctionRef<absl::Status(span<const Index> grid_cell_indices,
+                                   IndexTransformView<> cell_transform)>
         func) {
-  absl::optional<internal_grid_partition::IndexTransformGridPartition>
+  std::optional<internal_grid_partition::IndexTransformGridPartition>
       partition_info;
-  auto status =
-      internal_grid_partition::PrePartitionIndexTransformOverRegularGrid(
-          transform, grid_output_dimensions, grid_cell_shape, &partition_info);
+  auto status = internal_grid_partition::PrePartitionIndexTransformOverGrid(
+      transform, grid_output_dimensions, output_to_grid_cell, &partition_info);
+
   if (!status.ok()) return status;
   return internal_grid_partition::ConnectedSetIterateHelper(
              {/*.info=*/*partition_info,
               /*.grid_output_dimensions=*/grid_output_dimensions,
-              /*.grid_cell_shape=*/grid_cell_shape,
+              /*.output_to_grid_cell=*/output_to_grid_cell,
               /*.transform=*/transform,
               /*.func=*/std::move(func)})
       .Iterate();
 }
 
+absl::Status PartitionIndexTransformOverRegularGrid(
+    span<const DimensionIndex> grid_output_dimensions,
+    span<const Index> grid_cell_shape, IndexTransformView<> transform,
+    absl::FunctionRef<absl::Status(span<const Index> grid_cell_indices,
+                                   IndexTransformView<> cell_transform)>
+        func) {
+  assert(grid_cell_shape.size() == grid_output_dimensions.size());
+  internal_grid_partition::RegularGridRef grid{grid_cell_shape};
+  return PartitionIndexTransformOverGrid(grid_output_dimensions, grid,
+                                         transform, std::move(func));
+}
+
 }  // namespace internal
 }  // namespace tensorstore
