@@ -18,12 +18,17 @@
 
 #include <clocale>
 #include <cstddef>
+#include <limits>
 #include <memory>
+#include <optional>
 #include <string>
 #include <string_view>
 #include <utility>
 
+#include "absl/log/absl_log.h"
 #include "absl/status/status.h"
+#include "absl/strings/cord.h"
+#include "absl/strings/numbers.h"
 #include "absl/synchronization/mutex.h"
 #include "absl/time/time.h"
 #include <curl/curl.h>
@@ -32,12 +37,11 @@
 #include "tensorstore/internal/http/curl_handle.h"
 #include "tensorstore/internal/http/http_request.h"
 #include "tensorstore/internal/http/http_response.h"
-#include "tensorstore/internal/logging.h"
 #include "tensorstore/internal/metrics/counter.h"
 #include "tensorstore/internal/metrics/gauge.h"
+#include "tensorstore/internal/metrics/histogram.h"
 #include "tensorstore/internal/no_destructor.h"
 #include "tensorstore/internal/thread.h"
-#include "tensorstore/util/assert_macros.h"
 #include "tensorstore/util/future.h"
 
 namespace tensorstore {
@@ -50,18 +54,28 @@ auto& http_request_started = internal_metrics::Counter<int64_t>::New(
 auto& http_request_completed = internal_metrics::Counter<int64_t>::New(
     "/tensorstore/http/request_completed", "HTTP requests completed");
 
-auto& http_request_errors = internal_metrics::Counter<int64_t, int>::New(
-    "/tensorstore/http/request_errors", "code", "HTTP requests with errors");
-
 auto& http_request_bytes = internal_metrics::Counter<int64_t>::New(
     "/tensorstore/http/request_bytes", "HTTP request bytes transmitted");
 
+auto& http_response_codes = internal_metrics::Counter<int64_t, int>::New(
+    "/tensorstore/http/response_codes", "code",
+    "HTTP response status code counts");
+
 auto& http_response_bytes = internal_metrics::Counter<int64_t>::New(
     "/tensorstore/http/response_bytes", "HTTP response bytes received");
 
 auto& http_active = internal_metrics::Gauge<int64_t>::New(
     "/tensorstore/http/active", "HTTP requests considered active");
 
+auto& http_total_time_ms =
+    internal_metrics::Histogram<internal_metrics::DefaultBucketer>::New(
+        "/tensorstore/http/total_time_ms", "HTTP total latency (ms)");
+
+auto& http_first_byte_latency_us =
+    internal_metrics::Histogram<internal_metrics::DefaultBucketer>::New(
+        "/tensorstore/http/first_byte_latency_us",
+        "HTTP first byte received latency (us)");
+
 // Cached configuration from environment variables.
 struct CurlConfig {
   bool verbose = std::getenv("TENSORSTORE_CURL_VERBOSE") != nullptr;
@@ -254,17 +268,18 @@ struct CurlRequestState {
     if (code == CURLE_OK) {
       return absl::OkStatus();
     }
-    TENSORSTORE_LOG("Error [", code, "]=", curl_easy_strerror(code),
-                    " in curl operation\n", error_buffer_);
+    ABSL_LOG(WARNING) << "Error [" << code << "]=" << curl_easy_strerror(code)
+                      << " in curl operation\n"
+                      << error_buffer_;
     return ::tensorstore::internal_http::CurlCodeToStatus(code, error_buffer_);
   }
 
   static std::size_t CurlWriteCallback(void* contents, std::size_t size,
                                        std::size_t nmemb, void* userdata) {
     auto* self = static_cast<CurlRequestState*>(userdata);
-    http_response_bytes.IncrementBy(size);
     auto data =
         std::string_view(static_cast<char const*>(contents), size * nmemb);
+    http_response_bytes.IncrementBy(data.size());
     self->response_.payload.Append(data);
     return data.size();
   }
@@ -312,6 +327,28 @@ class MultiTransportImpl {
  public:
   explicit MultiTransportImpl(std::shared_ptr<CurlHandleFactory> factory)
       : factory_(factory), multi_(factory_->CreateMultiHandle()) {
+    // Without any option, the CURL library multiplexes up to 100 http/2 streams
+    // over a single connection. In practice there's a tradeoff between
+    // concurrent streams and latency/throughput of requests. Empirical tests
+    // suggest that using a small number of streams per connection increases
+    // throughput of large transfers, which is common in tensorstore.
+    static long max_concurrent_streams = []() -> long {
+      auto env = internal::GetEnv("TENSORSTORE_HTTP2_MAX_CONCURRENT_STREAMS");
+      if (env) {
+        uint32_t limit = 0;
+        if (absl::SimpleAtoi(*env, &limit) && limit > 0 && limit < 1000) {
+          return limit;
+        } else {
+          ABSL_LOG(WARNING)
+              << "Failed to parse TENSORSTORE_HTTP2_MAX_CONCURRENT_STREAMS: "
+              << *env;
+        }
+      }
+      return 4;  // New default streams.
+    }();
+
+    curl_multi_setopt(multi_.get(), CURLMOPT_MAX_CONCURRENT_STREAMS,
+                      max_concurrent_streams);
     thread_ = internal::Thread({"curl_handler"}, [this] { Run(); });
   }
 
@@ -383,20 +420,38 @@ void MultiTransportImpl::FinishRequest(CURL* e, CURLcode code) {
   std::unique_ptr<CurlRequestState> state(GetStatePointer(e));
 
   if (code == CURLE_HTTP2) {
-    TENSORSTORE_LOG("CURLE_HTTP2 ", state->error_buffer_);
+    ABSL_LOG(WARNING) << "CURLE_HTTP2 " << state->error_buffer_;
     // If there was an error in the HTTP2 framing, try and force
     // CURL to close the connection stream.
     // https://curl.haxx.se/libcurl/c/CURLOPT_FORBID_REUSE.html
     state->SetForbidReuse();
   }
 
+  // NOTE: Consider recording curl getinfo options:
+  // https://curl.se/libcurl/c/easy_getinfo_options.html
   http_request_completed.Increment();
+  // Record the first byte latency.
+  {
+    curl_off_t first_byte_us = 0;
+    if (CURLE_OK ==
+        curl_easy_getinfo(e, CURLINFO_STARTTRANSFER_TIME_T, &first_byte_us)) {
+      http_first_byte_latency_us.Observe(first_byte_us);
+    }
+  }
+  // Record the total time.
+  {
+    curl_off_t total_time_us = 0;
+    if (CURLE_OK ==
+        curl_easy_getinfo(e, CURLINFO_TOTAL_TIME_T, &total_time_us)) {
+      http_total_time_ms.Observe(total_time_us / 1000);
+    }
+  }
 
   if (code != CURLE_OK) {
     state->promise_.SetResult(state->CurlCodeToStatus(code));
   } else {
     state->response_.status_code = CurlGetResponseCode(e);
-    http_request_errors.Increment(state->response_.status_code);
+    http_response_codes.Increment(state->response_.status_code);
     state->promise_.SetResult(std::move(state->response_));
   }
 }
@@ -473,7 +528,7 @@ void MultiTransportImpl::Run() {
         curl_multi_poll(multi_.get(), nullptr, 0, timeout_ms, &numfds);
     if (mcode != CURLM_OK) {
       auto status = CurlMCodeToStatus(mcode, "in CurlMultiTransport");
-      TENSORSTORE_LOG("Error [", mcode, "] ", status, "\n");
+      ABSL_LOG(WARNING) << "Error [" << mcode << "] " << status;
     }
   }
 }
