@@ -27,13 +27,14 @@
 #include <utility>
 
 #include "absl/base/attributes.h"
+#include "absl/log/absl_check.h"
+#include "absl/log/absl_log.h"
 #include "absl/status/status.h"
 #include "absl/strings/str_format.h"
 #include "riegeli/bytes/reader.h"
 #include "tensorstore/data_type.h"
 #include "tensorstore/internal/image/image_info.h"
 #include "tensorstore/internal/image/image_view.h"
-#include "tensorstore/util/assert_macros.h"
 #include "tensorstore/util/span.h"
 #include "tensorstore/util/status.h"
 
@@ -67,7 +68,7 @@ tmsize_t ReadProc(thandle_t data, void* buf, tmsize_t len) {
   auto* reader = static_cast<TiffReader::Context*>(data)->reader_;
   assert(reader != nullptr);
   size_t read;
-  // TENSORSTORE_LOG("tiff read ", reader->pos(), " ", len);
+  // ABSL_LOG(INFO) << "tiff read " << reader->pos() << " " << len;
   if (!reader->Read(len, static_cast<char*>(buf), &read) && !reader->ok()) {
     errno = EBADF;
     return -1;
@@ -87,16 +88,16 @@ toff_t SeekProc(thandle_t data, toff_t pos, int whence) {
 
   switch (whence) {
     case SEEK_SET:
-      // TENSORSTORE_LOG("tiff seek ", pos);
+      // ABSL_LOG(INFO) << "tiff seek " << pos;
       reader->Seek(pos);
       break;
     case SEEK_CUR:
-      // TENSORSTORE_LOG("tiff skip ", reader->pos(), " ", pos);
+      // ABSL_LOG(INFO) << "tiff skip "<< reader->pos()<< " "<< pos;
       reader->Skip(pos);
       break;
     case SEEK_END:
       assert(pos <= 0);
-      // TENSORSTORE_LOG("tiff seek_end ", pos);
+      // ABSL_LOG(INFO) << "tiff seek_end "<< pos;
       if (auto size = reader->Size(); size) {
         reader->Seek(*size - static_cast<uint64_t>(-pos));
       } else {
@@ -231,7 +232,7 @@ absl::Status GetTIFFImageInfo(TIFF* tiff, TiffImageInfo& info) {
     samples_per_pixel = 1;
     TIFFSetField(tiff, TIFFTAG_SAMPLESPERPIXEL, samples_per_pixel);
   }
-  uint16* sample_info;
+  uint16_t* sample_info;
   if (TIFFGetField(tiff, TIFFTAG_EXTRASAMPLES, &info.extra_samples_,
                    &sample_info) != 1) {
     info.extra_samples_ = 0;
@@ -273,7 +274,7 @@ absl::Status ReadStripImpl(TIFF* tiff, TiffImageInfo& info,
   }
 
   const int strip_bytes = TIFFStripSize(tiff);
-  uint32 rows_per_strip = 1;
+  uint32_t rows_per_strip = 1;
   TIFFGetFieldDefaulted(tiff, TIFFTAG_ROWSPERSTRIP, &rows_per_strip);
 
   if (!mapping &&
@@ -292,7 +293,7 @@ absl::Status ReadStripImpl(TIFF* tiff, TiffImageInfo& info,
   }
 
   std::unique_ptr<unsigned char[]> buffer(new unsigned char[strip_bytes]);
-  uint32 line_bytes =
+  uint32_t line_bytes =
       info.width * (info.bits_per_sample_ / 8) * info.num_components;
 
   for (size_t y = 0; y < info.height; y += rows_per_strip) {
@@ -426,7 +427,7 @@ absl::Status TiffReader::Context::DefaultDecode(
     tensorstore::span<unsigned char> data) {
   TiffImageInfo info;
   TENSORSTORE_RETURN_IF_ERROR(GetTIFFImageInfo(tiff_, info));
-  TENSORSTORE_CHECK(data.size() == ImageRequiredBytes(info));
+  ABSL_CHECK_EQ(data.size(), ImageRequiredBytes(info));
 
   // Additional fields checks (beyond the info)
   uint32_t compress_tag = 0;
@@ -489,7 +490,7 @@ TiffReader::TiffReader(TiffReader&& src) = default;
 TiffReader& TiffReader::operator=(TiffReader&& src) = default;
 
 absl::Status TiffReader::Initialize(riegeli::Reader* reader) {
-  TENSORSTORE_CHECK(reader != nullptr);
+  ABSL_CHECK(reader != nullptr);
   context_ = nullptr;
 
   auto context = std::make_unique<TiffReader::Context>(reader);
