@@ -113,11 +113,12 @@
 ///
 
 #include <atomic>
+#include <cassert>
 #include <cstddef>
+#include <memory>
 #include <type_traits>
 #include <utility>
 
-#include "absl/base/macros.h"
 #include "tensorstore/internal/memory.h"
 #include "tensorstore/internal/type_traits.h"
 
@@ -147,6 +148,12 @@ class AtomicReferenceCount {
     return ref_count_.load(std::memory_order_acquire);
   }
 
+  template <typename D>
+  friend bool IncrementReferenceCountIfNonZero(
+      const AtomicReferenceCount<D>& base);
+  template <typename D>
+  friend bool DecrementReferenceCount(const AtomicReferenceCount<D>& base);
+
   /// Increments the reference count.
   ///
   /// This function is called by `DefaultIntrusivePtrTraits`.
@@ -159,7 +166,7 @@ class AtomicReferenceCount {
   ///
   /// This function is called by `DefaultIntrusivePtrTraits`.
   friend void intrusive_ptr_decrement(const AtomicReferenceCount* p) noexcept {
-    if (p->ref_count_.fetch_sub(1, std::memory_order_acq_rel) == 1) {
+    if (DecrementReferenceCount(*p)) {
       delete static_cast<const Derived*>(p);
     }
   }
@@ -168,6 +175,45 @@ class AtomicReferenceCount {
   mutable std::atomic<std::uint32_t> ref_count_{0};
 };
 
+template <typename Derived>
+inline bool IncrementReferenceCountIfNonZero(
+    const AtomicReferenceCount<Derived>& base) {
+  uint32_t count = base.ref_count_.load(std::memory_order_relaxed);
+  do {
+    if (count == 0) return false;
+  } while (!base.ref_count_.compare_exchange_weak(count, count + 1,
+                                                  std::memory_order_acq_rel));
+  return true;
+}
+
+/// Decrements the reference count of `base` and returns `true` if it reaches
+/// zero.
+template <typename Derived>
+inline bool DecrementReferenceCount(const AtomicReferenceCount<Derived>& base) {
+  return base.ref_count_.fetch_sub(1, std::memory_order_acq_rel) == 1;
+}
+
+/// Decrements `reference_count` if the result will be non-zero.
+///
+/// This is useful for caches where a simple decrement can be used to decrement
+/// the count down to any value > 0, but a mutex must be held to decrement the
+/// count to 0 in order to remove the object from the cache.
+///
+/// \pre `reference_count.load() > 0`
+/// \returns `true` if the count was successfully decremented.
+template <typename T>
+bool DecrementReferenceCountIfGreaterThanOne(std::atomic<T>& reference_count) {
+  auto count = reference_count.load(std::memory_order_relaxed);
+  while (true) {
+    if (count == 1) return false;
+    if (reference_count.compare_exchange_weak(count, count - 1,
+                                              std::memory_order_acq_rel)) {
+      // Decremented without the count reaching zero.
+      return true;
+    }
+  }
+}
+
 /// Specifies the default behavior of `IntrusivePtr<T>`.
 struct DefaultIntrusivePtrTraits {
   template <typename U>
@@ -190,9 +236,6 @@ struct DefaultIntrusivePtrTraits {
   }
 };
 
-template <typename T, typename R>
-class IntrusivePtr;
-
 /// Tag type to indicate that a new reference to a given object should be
 /// acquired.
 struct acquire_object_ref_t {
@@ -208,6 +251,9 @@ struct adopt_object_ref_t {
 constexpr acquire_object_ref_t acquire_object_ref{};
 constexpr adopt_object_ref_t adopt_object_ref{};
 
+template <typename T, typename R>
+class IntrusivePtr;
+
 template <typename T>
 struct IsIntrusivePtr : public std::false_type {};
 
@@ -270,71 +316,73 @@ class IntrusivePtr {
   using traits_type = R;
   using pointer = typename R::template pointer<T>;
 
+  ~IntrusivePtr() {
+    if (pointer p = get()) R::decrement(p);
+  }
+
   /// Constructs a null pointer.
-  constexpr IntrusivePtr() noexcept : ptr_(pointer{}) {}
+  constexpr IntrusivePtr() noexcept : ptr_(nullptr) {}
+  constexpr IntrusivePtr(std::nullptr_t) noexcept : ptr_(nullptr) {}
 
   /// Constructs from a given pointer.  If `p` is not null, acquires a new
   /// reference to `p` by calling `R::increment(p)`.
-  explicit IntrusivePtr(pointer p,
-                        acquire_object_ref_t = acquire_object_ref) noexcept
-      : ptr_(p) {
+  explicit IntrusivePtr(pointer p) noexcept : ptr_(p) {
+    if (ptr_) R::increment(ptr_);
+  }
+
+  /// Constructs from a given pointer.  If `p` is not null, acquires a new
+  /// reference to `p` by calling `R::increment(p)`.
+  explicit IntrusivePtr(pointer p, acquire_object_ref_t) noexcept : ptr_(p) {
     if (ptr_) R::increment(ptr_);
   }
 
   /// Constructs from a given pointer without acquiring a new reference.  If `p`
   /// is not null, this implicitly adopts an existing reference to `p`.
-  explicit IntrusivePtr(pointer p, adopt_object_ref_t) noexcept : ptr_(p) {}
+  constexpr explicit IntrusivePtr(pointer p, adopt_object_ref_t) noexcept
+      : ptr_(p) {}
 
   /// Copy constructs from `rhs`.  If `rhs` is not null, acquires a new
   /// reference to `rhs.get()` by calling `R::increment(rhs.get())`.
   IntrusivePtr(const IntrusivePtr& rhs) noexcept
       : IntrusivePtr(rhs.get(), acquire_object_ref) {}
 
-  template <typename U,
-            std::enable_if_t<std::is_convertible<
-                typename R::template pointer<U>, pointer>::value>* = nullptr>
-  IntrusivePtr(const IntrusivePtr<U, R>& rhs)
-      : IntrusivePtr(rhs.get(), acquire_object_ref) {}
-
-  /// Move constructs from `rhs`.  If `rhs` is not null, transfers ownership of
-  /// a reference from `rhs` to `*this`.
-  IntrusivePtr(IntrusivePtr&& rhs) noexcept
-      : IntrusivePtr(rhs.release(), adopt_object_ref) {}
-
-  template <typename U,
-            std::enable_if_t<std::is_convertible<
-                typename R::template pointer<U>, pointer>::value>* = nullptr>
-  IntrusivePtr(IntrusivePtr<U, R>&& rhs) noexcept
-      : IntrusivePtr(rhs.release(), adopt_object_ref) {}
-
-  ~IntrusivePtr() {
-    if (pointer p = get()) R::decrement(p);
-  }
-
-  IntrusivePtr& operator=(const IntrusivePtr& rhs) {
+  IntrusivePtr& operator=(const IntrusivePtr& rhs) noexcept {
     IntrusivePtr(rhs).swap(*this);
     return *this;
   }
 
-  template <typename U>
-  std::enable_if_t<
-      std::is_convertible<typename R::template pointer<U>, pointer>::value,
-      IntrusivePtr&>
-  operator=(const IntrusivePtr<U, R>& rhs) {
+  template <typename U,
+            std::enable_if_t<std::is_convertible_v<
+                typename R::template pointer<U>, pointer>>* = nullptr>
+  IntrusivePtr(const IntrusivePtr<U, R>& rhs) noexcept
+      : IntrusivePtr(rhs.get(), acquire_object_ref) {}
+
+  template <typename U, typename = std::enable_if_t<std::is_convertible_v<
+                            typename R::template pointer<U>, pointer>>>
+  IntrusivePtr& operator=(const IntrusivePtr<U, R>& rhs) noexcept {
+    IntrusivePtr(rhs).swap(*this);
+    return *this;
+  }
+
+  /// Move constructs from `rhs`.  If `rhs` is not null, transfers ownership of
+  /// a reference from `rhs` to `*this`.
+  constexpr IntrusivePtr(IntrusivePtr&& rhs) noexcept
+      : IntrusivePtr(rhs.release(), adopt_object_ref) {}
+
+  constexpr IntrusivePtr& operator=(IntrusivePtr&& rhs) noexcept {
     IntrusivePtr(std::move(rhs)).swap(*this);
     return *this;
   }
 
-  IntrusivePtr& operator=(IntrusivePtr&& rhs) noexcept {
-    IntrusivePtr(std::move(rhs)).swap(*this);
-    return *this;
-  }
+  template <typename U,
+            std::enable_if_t<std::is_convertible_v<
+                typename R::template pointer<U>, pointer>>* = nullptr>
+  constexpr IntrusivePtr(IntrusivePtr<U, R>&& rhs) noexcept
+      : IntrusivePtr(rhs.release(), adopt_object_ref) {}
 
-  template <typename U>
-  std::enable_if_t<
-      std::is_convertible<typename R::template pointer<U>, pointer>::value,
-      IntrusivePtr&>
-  operator=(IntrusivePtr<U, R>&& rhs) noexcept {
+  template <typename U, typename = std::enable_if_t<std::is_convertible_v<
+                            typename R::template pointer<U>, pointer>>>
+  constexpr IntrusivePtr& operator=(IntrusivePtr<U, R>&& rhs) noexcept {
     IntrusivePtr(std::move(rhs)).swap(*this);
     return *this;
   }
@@ -342,11 +390,13 @@ class IntrusivePtr {
   /// Assigns the stored pointer to null.  If the prior stored pointer was
   /// non-null, calls `R::decrement` on it.
   void reset() noexcept { IntrusivePtr().swap(*this); }
+  void reset(std::nullptr_t) noexcept { IntrusivePtr().swap(*this); }
 
   /// Assigns the stored pointer to `rhs`, and calls `R::increment` on `rhs` if
   /// non-null.  If the prior stored pointer was non-null, calls `R::decrement`
   /// on it.
-  void reset(pointer rhs, acquire_object_ref_t = acquire_object_ref) {
+  void reset(pointer rhs) { IntrusivePtr(rhs, acquire_object_ref).swap(*this); }
+  void reset(pointer rhs, acquire_object_ref_t) {
     IntrusivePtr(rhs, acquire_object_ref).swap(*this);
   }
 
@@ -358,32 +408,35 @@ class IntrusivePtr {
   }
 
   /// Returns `true` if the stored pointer is non-null.
-  explicit operator bool() const { return static_cast<bool>(get()); }
+  constexpr explicit operator bool() const { return static_cast<bool>(ptr_); }
 
-  pointer get() const noexcept { return ptr_; }
+  constexpr pointer get() const noexcept { return ptr_; }
 
-  pointer operator->() const {
+  constexpr pointer operator->() const {
     pointer ptr = get();
-    ABSL_ASSERT(static_cast<bool>(ptr));
+    assert(static_cast<bool>(ptr));
     return ptr;
   }
 
-  element_type& operator*() const {
+  constexpr element_type& operator*() const {
     pointer ptr = get();
-    ABSL_ASSERT(static_cast<bool>(ptr));
+    assert(static_cast<bool>(ptr));
     return *ptr;
   }
 
   /// Assigns the stored pointer to null, and returns the prior value.
   ///
   /// `R::decrement` is not called.
-  pointer release() noexcept {
+  constexpr pointer release() noexcept {
     pointer ptr = get();
     ptr_ = pointer{};
     return ptr;
   }
 
-  void swap(IntrusivePtr& rhs) noexcept { std::swap(ptr_, rhs.ptr_); }
+  void swap(IntrusivePtr& rhs) noexcept {
+    // FIXME: std::swap constexpr in C++20
+    std::swap(ptr_, rhs.ptr_);
+  }
 
   /// Abseil hash support.
   template <typename H>
@@ -411,19 +464,17 @@ inline T* to_address(const IntrusivePtr<T, R>& p) {
 }
 
 template <typename T, typename U, typename R>
-inline std::enable_if_t<
-    IsEqualityComparable<typename R::template pointer<T>,
-                         typename R::template pointer<U>>::value,
-    bool>
+inline std::enable_if_t<IsEqualityComparable<typename R::template pointer<T>,
+                                             typename R::template pointer<U>>,
+                        bool>
 operator==(const IntrusivePtr<T, R>& x, const IntrusivePtr<U, R>& y) {
   return x.get() == y.get();
 }
 
 template <typename T, typename U, typename R>
-inline std::enable_if_t<
-    IsEqualityComparable<typename R::template pointer<T>,
-                         typename R::template pointer<U>>::value,
-    bool>
+inline std::enable_if_t<IsEqualityComparable<typename R::template pointer<T>,
+                                             typename R::template pointer<U>>,
+                        bool>
 operator!=(const IntrusivePtr<T, R>& x, const IntrusivePtr<U, R>& y) {
   return x.get() != y.get();
 }
@@ -450,6 +501,38 @@ inline IntrusivePtr<T, R> dynamic_pointer_cast(IntrusivePtr<U, R> p) {
   }
 }
 
+/// Converts an `IntrusivePtr` to a `shared_ptr` that shares ownership.
+///
+/// A single reference count is owned by all copies of the returned
+/// `shared_ptr`.  When the last copy is destroyed, the intrusive reference
+/// count is decremented.
+///
+/// This requires an allocation for the `shared_ptr` control block.
+///
+/// Example:
+///   auto x = IntrusiveToShared(p);  //  'x' is a std::shared_ptr<T>
+template <typename T, typename Traits>
+std::shared_ptr<T> IntrusiveToShared(internal::IntrusivePtr<T, Traits> p) {
+  auto* ptr = p.get();
+  return std::shared_ptr<T>(
+      std::make_shared<internal::IntrusivePtr<T, Traits>>(std::move(p)), ptr);
+}
+
+/// Creates an `IntrusivePtr<T>` while avoiding issues creating temporaries
+/// during the construction process, a shorthand for
+/// `IntrusivePtr<T, R>(new T(...), acquire_object_ref)`.
+///
+/// Example:
+///   auto p = MakeIntrusivePtr<X>(args...);
+///   // 'p' is an IntrusivePtr<X, DefaultIntrusivePtrTraits>
+///
+///   auto px = MakeIntrusivePtr<X, XTraits>(5);
+///   // 'px' is an IntrusivePtr<X, XTraits>
+template <typename T, typename R = DefaultIntrusivePtrTraits, typename... Args>
+inline IntrusivePtr<T, R> MakeIntrusivePtr(Args&&... args) {
+  return IntrusivePtr<T, R>(new T(std::forward<Args>(args)...));
+}
+
 }  // namespace internal
 }  // namespace tensorstore
 
