@@ -20,6 +20,7 @@
 #include <vector>
 
 #include "absl/container/fixed_array.h"
+#include "absl/status/status.h"
 #include "tensorstore/array.h"
 #include "tensorstore/data_type.h"
 #include "tensorstore/index.h"
@@ -153,7 +154,7 @@ class IterableImpl : public NDIterable::Base<IterableImpl> {
         state_.input_byte_strides[dim_j], dir_j, size_j);
   }
 
-  DataType data_type() const override { return data_type_; }
+  DataType dtype() const override { return dtype_; }
 
   IterationBufferConstraint GetIterationBufferConstraint(
       IterationLayoutView layout) const override {
@@ -162,7 +163,7 @@ class IterableImpl : public NDIterable::Base<IterableImpl> {
                            input_dim_iter_flags::array_indexed) == 0) {
       return {(last_dim == -1 || state_.input_byte_strides[last_dim] *
                                          layout.directions[last_dim] ==
-                                     this->data_type_->size)
+                                     this->dtype_->size)
                   ? IterationBufferKind::kContiguous
                   : IterationBufferKind::kStrided,
               /*.external=*/false};
@@ -328,7 +329,8 @@ class IterableImpl : public NDIterable::Base<IterableImpl> {
     }
 
     Index GetBlock(span<const Index> indices, Index block_size,
-                   IterationBufferPointer* pointer, Status* status) override {
+                   IterationBufferPointer* pointer,
+                   absl::Status* status) override {
       IterationBufferPointer block_pointer = pointer_;
       // Add the contribution from the direct array byte strides (corresponding
       // to `single_input_dimension` output index maps).
@@ -375,11 +377,12 @@ class IterableImpl : public NDIterable::Base<IterableImpl> {
           } else {
             // Index array depends on the last iteration dimension.
             // Incorporate it into `offsets_array`.
+            Index block_start = indices.back();
             for (Index i = 0; i < block_size; ++i) {
               const Index cur_contribution = wrap_on_overflow::Multiply(
                   output_byte_stride,
                   index_array_pointer[wrap_on_overflow::Multiply(
-                      i, last_index_array_byte_stride)]);
+                      i + block_start, last_index_array_byte_stride)]);
               offsets_array[i] =
                   wrap_on_overflow::Add(offsets_array[i], cur_contribution);
             }
@@ -398,10 +401,12 @@ class IterableImpl : public NDIterable::Base<IterableImpl> {
     std::vector<Index, ArenaAllocator<Index>> buffer_;
   };
 
+  /// Maintains ownership of the array data.
+  std::shared_ptr<const void> data_owner_;
   IndexTransform<> transform_;
   // TODO(jbms): Use arena allocator for SingleArrayIterationState as well.
   internal_index_space::SingleArrayIterationState state_;
-  DataType data_type_;
+  DataType dtype_;
   std::vector<input_dim_iter_flags::Bitmask,
               ArenaAllocator<input_dim_iter_flags::Bitmask>>
       input_dimension_flags_;
@@ -411,8 +416,11 @@ Result<NDIterable::Ptr> MaybeConvertToArrayNDIterable(
     std::unique_ptr<IterableImpl, VirtualDestroyDeleter> impl, Arena* arena) {
   if (impl->state_.num_array_indexed_output_dimensions == 0) {
     return GetArrayNDIterable(
-        OffsetArrayView<const void>(
-            ElementPointer<void>(impl->state_.base_pointer, impl->data_type_),
+        SharedOffsetArrayView<const void>(
+            SharedElementPointer<const void>(
+                std::shared_ptr<const void>(std::move(impl->data_owner_),
+                                            impl->state_.base_pointer),
+                impl->dtype_),
             StridedLayoutView<>(impl->transform_.input_rank(),
                                 impl->transform_.input_shape().data(),
                                 impl->state_.input_byte_strides.data())),
@@ -424,34 +432,35 @@ Result<NDIterable::Ptr> MaybeConvertToArrayNDIterable(
 }  // namespace
 
 Result<NDIterable::Ptr> GetTransformedArrayNDIterable(
-    TransformedArrayView<const void> array, Arena* arena) {
-  if (!array.has_transform()) {
-    return GetArrayNDIterable(array.base_array(), arena);
+    SharedOffsetArrayView<const void> array, IndexTransformView<> transform,
+    Arena* arena) {
+  if (!transform.valid()) {
+    return GetArrayNDIterable(array, arena);
   }
 
   auto impl = MakeUniqueWithVirtualIntrusiveAllocator<IterableImpl>(
-      ArenaAllocator<>(arena), array.transform());
+      ArenaAllocator<>(arena), transform);
   TENSORSTORE_RETURN_IF_ERROR(InitializeSingleArrayIterationState(
-      array.base_array(),
-      internal_index_space::TransformAccess::rep(impl->transform_),
-      impl->transform_.input_origin().data(),
-      impl->transform_.input_shape().data(), &impl->state_,
-      impl->input_dimension_flags_.data()));
-  impl->data_type_ = array.data_type();
+      array, internal_index_space::TransformAccess::rep(transform),
+      transform.input_origin().data(), transform.input_shape().data(),
+      &impl->state_, impl->input_dimension_flags_.data()));
+  impl->dtype_ = array.dtype();
+  impl->data_owner_ = std::move(array.element_pointer().pointer());
   return MaybeConvertToArrayNDIterable(std::move(impl), arena);
 }
 
-Result<NDIterable::Ptr> GetNormalizedTransformedArrayNDIterable(
-    NormalizedTransformedArray<const void> array, Arena* arena) {
+Result<NDIterable::Ptr> GetTransformedArrayNDIterable(
+    TransformedArray<Shared<const void>> array, Arena* arena) {
   auto impl = MakeUniqueWithVirtualIntrusiveAllocator<IterableImpl>(
       ArenaAllocator<>(arena), std::move(array.transform()));
   TENSORSTORE_RETURN_IF_ERROR(InitializeSingleArrayIterationState(
-      array.element_pointer(),
+      ElementPointer<const void>(array.element_pointer()),
       internal_index_space::TransformAccess::rep(impl->transform_),
       impl->transform_.input_origin().data(),
       impl->transform_.input_shape().data(), &impl->state_,
       impl->input_dimension_flags_.data()));
-  impl->data_type_ = array.data_type();
+  impl->dtype_ = array.dtype();
+  impl->data_owner_ = std::move(array.element_pointer().pointer());
   return MaybeConvertToArrayNDIterable(std::move(impl), arena);
 }
 
