@@ -20,16 +20,14 @@
 #include <utility>
 
 #include "absl/status/status.h"
-#include "absl/strings/str_cat.h"
 #include "absl/time/time.h"
-#include "tensorstore/internal/env.h"
-#include "tensorstore/internal/http/curl_transport.h"
 #include "tensorstore/internal/http/http_request.h"
 #include "tensorstore/internal/http/http_response.h"
 #include "tensorstore/internal/json_binding/json_binding.h"
 #include "tensorstore/internal/path.h"
 #include "tensorstore/util/result.h"
 #include "tensorstore/util/status.h"
+#include "tensorstore/util/str_cat.h"
 
 namespace tensorstore {
 namespace internal_oauth2 {
@@ -45,9 +43,9 @@ std::string MakePayload(const internal_oauth2::RefreshToken& creds) {
   auto client_id = internal::PercentEncodeUriComponent(creds.client_id);
   auto client_secret = internal::PercentEncodeUriComponent(creds.client_secret);
   auto refresh_token = internal::PercentEncodeUriComponent(creds.refresh_token);
-  return absl::StrCat("grant_type=refresh_token", "&client_id=", client_id,
-                      "&client_secret=", client_secret,
-                      "&refresh_token=", refresh_token);
+  return tensorstore::StrCat(
+      "grant_type=refresh_token", "&client_id=", client_id,
+      "&client_secret=", client_secret, "&refresh_token=", refresh_token);
 }
 
 }  // namespace
