@@ -114,9 +114,9 @@ absl::StatusCode GetOsErrorStatusCode(OsErrorCode error) {
 absl::Status StatusFromOsError(OsErrorCode error_code, std::string_view a,
                                std::string_view b, std::string_view c,
                                std::string_view d) {
-  return absl::Status(
-      GetOsErrorStatusCode(error_code),
-      StrCat(a, b, c, d, " [OS error: ", GetOsErrorMessage(error_code), "]"));
+  return absl::Status(GetOsErrorStatusCode(error_code),
+                      tensorstore::StrCat(a, b, c, d, " [OS error: ",
+                                          GetOsErrorMessage(error_code), "]"));
 }
 
 }  // namespace internal
