@@ -14,21 +14,29 @@
 
 #include "tensorstore/internal/test_util.h"
 
-#include <cstdint>
-#include <cstdio>
+#include <functional>
 #include <iterator>
+#include <optional>
+#include <random>
 #include <string>
 #include <string_view>
+#include <utility>
 
 #include <gtest/gtest.h>
+#include "absl/log/absl_check.h"
+#include "absl/log/absl_log.h"
 #include "absl/random/random.h"
+#include "absl/status/status.h"
 #include "absl/strings/escaping.h"
 #include "absl/strings/numbers.h"
 #include "tensorstore/internal/env.h"
-#include "tensorstore/internal/logging.h"
 #include "tensorstore/internal/os_error_code.h"
 #include "tensorstore/internal/path.h"
+#include "tensorstore/internal/source_location.h"
+#include "tensorstore/kvstore/file/file_util.h"
+#include "tensorstore/util/result.h"
 #include "tensorstore/util/status.h"
+#include "tensorstore/util/str_cat.h"
 
 #if defined(_WIN32)
 #define TENSORSTORE_USE_STD_FILESYSTEM 1
@@ -66,7 +74,7 @@ namespace {
 std::string TemporaryDirectoryPath() {
   std::error_code ec;
   auto base_dir = std::filesystem::temp_directory_path(ec);
-  TENSORSTORE_CHECK(!ec);
+  ABSL_CHECK(!ec);
   return base_dir.generic_string();
 }
 
@@ -188,17 +196,27 @@ ScopedTemporaryDirectory::ScopedTemporaryDirectory() {
     x = kAlphabet[absl::Uniform(gen, 0u, std::size(kAlphabet) - 1)];
   }
 
-  std::string basename =
-      StrCat("tmp_tensorstore_test_", std::string_view(data, std::size(data)));
+  std::string basename = tensorstore::StrCat(
+      "tmp_tensorstore_test_", std::string_view(data, std::size(data)));
   path_ = tensorstore::internal::JoinPath(TemporaryDirectoryPath(), basename);
 
-  TENSORSTORE_CHECK(MakeDirectory(path_));
+  ABSL_CHECK(MakeDirectory(path_));
 }
 
 ScopedTemporaryDirectory::~ScopedTemporaryDirectory() {
   TENSORSTORE_CHECK_OK(RemoveAll(path_));
 }
 
+ScopedCurrentWorkingDirectory::ScopedCurrentWorkingDirectory(
+    const std::string& new_cwd) {
+  TENSORSTORE_CHECK_OK_AND_ASSIGN(old_cwd_, internal_file_util::GetCwd());
+  TENSORSTORE_CHECK_OK(internal_file_util::SetCwd(new_cwd));
+}
+
+ScopedCurrentWorkingDirectory::~ScopedCurrentWorkingDirectory() {
+  TENSORSTORE_CHECK_OK(internal_file_util::SetCwd(old_cwd_));
+}
+
 void RegisterGoogleTestCaseDynamically(std::string test_suite_name,
                                        std::string test_name,
                                        std::function<void()> test_func,
@@ -224,13 +242,14 @@ unsigned int GetRandomSeedForTest(const char* env_var) {
   unsigned int seed;
   if (auto env_seed = internal::GetEnv(env_var)) {
     if (absl::SimpleAtoi(*env_seed, &seed)) {
-      TENSORSTORE_LOG("Using deterministic random seed ", env_var, "=", seed);
+      ABSL_LOG(INFO) << "Using deterministic random seed " << env_var << "="
+                     << seed;
       return seed;
     }
   }
   seed = std::random_device()();
-  TENSORSTORE_LOG("Define environment variable ", env_var, "=", seed,
-                  " for deterministic seeding");
+  ABSL_LOG(INFO) << "Define environment variable " << env_var << "=" << seed
+                 << " for deterministic seeding";
   return seed;
 }
 
