@@ -19,6 +19,7 @@
 #include <utility>
 
 #include "absl/base/thread_annotations.h"
+#include "absl/log/absl_check.h"
 #include "absl/synchronization/mutex.h"
 #include "absl/time/clock.h"
 #include "absl/time/time.h"
@@ -26,7 +27,6 @@
 #include "tensorstore/internal/mutex.h"
 #include "tensorstore/internal/no_destructor.h"
 #include "tensorstore/internal/thread.h"
-#include "tensorstore/util/assert_macros.h"
 #include "tensorstore/util/executor.h"
 
 namespace tensorstore {
@@ -257,7 +257,7 @@ void ManagedTaskQueue::TaskDone() {
 }  // namespace
 
 Executor DetachedThreadPool(std::size_t num_threads) {
-  TENSORSTORE_CHECK(num_threads > 0);
+  ABSL_CHECK_GT(num_threads, 0);
   static internal::NoDestructor<SharedThreadPool> pool_;
   intrusive_ptr_increment(pool_.get());
   IntrusivePtr<ManagedTaskQueue> managed_task_queue(new ManagedTaskQueue(
