@@ -22,7 +22,9 @@
 
 #include "absl/container/inlined_vector.h"
 #include "tensorstore/internal/os_error_code.h"
+#include "tensorstore/kvstore/file/file_util.h"
 #include "tensorstore/kvstore/file/potentially_blocking_region.h"
+#include "tensorstore/util/quote_string.h"
 
 // Most modern unix allow 1024 iovs.
 #if defined(UIO_MAXIOV)
@@ -184,6 +186,32 @@ bool DirectoryIterator::Make(Entry entry,
   return true;
 }
 
+Result<std::string> GetCwd() {
+  std::string buf;
+  buf.resize(256);
+  while (true) {
+    if (::getcwd(buf.data(), buf.size()) != nullptr) {
+      buf.resize(std::strlen(buf.data()));
+      return buf;
+    }
+    if (errno == ERANGE) {
+      buf.resize(buf.size() * 2);
+      continue;
+    }
+    return internal::StatusFromOsError(
+        errno, "Failed to determine current working directory");
+  }
+}
+
+absl::Status SetCwd(const std::string& path) {
+  if (::chdir(path.c_str()) != 0) {
+    return internal::StatusFromOsError(
+        errno, "Failed to set current working directory to: ",
+        tensorstore::QuoteString(path));
+  }
+  return absl::OkStatus();
+}
+
 }  // namespace internal_file_util
 }  // namespace tensorstore
 
