@@ -22,20 +22,20 @@
 #include <utility>
 #include <variant>
 
+#include "absl/log/absl_log.h"
 #include "absl/strings/match.h"
 #include "absl/strings/numbers.h"
-#include "absl/strings/str_cat.h"
 #include "absl/strings/str_split.h"
 #include "absl/strings/substitute.h"
 #include "absl/synchronization/mutex.h"
 #include "tensorstore/internal/http/curl_handle.h"
 #include "tensorstore/internal/http/http_request.h"
 #include "tensorstore/internal/http/http_response.h"
-#include "tensorstore/internal/logging.h"
 #include "tensorstore/internal/path.h"
 #include "tensorstore/kvstore/test_util.h"
 #include "tensorstore/util/executor.h"
 #include "tensorstore/util/status.h"
+#include "tensorstore/util/str_cat.h"
 
 namespace tensorstore {
 namespace {
@@ -88,9 +88,9 @@ GCSMockStorageBucket::GCSMockStorageBucket(
     std::optional<std::string> requestor_pays_project_id)
     : bucket_(bucket),
       bucket_prefix_(
-          absl::StrCat("storage.googleapis.com/storage/v1/b/", bucket)),
-      upload_prefix_(
-          absl::StrCat("storage.googleapis.com/upload/storage/v1/b/", bucket)),
+          tensorstore::StrCat("storage.googleapis.com/storage/v1/b/", bucket)),
+      upload_prefix_(tensorstore::StrCat(
+          "storage.googleapis.com/upload/storage/v1/b/", bucket)),
       requestor_pays_project_id_(std::move(requestor_pays_project_id)) {}
 
 // Responds to a "www.google.apis/storage/v1/b/bucket" request.
@@ -106,7 +106,8 @@ Future<HttpResponse> GCSMockStorageBucket::IssueRequest(
   } else if (std::holds_alternative<HttpResponse>(match_result)) {
     return std::move(std::get<HttpResponse>(match_result));
   }
-  return absl::UnimplementedError("Mock cannot satisfy the request.");
+  return absl::UnimplementedError(
+      tensorstore::StrCat("Mock cannot satisfy the request: ", request.url()));
 }
 
 std::variant<std::monostate, HttpResponse, absl::Status>
@@ -303,7 +304,7 @@ GCSMockStorageBucket::HandleInsertRequest(std::string_view path,
     obj.generation = ++next_generation_;
     obj.data = payload;
 
-    TENSORSTORE_LOG("Uploaded: ", obj.name, " ", obj.generation);
+    ABSL_LOG(INFO) << "Uploaded: " << obj.name << " " << obj.generation;
 
     return ObjectMetadataResponse(obj);
   } while (false);
@@ -394,7 +395,7 @@ GCSMockStorageBucket::HandleDeleteRequest(std::string_view path,
       }
     }
 
-    TENSORSTORE_LOG("Deleted: ", name, " ", it->second.generation);
+    ABSL_LOG(INFO) << "Deleted: " << name << " " << it->second.generation;
 
     data_.erase(it);
     return HttpResponse{204, absl::Cord()};
@@ -408,7 +409,7 @@ HttpResponse GCSMockStorageBucket::ObjectMetadataResponse(
   std::string data = ObjectMetadata(object).dump();
   HttpResponse response{200, absl::Cord(std::move(data))};
   response.headers.insert(
-      {"content-length", absl::StrCat(response.payload.size())});
+      {"content-length", tensorstore::StrCat(response.payload.size())});
   response.headers.insert({"content-type", "application/json"});
   return response;
 }
@@ -416,35 +417,37 @@ HttpResponse GCSMockStorageBucket::ObjectMetadataResponse(
 ::nlohmann::json GCSMockStorageBucket::ObjectMetadata(const Object& object) {
   return {
       {"kind", "storage#object"},
-      {"id", absl::StrCat(bucket_, "/", object.name, "/", object.generation)},
+      {"id",
+       tensorstore::StrCat(bucket_, "/", object.name, "/", object.generation)},
       {"selfLink",
-       absl::StrCat("https://www.googleapis.com/storage/v1/b/", bucket_, "/o/",
-                    internal::PercentEncodeUriComponent(object.name))},
+       tensorstore::StrCat("https://www.googleapis.com/storage/v1/b/", bucket_,
+                           "/o/",
+                           internal::PercentEncodeUriComponent(object.name))},
       {"name", object.name},
       {"bucket", bucket_},
-      {"generation", absl::StrCat(object.generation)},
+      {"generation", tensorstore::StrCat(object.generation)},
       {"metageneration", "1"},
       {"contentType", "application/octet-stream"},
       {"timeCreated", "2018-10-24T00:41:38.264Z"},
       {"updated", "2018-10-24T00:41:38.264Z"},
       {"storageClass", "MULTI_REGIONAL"},
       {"timeStorageClassUpdated", "2018-10-24T00:41:38.264Z"},
-      {"size", absl::StrCat(object.data.size())},
+      {"size", tensorstore::StrCat(object.data.size())},
       {"mediaLink",
-       absl::StrCat("https://www.googleapis.com/download/storage/v1/b/",
-                    bucket_, "/o/",
-                    internal::PercentEncodeUriComponent(object.name),
-                    "?generation=", object.generation, "&alt=media")},
+       tensorstore::StrCat("https://www.googleapis.com/download/storage/v1/b/",
+                           bucket_, "/o/",
+                           internal::PercentEncodeUriComponent(object.name),
+                           "?generation=", object.generation, "&alt=media")},
   };
 }
 
 HttpResponse GCSMockStorageBucket::ObjectMediaResponse(const Object& object) {
   HttpResponse response{200, object.data};
   response.headers.insert(
-      {"content-length", absl::StrCat(response.payload.size())});
+      {"content-length", tensorstore::StrCat(response.payload.size())});
   response.headers.insert({"content-type", "application/octet-stream"});
   response.headers.insert(
-      {"x-goog-generation", absl::StrCat(object.generation)});
+      {"x-goog-generation", tensorstore::StrCat(object.generation)});
   response.headers.insert({"x-goog-metageneration", "1"});
   response.headers.insert({"x-goog-storage-class", "MULTI_REGIONAL"});
   // todo: x-goog-hash
