@@ -12,6 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+#include "tensorstore/kvstore/driver.h"
+
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
 #include "absl/time/time.h"
@@ -27,6 +29,7 @@
 
 namespace {
 namespace kvstore = ::tensorstore::kvstore;
+
 using ::tensorstore::Future;
 using ::tensorstore::MatchesStatus;
 using ::tensorstore::Promise;
@@ -418,12 +421,11 @@ TEST(SpecTest, SpecRoundtrip) {
   tensorstore::internal::KeyValueStoreSpecRoundtripOptions options;
   options.check_write_read = false;
   options.check_data_persists = false;
-  tensorstore::internal::TestKeyValueStoreSpecRoundtrip(
-      {{"driver", "http"},
-       {"base_url", "https://example.com?query"},
-       {"headers", {"a: b"}},
-       {"path", "/abc"}},
-      options);
+  options.full_spec = {{"driver", "http"},
+                       {"base_url", "https://example.com?query"},
+                       {"headers", {"a: b"}},
+                       {"path", "/abc"}};
+  tensorstore::internal::TestKeyValueStoreSpecRoundtrip(options);
 }
 
 TEST(SpecTest, NormalizeSpecRelativePath) {
