@@ -25,6 +25,8 @@
 #include "tensorstore/context.h"
 #include "tensorstore/internal/cache_key/cache_key.h"
 #include "tensorstore/internal/json_gtest.h"
+#include "tensorstore/kvstore/driver.h"
+#include "tensorstore/kvstore/key_range.h"
 #include "tensorstore/kvstore/test_util.h"
 #include "tensorstore/serialization/serialization.h"
 #include "tensorstore/serialization/test_util.h"
@@ -32,7 +34,6 @@
 #include "tensorstore/util/execution/sender.h"
 #include "tensorstore/util/execution/sender_testutil.h"
 #include "tensorstore/util/future.h"
-#include "tensorstore/util/status.h"
 #include "tensorstore/util/status_testutil.h"
 
 namespace {
@@ -201,27 +202,28 @@ TEST(MemoryKeyValueStoreTest, Open) {
 }
 
 TEST(MemoryKeyValueStoreTest, SpecRoundtrip) {
-  tensorstore::internal::TestKeyValueStoreSpecRoundtrip({
+  tensorstore::internal::KeyValueStoreSpecRoundtripOptions options;
+  options.full_spec = {
       {"driver", "memory"},
-  });
+  };
+  tensorstore::internal::TestKeyValueStoreSpecRoundtrip(options);
 }
 
 TEST(MemoryKeyValueStoreTest, SpecRoundtripWithContextSpec) {
   tensorstore::internal::KeyValueStoreSpecRoundtripOptions options;
   options.spec_request_options.Set(tensorstore::unbind_context);
+  options.full_spec = {
+      {"driver", "memory"},
+      {"memory_key_value_store", "memory_key_value_store#a"},
+      {"context",
+       {
+           {"memory_key_value_store#a", ::nlohmann::json::object_t()},
+       }},
+  };
   // Since spec includes context resources, if we re-open we get a different
   // context resource.
   options.check_data_persists = false;
-  tensorstore::internal::TestKeyValueStoreSpecRoundtrip(
-      {
-          {"driver", "memory"},
-          {"memory_key_value_store", "memory_key_value_store#a"},
-          {"context",
-           {
-               {"memory_key_value_store#a", ::nlohmann::json::object_t()},
-           }},
-      },
-      options);
+  tensorstore::internal::TestKeyValueStoreSpecRoundtrip(options);
 }
 
 TEST(MemoryKeyValueStoreTest, InvalidSpec) {
