@@ -21,13 +21,13 @@
 #include "absl/algorithm/container.h"
 #include "absl/base/internal/endian.h"
 #include "absl/status/status.h"
+#include "absl/synchronization/mutex.h"
 #include "tensorstore/internal/cache/async_cache.h"
 #include "tensorstore/internal/cache/cache_pool_resource.h"
 #include "tensorstore/internal/cache/kvs_backed_cache.h"
-#include "tensorstore/internal/compression/zlib.h"
 #include "tensorstore/internal/data_copy_concurrency_resource.h"
 #include "tensorstore/internal/estimate_heap_usage/estimate_heap_usage.h"
-#include "tensorstore/internal/estimate_heap_usage/std_vector.h"
+#include "tensorstore/internal/estimate_heap_usage/std_vector.h"  // iwyu: keep
 #include "tensorstore/internal/json_binding/bindable.h"
 #include "tensorstore/internal/json_binding/json_binding.h"
 #include "tensorstore/internal/mutex.h"
@@ -178,7 +178,7 @@ class MinishardIndexKeyValueStore : public kvstore::Driver {
                            absl::Status error) {
         promise.SetResult(MaybeAnnotateStatus(
             ConvertInvalidArgumentToFailedPrecondition(std::move(error)),
-            StrCat("Error retrieving shard index entry")));
+            "Error retrieving shard index entry"));
       }
 
       void operator()(Promise<kvstore::ReadResult> promise,
