@@ -14,9 +14,11 @@
 
 #include "tensorstore/transaction.h"
 
+#include "absl/base/optimization.h"
 #include "absl/functional/function_ref.h"
 #include "tensorstore/serialization/serialization.h"
 #include "tensorstore/transaction_impl.h"
+#include "tensorstore/util/str_cat.h"
 
 namespace tensorstore {
 namespace internal {
@@ -388,7 +390,7 @@ absl::Status TransactionState::Node::Register() {
     case kAbortRequested:
       return GetCancelledError();
     default:
-      TENSORSTORE_UNREACHABLE;
+      ABSL_UNREACHABLE();  // COV_NF_LINE
   }
   if (phase_ == kInvalidPhase) {
     // Duplicate `Transaction::phase()` logic rather than call it, since we are
@@ -420,7 +422,7 @@ TransactionState::GetOrCreateMultiPhaseNode(
     case kAbortRequested:
       return GetCancelledError();
     default:
-      TENSORSTORE_UNREACHABLE;
+      ABSL_UNREACHABLE();  // COV_NF_LINE
   }
   return OpenTransactionNodePtr<Node>(
       nodes_
