@@ -34,6 +34,7 @@
 #include "tensorstore/util/executor.h"
 #include "tensorstore/util/future_impl.h"  // IWYU pragma: export
 #include "tensorstore/util/result.h"
+#include "tensorstore/util/span.h"
 
 namespace tensorstore {
 
@@ -459,9 +460,9 @@ class AnyFuture {
   /// (when a value is present) or a copy of result.status().
   ///
   /// \dchecks `!null()`
-  absl::Status status() const {
+  const absl::Status& status() const& noexcept TENSORSTORE_LIFETIME_BOUND {
     Wait();
-    return rep().GetStatusCopy();
+    return rep().status();
   }
 
   /// Executes `callback` with the signature `void (AnyFuture)` when this
@@ -817,7 +818,7 @@ class Future : public AnyFuture {
     return result().value();
   }
 
-  /// Returns a copy of `result().status()`
+  /// Equivalent to `result().status()`.
   using AnyFuture::status;
 
  private:
@@ -942,6 +943,32 @@ constexpr inline bool IsCallbackInvocableWithReadyFutures =
      std::is_invocable_v<Callback, PromiseType,
                          internal_future::ReadyFutureType<Futures>...>);
 
+inline size_t FutureReferenceCount(const AnyFuture& future) {
+  auto* ptr = FutureAccess::rep_pointer(future).get();
+  return ptr ? ptr->future_reference_count_.load(std::memory_order_acquire) : 0;
+}
+
+inline size_t PromiseReferenceCount(const AnyFuture& future) {
+  auto* ptr = FutureAccess::rep_pointer(future).get();
+  return ptr ? ptr->promise_reference_count_.load(std::memory_order_acquire)
+             : 0;
+}
+
+template <typename T>
+size_t FutureReferenceCount(const Promise<T>& promise) {
+  auto* ptr = FutureAccess::rep_pointer(promise).get();
+  return ptr ? ptr->future_reference_count_.load(std::memory_order_acquire) : 0;
+}
+
+template <typename T>
+size_t PromiseReferenceCount(const Promise<T>& promise) {
+  auto* ptr = FutureAccess::rep_pointer(promise).get();
+  return ptr ? ptr->promise_reference_count_.load(std::memory_order_acquire)
+             : 0;
+}
+
+constexpr size_t kFutureReferencesPerLink = 2;
+
 }  // namespace internal_future
 
 /// Creates a "link", which ties a `promise` to one or more `future` objects and
@@ -1392,7 +1419,6 @@ MapFutureError(Executor&& executor, Func func, Future<T> future) {
                    std::move(future));
 }
 
-
 /// If `promise` does not already have a result set, sets its result to `result`
 /// and sets `promise.result_needed() = false`.
 ///
