@@ -26,14 +26,15 @@
 #include <utility>
 
 #include "absl/base/attributes.h"
+#include "absl/base/optimization.h"
 #include "absl/synchronization/mutex.h"
 #include "absl/time/time.h"
 #include "absl/utility/utility.h"
 #include "tensorstore/internal/intrusive_ptr.h"
 #include "tensorstore/internal/tagged_ptr.h"
+#include "tensorstore/internal/tracing/tracing.h"
 #include "tensorstore/internal/type_traits.h"
 #include "tensorstore/util/result.h"
-#include "tensorstore/util/status.h"
 
 namespace tensorstore {
 
@@ -177,7 +178,7 @@ class FutureStateBase {
   virtual ~FutureStateBase();
 
   virtual bool has_value() = 0;
-  virtual absl::Status GetStatusCopy() = 0;
+  virtual const absl::Status& status() const& noexcept = 0;
 
   /// Registers a ready callback.
   ///
@@ -486,11 +487,7 @@ class FutureState : public FutureStateBase {
   ~FutureState() override {}
 
   bool has_value() final { return result.has_value(); };
-  absl::Status GetStatusCopy() final {
-    // FIXME: This should be a feature of Result<T>
-    if (result.has_value()) return absl::OkStatus();
-    return result.status();
-  }
+  const absl::Status& status() const& noexcept final { return result.status(); }
 
   Result<T> result;
 };
@@ -528,7 +525,9 @@ class CallbackBase : public CallbackListNode {
       kLinkCallback = 3;
 
   explicit CallbackBase(SharedStatePointer shared_state)
-      : shared_state_(shared_state), reference_count_(2) {}
+      : shared_state_(shared_state),
+        reference_count_(2),
+        trace_context_(internal_tracing::TraceContext::kThread) {}
 
   virtual ~CallbackBase();
 
@@ -576,6 +575,11 @@ class CallbackBase : public CallbackListNode {
   /// function object itself (contained in a derived class) may be destroyed
   /// before this object itself is destroyed.
   std::atomic<std::size_t> reference_count_;
+
+  /// Tracing context for the callback, initialized when the callback is
+  /// created.
+  TENSORSTORE_ATTRIBUTE_NO_UNIQUE_ADDRESS internal_tracing::TraceContext
+      trace_context_;
 };
 
 struct CallbackPointerTraits {
@@ -669,9 +673,11 @@ class ReadyCallback final : public ReadyCallbackBase {
       : ReadyCallbackBase(state), callback_(std::forward<U>(u)) {}
 
   void OnReady() noexcept override {
+    internal_tracing::SwapCurrentTraceContext(&trace_context_);
     std::move(callback_)(
         FutureAccess::Construct<ReadyType>(TakeStatePointer()));
     callback_.~Callback();
+    internal_tracing::SwapCurrentTraceContext(&trace_context_);
   }
 
   void OnUnregistered() noexcept override {
@@ -703,9 +709,11 @@ class ForceCallback final : public ForceCallbackBase {
         callback_(std::forward<U>(u)) {}
 
   void OnForced() noexcept override {
+    internal_tracing::SwapCurrentTraceContext(&trace_context_);
     std::move(callback_)(
         FutureAccess::Construct<Promise<T>>(TakeStatePointer()));
     callback_.~Callback();
+    internal_tracing::SwapCurrentTraceContext(&trace_context_);
   }
 
   void OnUnregistered() noexcept override {
@@ -735,8 +743,10 @@ struct ResultNotNeededCallback final : public ResultNotNeededCallbackBase {
       : ResultNotNeededCallbackBase(state), callback_(std::forward<U>(u)) {}
 
   void OnResultNotNeeded() noexcept override {
+    internal_tracing::SwapCurrentTraceContext(&trace_context_);
     std::move(callback_)();
     callback_.~Callback();
+    internal_tracing::SwapCurrentTraceContext(&trace_context_);
   }
   void OnUnregistered() noexcept override { callback_.~Callback(); }
   void DestroyCallback() noexcept override { delete this; }
@@ -929,7 +939,7 @@ struct FutureLinkPropagateFirstErrorPolicy {
   static bool OnFutureReady(FutureStateBase* future_state,
                             FutureState<PromiseValue>* promise_state) {
     if (future_state->has_value()) return true;
-    promise_state->SetResult(future_state->GetStatusCopy());
+    promise_state->SetResult(future_state->status());
     return false;
   }
 };
@@ -1427,7 +1437,7 @@ CallbackPointer MakeLink(Callback&& callback, Promise<PromiseValue> promise,
       return link->GetCallbackPointer();
     }
   }
-  TENSORSTORE_UNREACHABLE;  // COV_NF_LINE
+  ABSL_UNREACHABLE();  // COV_NF_LINE
 }
 
 /// Overload for the case of zero futures.
