@@ -15,14 +15,14 @@
 #include "tensorstore/util/quote_string.h"
 
 #include <string>
+#include <string_view>
 
 #include "absl/strings/escaping.h"
-#include "absl/strings/string_view.h"
 
 namespace tensorstore {
 
-std::string QuoteString(absl::string_view s) {
-  return '"' + absl::CHexEscape(s) + '"';
+std::string QuoteString(std::string_view s) {
+  return '"' + absl::CHexEscape(absl::string_view(s.data(), s.size())) + '"';
 }
 
 }  // namespace tensorstore
