@@ -20,13 +20,13 @@
 #include <utility>
 
 #include "absl/base/optimization.h"
+#include "absl/log/absl_check.h"
+#include "absl/log/absl_log.h"
 #include "absl/status/status.h"
 #include "tensorstore/internal/attributes.h"
-#include "tensorstore/internal/logging.h"
 #include "tensorstore/internal/preprocessor/cat.h"
 #include "tensorstore/internal/preprocessor/expand.h"
 #include "tensorstore/internal/type_traits.h"
-#include "tensorstore/util/assert_macros.h"
 #include "tensorstore/util/result_impl.h"  // IWYU pragma: export
 #include "tensorstore/util/status.h"
 
@@ -91,12 +91,14 @@ using PipelineResultType =
 using std::in_place;
 using std::in_place_t;
 
-/// `Result<T>` implements a value-or-error concept using the existing
-/// absl::Status mechanism. It provides a discriminated union of a usable value,
-/// `T`, or an error `absl::Status` explaining why the value is not present.
+/// `Result<T>` implements a value-or-error concept providing a discriminated
+/// union of a usable value, `T`, or an `absl::Status` error describing why the
+/// value is not present. `Result<void>` is permitted, which becomes a wrapper
+/// for `absl::Status`, and which enables use of `Result<T>` in a wide variety
+/// of template contexts.
 ///
-/// The primary use case for Result<T> is as the return value of a
-/// function which might fail.
+/// The primary use case for `Result<T>` is as the return value of a
+/// function which may fail.
 ///
 /// Initialization with a non-error `absl::Status` is only allowed for
 /// `Result<void>`, otherwise non-error `absl::Status` initilization is
@@ -105,15 +107,15 @@ using std::in_place_t;
 /// Conversion from `Result<T>` to `Result<void>` is allowed; the status
 /// is retained but any value is discarded by such a conversion.
 ///
-/// There are quite a few similar classes extant:
-///
 /// Assignment operators always destroy the existing value and reconstruct the
 /// value. This may be surprising, since it is unlike `std::optional` and many
 /// other monadic C++ types.
 ///
-/// - The StatusOr concept used by Google protocol buffers.
+/// There are quite a few similar classes extant:
 ///
-/// - An `std::variant<T, absl::Status>`, except that it allows ``T=void``.
+/// - The `absl::StatusOr<T>` concept used by Google.
+///
+/// - An `std::variant<T, absl::Status>`.
 ///
 /// - The proposed ``std::expected<>``, except that the error type is not
 ///   template-selectable.
@@ -142,18 +144,17 @@ class Result : private internal_result::ResultStorage<T>,
                 "T must not be cv-qualified");
 
   using Base = internal_result::ResultStorage<T>;
+  using Base::status_;
 
  public:
   /// The type of the contained success value.
   using value_type = T;
 
   /// Equal to `T&`, or `void` if `T` is void.
-  using reference_type =
-      typename internal_result::ResultStorage<T>::reference_type;
+  using reference_type = T&;
 
   /// Equal to `const T&`, or `void` if `T` is `void`.
-  using const_reference_type =
-      typename internal_result::ResultStorage<T>::const_reference_type;
+  using const_reference_type = const T&;
 
   /// Always equal to `absl::Status`.
   using error_type = absl::Status;
@@ -166,9 +167,8 @@ class Result : private internal_result::ResultStorage<T>,
   /// Constructs an error Result with a code of `absl::StatusCode::kUnknown`.
   ///
   /// \id default
-  explicit Result() : Base(internal_result::noinit_t{}) {
-    this->construct_status(absl::UnknownError(""));
-  }
+  explicit Result()
+      : Base(internal_result::status_t{}, absl::StatusCode::kUnknown, "") {}
 
   /// Constructs from an existing result.
   ///
@@ -188,25 +188,13 @@ class Result : private internal_result::ResultStorage<T>,
   ///
   /// \pre `!status` unless `T` is `void`.
   /// \id status
-  Result(const absl::Status& status) : Base(internal_result::noinit_t{}) {
-    if constexpr (std::is_void_v<value_type>) {
-      if (status.ok()) {
-        this->construct_value();
-        return;
-      }
-    }
-    TENSORSTORE_CHECK(!status.ok());
-    this->construct_status(status);
+  Result(const absl::Status& status)
+      : Base(internal_result::status_t{}, status) {
+    ABSL_CHECK(!status_.ok());
   }
-  Result(absl::Status&& status) : Base(internal_result::noinit_t{}) {
-    if constexpr (std::is_void_v<value_type>) {
-      if (status.ok()) {
-        this->construct_value();
-        return;
-      }
-    }
-    TENSORSTORE_CHECK(!status.ok());
-    this->construct_status(status);
+  Result(absl::Status&& status)
+      : Base(internal_result::status_t{}, std::move(status)) {
+    ABSL_CHECK(!status_.ok());
   }
 
   /// Assigns from a status object.
@@ -214,24 +202,12 @@ class Result : private internal_result::ResultStorage<T>,
   /// \pre `!status` unless `T` is `void`.
   /// \id status
   Result& operator=(const absl::Status& status) {
-    if constexpr (std::is_void_v<value_type>) {
-      if (status.ok()) {
-        this->emplace_value();
-        return *this;
-      }
-    }
-    TENSORSTORE_CHECK(!status.ok());
+    ABSL_CHECK(!status.ok());
     this->assign_status(status);
     return *this;
   }
   Result& operator=(absl::Status&& status) {
-    if constexpr (std::is_void_v<value_type>) {
-      if (status.ok()) {
-        this->emplace_value();
-        return *this;
-      }
-    }
-    TENSORSTORE_CHECK(!status.ok());
+    ABSL_CHECK(!status.ok());
     this->assign_status(std::move(status));
     return *this;
   }
@@ -263,7 +239,7 @@ class Result : private internal_result::ResultStorage<T>,
                std::add_const_t<std::add_lvalue_reference_t<U>>, T> &&        //
            !internal_result::is_constructible_convertible_from<T, Result<U>>  //
            )>* = nullptr>
-  Result(const Result<U>& rhs) : Base(internal_result::noinit_t{}) {
+  Result(const Result<U>& rhs) {
     construct_from(rhs);
   }
   template <
@@ -274,7 +250,7 @@ class Result : private internal_result::ResultStorage<T>,
            std::is_convertible_v<std::add_rvalue_reference_t<U>, T> &&        //
            !internal_result::is_constructible_convertible_from<T, Result<U>>  //
            )>* = nullptr>
-  Result(Result<U>&& rhs) : Base(internal_result::noinit_t{}) {
+  Result(Result<U>&& rhs) {
     construct_from(std::move(rhs));
   }
 
@@ -289,7 +265,7 @@ class Result : private internal_result::ResultStorage<T>,
                std::add_const_t<std::add_lvalue_reference_t<U>>, T> &&        //
            !internal_result::is_constructible_convertible_from<T, Result<U>>  //
            )>* = nullptr>
-  explicit Result(const Result<U>& rhs) : Base(internal_result::noinit_t{}) {
+  explicit Result(const Result<U>& rhs) {
     construct_from(rhs);
   }
 
@@ -302,7 +278,7 @@ class Result : private internal_result::ResultStorage<T>,
            !std::is_convertible_v<std::add_rvalue_reference_t<U>, T> &&       //
            !internal_result::is_constructible_convertible_from<T, Result<U>>  //
            )>* = nullptr>
-  explicit Result(Result<U>&& rhs) : Base(internal_result::noinit_t{}) {
+  explicit Result(Result<U>&& rhs) {
     construct_from(std::move(rhs));
   }
 
@@ -336,53 +312,6 @@ class Result : private internal_result::ResultStorage<T>,
     return *this;
   }
 
-  // Constructs a Result<void> from a Result<U>.
-  template <typename V,  //
-            std::enable_if_t<internal_result::result_void_conversion<T, V>>* =
-                nullptr>
-  Result(const Result<V>& rhs) : Base(internal_result::noinit_t{}) {
-    if (rhs.has_value()) {
-      this->construct_value();
-    } else {
-      this->construct_status(rhs.status());
-    }
-  }
-  template <typename V,  //
-            std::enable_if_t<internal_result::result_void_conversion<T, V>>* =
-                nullptr>
-  Result(Result<V>&& rhs) : Base(internal_result::noinit_t{}) {
-    if (rhs.has_value()) {
-      this->construct_value();
-    } else {
-      this->construct_status(std::move(rhs).status());
-    }
-  }
-
-  // Conversion to Result<void> assignment.
-  template <typename V,  //
-            std::enable_if_t<internal_result::result_void_conversion<T, V>>* =
-                nullptr>
-  Result& operator=(const Result<V>& rhs) {
-    if (rhs.has_value()) {
-      this->emplace_value();
-    } else {
-      this->assign_status(rhs.status());
-    }
-    return *this;
-  }
-
-  template <typename V,  //
-            std::enable_if_t<internal_result::result_void_conversion<T, V>>* =
-                nullptr>
-  Result& operator=(Result<V>&& rhs) {
-    if (rhs.has_value()) {
-      this->emplace_value();
-    } else {
-      this->assign_status(std::move(rhs).status());
-    }
-    return *this;
-  }
-
   /// Forwarding constructors.
 
   /// Directly constructs the contained value from the specified arguments.
@@ -449,15 +378,14 @@ class Result : private internal_result::ResultStorage<T>,
   ///   opt.emplace(arg1,arg2,arg3);  // Constructs Foo(arg1,arg2,arg3)
   ///
   template <typename... Args>
-  reference_type emplace(Args&&... args) {
+  T& emplace(Args&&... args) TENSORSTORE_LIFETIME_BOUND {
     static_assert(sizeof...(Args) == 0 || !std::is_void_v<T>);
     this->emplace_value(std::forward<Args>(args)...);
-    if constexpr (!std::is_void_v<value_type>) {
-      return this->value_;
-    }
+    return this->value_;
   }
   template <typename U, typename... Args>
-  reference_type emplace(std::initializer_list<U> il, Args&&... args) {
+  T& emplace(std::initializer_list<U> il,
+             Args&&... args) TENSORSTORE_LIFETIME_BOUND {
     this->emplace_value(il, std::forward<Args>(args)...);
     return this->value_;
   }
@@ -473,71 +401,68 @@ class Result : private internal_result::ResultStorage<T>,
   ///
   /// `value()` is valid only iff `has_value()` is `true`. `status()` is valid
   /// iff `has_value()` is false.
-  constexpr bool ok() const { return has_value(); }
-  constexpr bool has_value() const { return this->has_value_; }
-  explicit constexpr operator bool() const noexcept { return has_value(); }
+  bool ok() const { return status_.ok(); }
+  bool has_value() const { return status_.ok(); }
+  explicit operator bool() const noexcept { return status_.ok(); }
 
   /// Checked value accessor.
   ///
   /// Terminates the process if `*this` represents a failure state.
   ///
   /// \pre `has_value() == true`
-  const_reference_type value() const& noexcept TENSORSTORE_LIFETIME_BOUND {
+  const T& value() const& noexcept TENSORSTORE_LIFETIME_BOUND {
     if (!has_value()) TENSORSTORE_CHECK_OK(status());
-    if constexpr (!std::is_void_v<value_type>) {
-      return this->value_;
-    }
+    return this->value_;
   }
-  reference_type value() & noexcept TENSORSTORE_LIFETIME_BOUND {
+  T& value() & noexcept TENSORSTORE_LIFETIME_BOUND {
     if (!has_value()) TENSORSTORE_CHECK_OK(status());
-    if constexpr (!std::is_void_v<value_type>) {
-      return this->value_;
-    }
+    return this->value_;
   }
-  value_type value() && noexcept {
+  const T&& value() const&& noexcept TENSORSTORE_LIFETIME_BOUND {
     if (!has_value()) TENSORSTORE_CHECK_OK(status());
-    if constexpr (!std::is_void_v<value_type>) {
-      return std::move(this->value_);
-    }
+    return std::move(this->value_);
+  }
+  T&& value() && noexcept TENSORSTORE_LIFETIME_BOUND {
+    if (!has_value()) TENSORSTORE_CHECK_OK(status());
+    return std::move(this->value_);
   }
 
   /// Returns the error status.
   const absl::Status& status() const& noexcept TENSORSTORE_LIFETIME_BOUND {
-    TENSORSTORE_CHECK(!has_value());
-    return this->status_;
+    return status_;
   }
-  absl::Status status() && {
-    TENSORSTORE_CHECK(!has_value());
-    return std::move(this->status_);
+  absl::Status status() && noexcept {
+    // Note: This relies on the fact that the moved-from `absl::Status` does not
+    // have a status code of `absl::StatusCode::kOk`.
+    return status_.ok() ? absl::OkStatus() : std::move(status_);
   }
 
   /// Returns a pointer to the contained value.
   ///
   /// \pre has_value() == true
-  template <typename U = T>
-  constexpr const U* operator->() const noexcept {
+  constexpr const T* operator->() const noexcept TENSORSTORE_LIFETIME_BOUND {
     assert_has_value();
     return &this->value_;
   }
-  template <typename U = T>
-  constexpr U* operator->() noexcept {
+  constexpr T* operator->() noexcept TENSORSTORE_LIFETIME_BOUND {
     assert_has_value();
     return &this->value_;
   }
 
   /// Returns a reference to the contained value.
-  template <typename U = T>
-  constexpr const U& operator*() const& noexcept {
+  constexpr const T& operator*() const& noexcept TENSORSTORE_LIFETIME_BOUND {
     assert_has_value();
     return this->value_;
   }
-  template <typename U = T>
-  constexpr U& operator*() & noexcept {
+  constexpr T& operator*() & noexcept TENSORSTORE_LIFETIME_BOUND {
     assert_has_value();
     return this->value_;
   }
-  template <typename U = T>
-  constexpr U&& operator*() && noexcept {
+  constexpr const T&& operator*() const&& noexcept TENSORSTORE_LIFETIME_BOUND {
+    assert_has_value();
+    return std::move(this->value_);
+  }
+  constexpr T&& operator*() && noexcept TENSORSTORE_LIFETIME_BOUND {
     assert_has_value();
     return std::move(this->value_);
   }
@@ -555,13 +480,13 @@ class Result : private internal_result::ResultStorage<T>,
   ///    return !x.ok() ? x.status() : Result<U>(y(x.value()))
   ///
   template <typename Func>
-  inline FlatResult<std::invoke_result_t<Func&&, reference_type>>  //
+  inline FlatResult<std::invoke_result_t<Func&&, T&>>  //
   operator|(Func&& func) const& {
     if (!ok()) return status();
     return static_cast<Func&&>(func)(value());
   }
   template <typename Func>
-  inline FlatResult<std::invoke_result_t<Func&&, T>>  //
+  inline FlatResult<std::invoke_result_t<Func&&, T&&>>  //
   operator|(Func&& func) && {
     if (!ok()) return status();
     return static_cast<Func&&>(func)(value());
@@ -657,7 +582,7 @@ class Result : private internal_result::ResultStorage<T>,
     if (other.has_value()) {
       this->construct_value(std::forward<Other>(other).value());
     } else {
-      this->construct_status(std::forward<Other>(other).status());
+      status_ = std::forward<Other>(other).status();
     }
   }
 
@@ -676,12 +601,86 @@ class Result : private internal_result::ResultStorage<T>,
   inline void assert_has_value() const {
 #if !defined(NDEBUG)
     if (!has_value()) {
-      TENSORSTORE_LOG_FATAL("assert_has_value: ", this->status_);
+      ABSL_LOG(FATAL) << "assert_has_value: " << this->status_;
     }
 #endif
   }
 };
 
+// Specialization of `Result` for `void`.  This just wraps a plain
+// `absl::Status` to provide the same API as `Result`.
+template <>
+class Result<void> {
+ public:
+  using value_type = void;
+  using reference_type = void;
+  using const_reference_type = void;
+  using error_type = absl::Status;
+
+  template <typename U>
+  using rebind = Result<U>;
+
+  explicit Result() : status_(absl::Status(absl::StatusCode::kUnknown, "")) {}
+
+  Result(const absl::Status& status) : status_(status) {}
+  Result(absl::Status&& status) : status_(std::move(status)) {}
+
+  template <typename T>
+  Result(const Result<T>& other) : status_(other.status()) {}
+
+  template <typename T>
+  Result(Result<T>&& other) : status_(std::move(other).status()) {}
+
+  Result(std::in_place_t) {}
+
+  Result& operator=(const absl::Status& status) {
+    status_ = status;
+    return *this;
+  }
+
+  Result& operator=(absl::Status&& status) {
+    status_ = std::move(status);
+    return *this;
+  }
+
+  template <typename T>
+  Result& operator=(const Result<T>& other) {
+    status_ = other.status();
+    return *this;
+  }
+
+  template <typename T>
+  Result& operator=(Result<T>&& other) {
+    status_ = std::move(other).status();
+    return *this;
+  }
+
+  void emplace() { status_ = absl::OkStatus(); }
+
+  void IgnoreResult() const {}
+
+  bool ok() const { return status_.ok(); }
+  bool has_value() const { return status_.ok(); }
+  explicit operator bool() const noexcept { return status_.ok(); }
+
+  void value() const { TENSORSTORE_CHECK_OK(status()); }
+
+  const absl::Status& status() const& noexcept TENSORSTORE_LIFETIME_BOUND {
+    return status_;
+  }
+
+  absl::Status&& status() && { return std::move(status_); }
+
+  friend bool operator==(const Result& a, const Result& b) {
+    return a.status_ == b.status_;
+  }
+
+  friend bool operator!=(const Result& a, const Result& b) { return !(a == b); }
+
+ private:
+  absl::Status status_;
+};
+
 /// Returns a Result<T> with a (possibly-default) value.
 ///
 /// Example:
@@ -726,11 +725,11 @@ inline Result<U> MakeResult(absl::Status status) {
 /// \id result
 template <typename T>
 inline absl::Status GetStatus(const Result<T>& result) {
-  return result.has_value() ? absl::Status() : result.status();
+  return result.status();
 }
 template <typename T>
 inline absl::Status GetStatus(Result<T>&& result) {
-  return result.has_value() ? absl::Status() : std::move(result).status();
+  return std::move(result).status();
 }
 
 /// UnwrapResult returns the value contained by the Result<T> instance,
